@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -50,11 +50,6 @@ def _moduleRepr(module):
             )
 
         file_desc = "file " + _normalizePath(module_file).replace(".pyc", ".py")
-
-
-        if file_desc.endswith("not_present.py"):
-            raise AttributeError
-
     except AttributeError:
         file_desc = "built-in"
 
@@ -66,12 +61,14 @@ def _moduleRepr(module):
 def enableImportTracing(normalize_paths = True, show_source = False):
 
     def _ourimport(name, globals = None, locals = None, fromlist = None,  # @ReservedAssignment
-                   level = -1):
+                   level = -1 if sys.version_info[0] < 2 else 0):
+        builtins.__import__ = original_import
+
         global _indentation
         try:
             _indentation += 1
 
-            print(_indentation * " " + "called with: name=%r level=%d" % (name, level))
+            print(_indentation * " " + "called with: name=%r level=%d fromlist=%s" % (name, level, fromlist))
 
             for entry in traceback.extract_stack()[:-1]:
                 if entry[2] == "_ourimport":
@@ -90,8 +87,20 @@ def enableImportTracing(normalize_paths = True, show_source = False):
 
             print(_indentation * " " + "*" * 40)
 
-            result = original_import(name, globals, locals, fromlist, level)
+            builtins.__import__ = _ourimport
+            try:
+                result = original_import(name, globals, locals, fromlist, level)
+            except ImportError as e:
+                print(_indentation * " " + "EXCEPTION:", e)
+                raise
+            finally:
+                builtins.__import__ = original_import
+
             print(_indentation * " " + "RESULT:", _moduleRepr(result))
+            print(_indentation * " " + "*" * 40)
+
+            builtins.__import__ = _ourimport
+
             return result
         finally:
             _indentation -= 1
