@@ -1,5 +1,5 @@
 #!/bin/sh
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,8 +21,8 @@ cd `dirname $0`/..
 
 find nuitka -name \*.py -a \! -path *inline_copy*
 find bin -name \*.py
-find nuitka/build/static_src -name \*.cpp
-find nuitka/build/include -name \*.hpp
+find nuitka/build/static_src -name \*.c
+find nuitka/build/include/nuitka -name \*.h
 find nuitka/build/ -name \*.scons
 find misc -name \*.sh
 find bin -name \*.sh
