@@ -32,7 +32,7 @@ import marshal
 import re
 import struct
 import sys
-from logging import warning
+from logging import info, warning
 
 from nuitka import Options
 from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
@@ -81,9 +81,14 @@ def generateConstantTrueReferenceCode(to_name, expression, emit, context):
 
     # No context or other knowledge needed, pylint: disable=unused-argument
 
-    emit(
-        "%s = Py_True;" % to_name
-    )
+    if to_name.c_type == "nuitka_bool":
+        emit(
+            "%s = NUITKA_BOOL_TRUE;" % to_name
+        )
+    else:
+        emit(
+            "%s = Py_True;" % to_name
+        )
 
 
 def generateConstantFalseReferenceCode(to_name, expression, emit, context):
@@ -91,9 +96,14 @@ def generateConstantFalseReferenceCode(to_name, expression, emit, context):
 
     # No context or other knowledge needed, pylint: disable=unused-argument
 
-    emit(
-        "%s = Py_False;" % to_name
-    )
+    if to_name.c_type == "nuitka_bool":
+        emit(
+            "%s = NUITKA_BOOL_FALSE;" % to_name
+        )
+    else:
+        emit(
+            "%s = Py_False;" % to_name
+        )
 
 
 def generateConstantEllipsisReferenceCode(to_name, expression, emit, context):
@@ -240,7 +250,14 @@ def isMarshalConstant(constant_value):
     if getConstantWeight(constant_value) < 20:
         return False
 
-    marshal_value = marshal.dumps(constant_value)
+    try:
+        marshal_value = marshal.dumps(constant_value)
+    except ValueError:
+        if Options.isDebug():
+            warning("Failed to marshal constant %r." % constant_value)
+
+        return False
+
     restored = marshal.loads(marshal_value)
 
     r = compareConstants(constant_value, restored)
@@ -456,13 +473,23 @@ CHECK_OBJECT( const_int_pos_1 );
                     )
                 )
             else:
-                emit(
-                    "%s = UNSTREAM_STRING( %s, %d );" % (
-                        constant_identifier,
-                        stream_data.getStreamDataCode(encoded),
-                        1 if _isAttributeName(constant_value) else 0
+                if str is not bytes and \
+                   len(constant_value) == len(encoded):
+                    emit(
+                        "%s = UNSTREAM_STRING_ASCII( %s, %d );" % (
+                            constant_identifier,
+                            stream_data.getStreamDataCode(encoded),
+                            1 if _isAttributeName(constant_value) else 0
+                        )
+                    )
+                else:
+                    emit(
+                        "%s = UNSTREAM_STRING( %s, %d );" % (
+                            constant_identifier,
+                            stream_data.getStreamDataCode(encoded),
+                            1 if _isAttributeName(constant_value) else 0
+                        )
                     )
-                )
 
             return
         except UnicodeEncodeError:
@@ -944,6 +971,9 @@ def getConstantAccess(to_name, constant, emit, context):
     # Many cases, because for each type, we may copy or optimize by creating
     # empty.  pylint: disable=too-many-branches,too-many-statements
 
+    if to_name.c_type == "nuitka_bool" and Options.isDebug():
+        info("Missing optimization for constant to C bool.")
+
     if type(constant) is dict:
         if constant:
             for key, value in iterItems(constant):
@@ -1013,15 +1043,29 @@ def getConstantAccess(to_name, constant, emit, context):
 
         ref_count = 0
 
+    if to_name.c_type == "PyObject *":
+        value_name = to_name
+    else:
+        value_name = context.allocateTempName("constant_value")
+
     emit(
         "%s = %s;" % (
-            to_name,
+            value_name,
             code,
         )
     )
 
     if ref_count:
-        context.addCleanupTempName(to_name)
+        context.addCleanupTempName(value_name)
+
+    if to_name is not value_name:
+        to_name.getCType().emitAssignConversionCode(
+            to_name     = to_name,
+            value_name  = value_name,
+            needs_check = False,
+            emit        = emit,
+            context     = context
+        )
 
 
 def getModuleConstantCode(constant):
