@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,30 +21,33 @@
 
 from .CodeHelpers import (
     generateChildExpressionsCode,
-    generateStatementSequenceCode
+    generateStatementSequenceCode,
+    withObjectCodeTemporaryAssignment
 )
 from .Emission import SourceCodeCollector
-from .ErrorCodes import getErrorExitCode, getReleaseCode
+from .ErrorCodes import getErrorExitCode
 from .FunctionCodes import (
     finalizeFunctionLocalVariables,
+    getClosureCopyCode,
+    getFunctionQualnameObj,
     setupFunctionLocalVariables
 )
-from .GeneratorCodes import getClosureCopyCode
 from .Indentation import indented
 from .LineNumberCodes import emitLineNumberUpdateCode
-from .PythonAPICodes import getReferenceExportCode
+from .ModuleCodes import getModuleAccessCode
 from .templates.CodeTemplatesCoroutines import (
     template_coroutine_exception_exit,
     template_coroutine_noexception_exit,
     template_coroutine_object_body_template,
-    template_coroutine_object_decl_template,
+    template_coroutine_object_maker_template,
     template_coroutine_return_exit,
-    template_make_coroutine_template
+    template_make_coroutine
 )
+from .YieldCodes import getYieldReturnDispatchCode
 
 
 def getCoroutineObjectDeclCode(function_identifier):
-    return template_coroutine_object_decl_template % {
+    return template_coroutine_object_maker_template % {
         "function_identifier" : function_identifier,
     }
 
@@ -53,7 +56,9 @@ def getCoroutineObjectCode(context, function_identifier, closure_variables,
                            user_variables, outline_variables,
                            temp_variables, needs_exception_exit,
                            needs_generator_return):
-    function_locals, function_cleanup = setupFunctionLocalVariables(
+    # A bit of details going on here, pylint: disable=too-many-locals
+
+    setupFunctionLocalVariables(
         context           = context,
         parameters        = None,
         closure_variables = closure_variables,
@@ -63,19 +68,27 @@ def getCoroutineObjectCode(context, function_identifier, closure_variables,
 
     function_codes = SourceCodeCollector()
 
+    coroutine_object_body = context.getOwner()
+
     generateStatementSequenceCode(
-        statement_sequence = context.getOwner().getBody(),
+        statement_sequence = coroutine_object_body.getBody(),
         allow_none         = True,
         emit               = function_codes,
         context            = context
     )
 
-    finalizeFunctionLocalVariables(context, function_locals, function_cleanup)
+    function_cleanup = finalizeFunctionLocalVariables(context)
 
     if needs_exception_exit:
+        exception_type, exception_value, exception_tb, _exception_lineno = \
+          context.variable_storage.getExceptionVariableDescriptions()
+
         generator_exit = template_coroutine_exception_exit % {
             "function_identifier" : function_identifier,
-            "function_cleanup"    : indented(function_cleanup)
+            "function_cleanup"    : indented(function_cleanup),
+            "exception_type"      : exception_type,
+            "exception_value"     : exception_value,
+            "exception_tb"        : exception_tb
         }
     else:
         generator_exit = template_coroutine_noexception_exit % {
@@ -84,13 +97,41 @@ def getCoroutineObjectCode(context, function_identifier, closure_variables,
         }
 
     if needs_generator_return:
-        generator_exit += template_coroutine_return_exit % {}
+        generator_exit += template_coroutine_return_exit % {
+            "return_value" : context.getReturnValueName()
+        }
+
+    function_locals = context.variable_storage.makeCFunctionLevelDeclarations()
+
+    local_type_decl = context.variable_storage.makeCStructLevelDeclarations()
+    function_locals += context.variable_storage.makeCStructInits()
+
+    if local_type_decl:
+        heap_declaration = """\
+struct %(function_identifier)s_locals *coroutine_heap = \
+(struct %(function_identifier)s_locals *)coroutine->m_heap_storage;""" % {
+            "function_identifier" : function_identifier
+        }
+    else:
+        heap_declaration = ""
 
     return template_coroutine_object_body_template % {
-        "function_identifier" : function_identifier,
-        "function_body"       : indented(function_codes.codes),
-        "function_var_inits"  : indented(function_locals),
-        "coroutine_exit"      : generator_exit
+        "function_identifier"    : function_identifier,
+        "function_body"          : indented(function_codes.codes),
+        "heap_declaration"       : indented(heap_declaration),
+        "function_local_types"   : indented(local_type_decl),
+        "function_var_inits"     : indented(function_locals),
+        "function_dispatch"      : indented(getYieldReturnDispatchCode(context)),
+        "coroutine_exit"         : generator_exit,
+        "coroutine_module"       : getModuleAccessCode(context),
+        "coroutine_name_obj"     : context.getConstantCode(
+            constant = coroutine_object_body.getFunctionName()
+        ),
+        "coroutine_qualname_obj" : getFunctionQualnameObj(coroutine_object_body, context),
+        "code_identifier"        : context.getCodeObjectHandle(
+            code_object = coroutine_object_body.getCodeObject(),
+        ),
+        "closure_count"          : len(closure_variables)
     }
 
 
@@ -107,14 +148,10 @@ def generateMakeCoroutineObjectCode(to_name, expression, emit, context):
     )
 
     emit(
-        template_make_coroutine_template % {
-            "closure_copy"         : indented(closure_copy, 0, True),
-            "coroutine_identifier" : coroutine_object_body.getCodeName(),
+        template_make_coroutine % {
             "to_name"              : to_name,
-            "code_identifier"      : context.getCodeObjectHandle(
-                code_object = expression.getCodeObject(),
-            ),
-            "closure_count"        : len(closure_variables)
+            "coroutine_identifier" : coroutine_object_body.getCodeName(),
+            "closure_copy"         : indented(closure_copy, 0, True),
         }
     )
 
@@ -130,41 +167,29 @@ def generateAsyncWaitCode(to_name, expression, emit, context):
         context    = context
     )
 
-    # In handlers, we must preserve/restore the exception.
-    preserve_exception = expression.isExceptionPreserving()
+    if expression.isExpressionAsyncWaitEnter():
+        wait_kind = "await_enter"
+    elif expression.isExpressionAsyncWaitExit():
+        wait_kind = "await_exit"
+    else:
+        wait_kind = "await_normal"
 
-    context_identifier = context.getContextObjectName()
-
-    # This produces AWAIT_COROUTINE or AWAIT_ASYNCGEN calls.
-    getReferenceExportCode(value_name, emit, context)
 
     emit(
-        "%s = %s_%s( %s, %s );" % (
+        "%s = ASYNC_AWAIT( %s, %s );" % (
             to_name,
-            context_identifier.upper(),
-            "AWAIT"
-              if not preserve_exception else
-            "AWAIT_IN_HANDLER",
-            context_identifier,
-            value_name
+            value_name,
+            wait_kind
         )
     )
 
-    if not context.needsCleanup(value_name):
-        context.addCleanupTempName(value_name)
-
-    getReleaseCode(
+    getErrorExitCode(
+        check_name   = to_name,
         release_name = value_name,
         emit         = emit,
         context      = context
     )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -175,28 +200,24 @@ def generateAsyncIterCode(to_name, expression, emit, context):
         context    = context
     )
 
-    emit(
-        "%s = %s_ASYNC_MAKE_ITERATOR( %s, %s );" % (
-            to_name,
-            context.getContextObjectName().upper(),
-            context.getContextObjectName(),
-            value_name
+    with withObjectCodeTemporaryAssignment(to_name, "aiter_result", expression, emit, context) \
+      as result_name:
+
+        emit(
+            "%s = ASYNC_MAKE_ITERATOR( %s );" % (
+                result_name,
+                value_name
+            )
         )
-    )
 
-    getReleaseCode(
-        release_name = value_name,
-        emit         = emit,
-        context      = context
-    )
+        getErrorExitCode(
+            check_name   = result_name,
+            release_name = value_name,
+            emit         = emit,
+            context      = context
+        )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
-    context.addCleanupTempName(to_name)
+        context.addCleanupTempName(result_name)
 
 
 def generateAsyncNextCode(to_name, expression, emit, context):
@@ -206,26 +227,21 @@ def generateAsyncNextCode(to_name, expression, emit, context):
         context    = context
     )
 
-    emit(
-        "%s = %s_ASYNC_ITERATOR_NEXT( %s, %s );" % (
-            to_name,
-            context.getContextObjectName().upper(),
-            context.getContextObjectName(),
-            value_name,
+    with withObjectCodeTemporaryAssignment(to_name, "anext_result", expression, emit, context) \
+      as result_name:
+
+        emit(
+            "%s = ASYNC_ITERATOR_NEXT( %s );" % (
+                result_name,
+                value_name
+            )
         )
-    )
 
-    getReleaseCode(
-        release_name = value_name,
-        emit         = emit,
-        context      = context
-    )
+        getErrorExitCode(
+            check_name   = result_name,
+            release_name = value_name,
+            emit         = emit,
+            context      = context
+        )
 
-    getErrorExitCode(
-        check_name      = to_name,
-        quick_exception = "StopAsyncIteration",
-        emit            = emit,
-        context         = context
-    )
-
-    context.addCleanupTempName(to_name)
+        context.addCleanupTempName(result_name)
