@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,20 +18,21 @@
 """ Eval/exec/execfile/compile built-in related codes. """
 
 from nuitka import Options
+from nuitka.nodes.shapes.BuiltinTypeShapes import ShapeTypeDict
 from nuitka.PythonVersions import python_version
 
-from .CodeHelpers import generateExpressionCode
-from .ErrorCodes import (
-    getErrorExitBoolCode,
-    getErrorExitCode,
-    getReleaseCode,
-    getReleaseCodes
+from .CodeHelpers import (
+    generateExpressionCode,
+    withObjectCodeTemporaryAssignment
 )
+from .ConstantCodes import getConstantAccess
+from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode, getReleaseCode
+from .PythonAPICodes import getReferenceExportCode
 from .VariableCodes import getVariableAssignmentCode
 
 
-def getStoreLocalsCode(locals_name, variables, is_foreign, emit, context):
-    for variable, version in variables:
+def _getStoreLocalsCode(locals_name, variable_traces, is_dict, emit, context):
+    for variable, variable_trace in sorted(variable_traces, key = lambda x: x[0].getName()):
         if not variable.isModuleVariable():
             key_name = context.getConstantCode(
                 constant = variable.getName()
@@ -39,7 +40,15 @@ def getStoreLocalsCode(locals_name, variables, is_foreign, emit, context):
 
             value_name = context.allocateTempName("locals_value", unique = True)
 
-            if is_foreign:
+            if is_dict:
+                emit(
+                    "%s = PyDict_GetItem( %s, %s );" % (
+                       value_name,
+                       locals_name,
+                       key_name,
+                    )
+                )
+            else:
                 emit(
                    "%s = PyObject_GetItem( %s, %s );" % (
                        value_name,
@@ -50,7 +59,7 @@ def getStoreLocalsCode(locals_name, variables, is_foreign, emit, context):
 
                 getErrorExitBoolCode(
                     condition = """\
-    %s == NULL && !EXCEPTION_MATCH_BOOL_SINGLE( GET_ERROR_OCCURRED(), PyExc_KeyError )""" % value_name,
+%s == NULL && !EXCEPTION_MATCH_BOOL_SINGLE( GET_ERROR_OCCURRED(), PyExc_KeyError )""" % value_name,
                     emit      = emit,
                     context   = context
                 )
@@ -58,35 +67,24 @@ def getStoreLocalsCode(locals_name, variables, is_foreign, emit, context):
                 emit("CLEAR_ERROR_OCCURRED();")
 
                 context.addCleanupTempName(value_name)
-            else:
-                emit(
-                    "%s = PyDict_GetItem( %s, %s );" % (
-                       value_name,
-                       locals_name,
-                       key_name,
-                    )
-                )
 
             emit("if ( %s != NULL )" % value_name)
             emit('{')
 
             getVariableAssignmentCode(
-                variable      = variable,
-                version       = version,
-                tmp_name      = value_name,
-                needs_release = None, # TODO: Could be known maybe.
-                in_place      = False,
-                emit          = emit,
-                context       = context
+                variable       = variable,
+                variable_trace = variable_trace,
+                tmp_name       = value_name,
+                needs_release  = None, # TODO: Could be known maybe.
+                in_place       = False,
+                emit           = emit,
+                context        = context
             )
 
             emit('}')
 
 
-
-
 def generateBuiltinCompileCode(to_name, expression, emit, context):
-
     source_name = context.allocateTempName("compile_source")
     filename_name = context.allocateTempName("compile_filename")
     mode_name = context.allocateTempName("compile_mode")
@@ -150,20 +148,23 @@ def generateBuiltinCompileCode(to_name, expression, emit, context):
         expression.getCompatibleSourceReference()
     )
 
-    getBuiltinCompileCode(
-        to_name           = to_name,
-        source_name       = source_name,
-        filename_name     = filename_name,
-        mode_name         = mode_name,
-        flags_name        = flags_name,
-        dont_inherit_name = dont_inherit_name,
-        optimize_name     = optimize_name,
-        emit              = emit,
-        context           = context
-    )
+    with withObjectCodeTemporaryAssignment(to_name, "compile_result", expression, emit, context) \
+      as value_name:
+
+        _getBuiltinCompileCode(
+            to_name           = value_name,
+            source_name       = source_name,
+            filename_name     = filename_name,
+            mode_name         = mode_name,
+            flags_name        = flags_name,
+            dont_inherit_name = dont_inherit_name,
+            optimize_name     = optimize_name,
+            emit              = emit,
+            context           = context
+        )
 
 
-def getBuiltinCompileCode(to_name, source_name, filename_name, mode_name,
+def _getBuiltinCompileCode(to_name, source_name, filename_name, mode_name,
                           flags_name, dont_inherit_name, optimize_name, emit,
                           context):
     if python_version < 300:
@@ -187,23 +188,28 @@ def getBuiltinCompileCode(to_name, source_name, filename_name, mode_name,
     emit(
         "%s = COMPILE_CODE( %s );" % (
             to_name,
-            ", ".join(args)
+            ", ".join(
+                str(arg)
+                for arg in
+                args
+            )
         )
     )
 
-    getReleaseCodes(
-        release_names = (source_name, filename_name, mode_name, flags_name,
-                         dont_inherit_name, optimize_name),
+    getErrorExitCode(
+        check_name    = to_name,
+        release_names = (
+            source_name,
+            filename_name,
+            mode_name,
+            flags_name,
+            dont_inherit_name,
+            optimize_name
+        ),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -211,7 +217,7 @@ def getBuiltinEvalCode(to_name, source_name, filename_name, globals_name,
                        locals_name, mode_name, emit, context):
     compiled_name = context.allocateTempName("eval_compiled")
 
-    getBuiltinCompileCode(
+    _getBuiltinCompileCode(
         to_name           = compiled_name,
         source_name       = source_name,
         filename_name     = filename_name,
@@ -232,18 +238,13 @@ def getBuiltinEvalCode(to_name, source_name, filename_name, globals_name,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (compiled_name, globals_name, locals_name),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -279,15 +280,23 @@ def generateExecCode(statement, emit, context):
 
     source_ref = statement.getSourceReference()
 
-    # Filename with origin in improved mode.
-    if Options.isFullCompat():
-        filename_name = context.getConstantCode(
-            constant = "<string>"
-        )
-    else:
-        filename_name = context.getConstantCode(
-            constant = "<string at %s>" % source_ref.getAsString()
-        )
+    filename_name = context.allocateTempName("exec_filename")
+
+    # Default filename with origin in improved mode.
+    getConstantAccess(
+        to_name  = filename_name,
+        constant = "<string>"
+                     if Options.isFullCompat() else
+                   "<string at %s>" % source_ref.getAsString(),
+        emit     = emit,
+        context  = context
+    )
+
+    getReferenceExportCode(filename_name, emit, context)
+    context.addCleanupTempName(filename_name)
+
+    getReferenceExportCode(source_name, emit, context)
+    context.addCleanupTempName(source_name)
 
     old_source_ref = context.setCurrentSourceCodeReference(
         locals_arg.getSourceReference()
@@ -295,9 +304,25 @@ def generateExecCode(statement, emit, context):
         statement.getSourceReference()
     )
 
+    res_name = context.getBoolResName()
+
+    emit(
+        "%s = EXEC_FILE_ARG_HANDLING( &%s, &%s );" % (
+            res_name,
+            source_name,
+            filename_name
+        )
+    )
+
+    getErrorExitBoolCode(
+        condition = "%s == false" % res_name,
+        emit      = emit,
+        context   = context
+    )
+
     compiled_name = context.allocateTempName("exec_compiled")
 
-    getBuiltinCompileCode(
+    _getBuiltinCompileCode(
         to_name           = compiled_name,
         source_name       = source_name,
         filename_name     = filename_name,
@@ -322,19 +347,15 @@ def generateExecCode(statement, emit, context):
         )
     )
 
-    getReleaseCodes(
-        release_names = (compiled_name, globals_name, locals_name),
+    getErrorExitCode(
+        check_name    = to_name,
+        release_names = (compiled_name, globals_name, locals_name, source_name, filename_name),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
-    # Immediately release the exec result.
+    # Immediately release the exec result, no point in keeping it, it's a
+    # statement.
     context.addCleanupTempName(to_name)
     getReleaseCode(
         release_name = to_name,
@@ -345,7 +366,6 @@ def generateExecCode(statement, emit, context):
     context.setCurrentSourceCodeReference(old_source_ref)
 
 
-
 def _generateEvalCode(to_name, node, emit, context):
     source_name = context.allocateTempName("eval_source")
     globals_name = context.allocateTempName("eval_globals")
@@ -395,23 +415,27 @@ def _generateEvalCode(to_name, node, emit, context):
 
 
 def generateEvalCode(to_name, expression, emit, context):
-    return _generateEvalCode(
-        to_name = to_name,
-        node    = expression,
-        emit    = emit,
-        context = context
-    )
+    with withObjectCodeTemporaryAssignment(to_name, "eval_result", expression, emit, context) \
+      as value_name:
+        _generateEvalCode(
+            to_name = value_name,
+            node    = expression,
+            emit    = emit,
+            context = context
+        )
 
 
 def generateExecfileCode(to_name, expression, emit, context):
     assert python_version < 300
 
-    return _generateEvalCode(
-        to_name = to_name,
-        node    = expression,
-        emit    = emit,
-        context = context
-    )
+    with withObjectCodeTemporaryAssignment(to_name, "execfile_result", expression, emit, context) \
+      as value_name:
+        _generateEvalCode(
+            to_name = value_name,
+            node    = expression,
+            emit    = emit,
+            context = context
+        )
 
 
 def generateLocalsDictSyncCode(statement, emit, context):
@@ -429,12 +453,12 @@ def generateLocalsDictSyncCode(statement, emit, context):
         statement.getSourceReference()
     )
 
-    getStoreLocalsCode(
-        locals_name = locals_name,
-        variables   = statement.previous_traces,
-        is_foreign  = context.hasForeignLocalsDict(),
-        emit        = emit,
-        context     = context
+    _getStoreLocalsCode(
+        locals_name     = locals_name,
+        variable_traces = statement.getPreviousVariablesTraces(),
+        is_dict         = locals_arg.getTypeShape() is ShapeTypeDict,
+        emit            = emit,
+        context         = context
     )
 
     context.setCurrentSourceCodeReference(old_source_ref)
