@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,8 +19,15 @@
 
 """
 
+from nuitka.__past__ import iterItems
+from nuitka.codegen import Emission
 from nuitka.Version import getNuitkaVersion, getNuitkaVersionYear
 
+from .CodeHelpers import (
+    decideConversionCheckNeeded,
+    generateStatementSequenceCode,
+    withObjectCodeTemporaryAssignment
+)
 from .CodeObjectCodes import getCodeObjectsDeclCode, getCodeObjectsInitCode
 from .ConstantCodes import allocateNestedConstants, getConstantInitCodes
 from .Indentation import indented
@@ -30,22 +37,25 @@ from .templates.CodeTemplatesModules import (
     template_module_exception_exit,
     template_module_noexception_exit
 )
+from .VariableCodes import getVariableReferenceCode
 
 
 def getModuleAccessCode(context):
     return "module_%s" % context.getModuleCodeName()
 
 
-def getModuleValues(context, module_name, module_identifier, codes,
-                    function_decl_codes, function_body_codes, outline_variables,
-                    temp_variables, is_main_module, is_internal_module):
+def getModuleValues(context, module_name, module_identifier, function_decl_codes,
+                    function_body_codes, outline_variables, temp_variables,
+                    is_main_module, is_top_module, is_internal_module,
+                    is_package):
     # For the module code, lots of arguments and attributes come together.
     # pylint: disable=too-many-locals
 
     # Temporary variable initializations
     # TODO: Move that to a place outside of functions.
     from .FunctionCodes import setupFunctionLocalVariables, finalizeFunctionLocalVariables
-    local_var_inits, cleanup = setupFunctionLocalVariables(
+
+    setupFunctionLocalVariables(
         context           = context,
         parameters        = None,
         closure_variables = (),
@@ -53,9 +63,34 @@ def getModuleValues(context, module_name, module_identifier, codes,
         temp_variables    = temp_variables
     )
 
-    finalizeFunctionLocalVariables(context, local_var_inits, cleanup)
+    module_codes = Emission.SourceCodeCollector()
 
-    if context.needsExceptionVariables():
+    module_body = context.getOwner().getBody()
+
+    generateStatementSequenceCode(
+        statement_sequence = module_body,
+        emit               = module_codes,
+        allow_none         = True,
+        context            = context,
+    )
+
+    for _identifier, code in sorted(iterItems(context.getHelperCodes())):
+        function_body_codes.append(code)
+
+    for _identifier, code in sorted(iterItems(context.getDeclarations())):
+        function_decl_codes.append(code)
+
+    function_body_codes = "\n\n".join(function_body_codes)
+    function_decl_codes = "\n\n".join(function_decl_codes)
+
+    _cleanup = finalizeFunctionLocalVariables(context)
+
+    # TODO: Seems like a bug, classes could produce those.
+    # assert not _cleanup, _cleanup
+
+    local_var_inits = context.variable_storage.makeCFunctionLevelDeclarations()
+
+    if module_body is not None and module_body.mayRaiseException(BaseException):
         module_exit = template_module_exception_exit
     else:
         module_exit = template_module_noexception_exit
@@ -66,11 +101,13 @@ def getModuleValues(context, module_name, module_identifier, codes,
             constant = module_name
         ),
         "is_main_module"           : 1 if is_main_module else 0,
+        "is_top_module"            : 1 if is_top_module else 0,
+        "is_package"               : 1 if is_package else 0,
         "module_identifier"        : module_identifier,
         "module_functions_decl"    : function_decl_codes,
         "module_functions_code"    : function_body_codes,
         "temps_decl"               : indented(local_var_inits),
-        "module_code"              : indented(codes),
+        "module_code"              : indented(module_codes.codes),
         "module_exit"              : module_exit,
         "module_code_objects_decl" : indented(
             getCodeObjectsDeclCode(context),
@@ -123,25 +160,26 @@ def getModuleCode(module_context, template_values):
     return header + template_module_body_template % template_values
 
 
-def generateModuleFileAttributeCode(to_name, expression, emit, context):
-    # The expression doesn't really matter, but it is part of the API for
-    # the expression registry, pylint: disable=unused-argument
-
-    emit(
-        "%s = module_filename_obj;" % (
-            to_name,
-        )
-    )
-
+def generateModuleAttributeFileCode(to_name, expression, emit, context):
+    # TODO: Special treatment justified?
     context.markAsNeedsModuleFilenameObject()
 
-
-def generateModuleLoaderRefCode(to_name, expression, emit, context):
-    # The expression doesn't really matter, but it is part of the API for
-    # the expression registry, pylint: disable=unused-argument
-
-    emit(
-        "%s = metapath_based_loader;" % (
-            to_name,
+    with withObjectCodeTemporaryAssignment(to_name, "module_fileattr_value", expression, emit, context) \
+      as result_name:
+        emit(
+            "%s = module_filename_obj;" % (
+                result_name,
+            )
         )
+
+
+def generateModuleAttributeCode(to_name, expression, emit, context):
+    getVariableReferenceCode(
+        to_name          = to_name,
+        variable         = expression.getVariable(),
+        variable_trace   = None,
+        needs_check      = False,
+        conversion_check = decideConversionCheckNeeded(to_name, expression),
+        emit             = emit,
+        context          = context
     )
