@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -281,7 +281,7 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d( PyObject *called, PyObject **ar
             return NULL;
 #endif
         }
-        else
+        else if ( flags & METH_VARARGS )
         {
             PyCFunction method = PyCFunction_GET_FUNCTION( called );
             PyObject *self = PyCFunction_GET_SELF( called );
@@ -290,8 +290,6 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d( PyObject *called, PyObject **ar
 
             PyObject *result;
 
-            assert( flags && METH_VARARGS );
-
             // Recursion guard is not strictly necessary, as we already have
             // one on our way to here.
 #ifdef _NUITKA_FULL_COMPAT
@@ -302,7 +300,7 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d( PyObject *called, PyObject **ar
 #endif
 
 #if PYTHON_VERSION < 360
-            if ( flags && METH_KEYWORDS )
+            if ( flags & METH_KEYWORDS )
             {
                 result = (*(PyCFunctionWithKeywords)method)( self, pos_args, NULL );
             }
@@ -317,7 +315,11 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d( PyObject *called, PyObject **ar
             }
             else if ( flags == METH_FASTCALL )
             {
+#if PYTHON_VERSION < 370
                 result = (*(_PyCFunctionFast)method)( self, &PyTuple_GET_ITEM( pos_args, 0 ), %(args_count)d, NULL );;
+#else
+                result = (*(_PyCFunctionFast)method)( self, &pos_args, %(args_count)d );;
+#endif
             }
             else
             {
@@ -331,8 +333,8 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d( PyObject *called, PyObject **ar
 
             if ( result != NULL )
             {
-            // Some buggy C functions do set an error, but do not indicate it
-            // and Nuitka inner workings can get upset/confused from it.
+                // Some buggy C functions do set an error, but do not indicate it
+                // and Nuitka inner workings can get upset/confused from it.
                 DROP_ERROR_OCCURRED();
 
                 Py_DECREF( pos_args );
@@ -523,10 +525,14 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d( PyObject *source, PyObject *attr_
         if ( descr != NULL )
         {
             CHECK_OBJECT( descr );
-            return CALL_FUNCTION_WITH_ARGS%(args_count)d(
+
+            PyObject *result = CALL_FUNCTION_WITH_ARGS%(args_count)d(
                 descr,
                 args
             );
+            Py_DECREF( descr );
+
+            return result;
         }
 
 #if PYTHON_VERSION < 300
@@ -590,7 +596,7 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d( PyObject *source, PyObject *attr_
                     %(args_count)d
                 );
             }
-            else
+            else if ( descr_get != NULL )
             {
                 PyObject *method = descr_get(
                     called_object,
@@ -607,6 +613,11 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d( PyObject *source, PyObject *attr_
                 Py_DECREF( method );
                 return result;
             }
+            else
+            {
+                return CALL_FUNCTION_WITH_ARGS%(args_count)d( called_object, args );
+            }
+
         }
         else if (unlikely( source_instance->in_class->cl_getattr == NULL ))
         {
@@ -671,7 +682,7 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d( PyObject *source, PyObject *attr_
     {
         PyObject *called_object = (*type->tp_getattr)(
             source,
-            Nuitka_String_AsString_Unchecked( attr_name )
+            (char *)Nuitka_String_AsString_Unchecked( attr_name )
         );
 
         if (unlikely( called_object == NULL ))
