@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,13 +27,13 @@ template_function_direct_declaration = """\
 %(file_scope)s PyObject *impl_%(function_identifier)s( %(direct_call_arg_spec)s );
 """
 
-template_make_function_template = """
+template_make_function_body = """
 static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
 {
     struct Nuitka_FunctionObject *result = Nuitka_Function_New(
         %(function_impl_identifier)s,
         %(function_name_obj)s,
-#if PYTHON_VERSION >= 330
+#if PYTHON_VERSION >= 300
         %(function_qualname_obj)s,
 #endif
         %(code_identifier)s,
@@ -46,11 +46,16 @@ static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args
         %(function_doc)s,
         %(closure_count)d
     );
-%(closure_copy)s
+
     return (PyObject *)result;
 }
 """
 
+template_make_function = """\
+%(to_name)s = MAKE_FUNCTION_%(function_identifier)s( %(args)s );
+%(closure_copy)s
+"""
+
 template_function_body = """\
 static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
 {
@@ -62,7 +67,7 @@ static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
     // Local variable declarations.
 %(function_locals)s
 
-    // Actual function code.
+    // Actual function body.
 %(function_body)s
 
 %(function_exit)s
@@ -72,19 +77,22 @@ static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
 template_function_exception_exit = """\
 function_exception_exit:
 %(function_cleanup)s\
-    assert( exception_type );
-    RESTORE_ERROR_OCCURRED( exception_type, exception_value, exception_tb );
+    assert( %(exception_type)s );
+    RESTORE_ERROR_OCCURRED( %(exception_type)s, %(exception_value)s, %(exception_tb)s );
 
     return NULL;
 """
 
-template_function_return_exit = """\
+template_function_return_exit = """
 function_return_exit:
+   // Function cleanup code if any.
 %(function_cleanup)s
-CHECK_OBJECT( tmp_return_value );
-assert( had_error || !ERROR_OCCURRED() );
-return tmp_return_value;
-"""
+
+   // Actual function exit with return value, making sure we did not make
+   // the error status worse despite non-NULL return.
+   CHECK_OBJECT( tmp_return_value );
+   assert( had_error || !ERROR_OCCURRED() );
+   return tmp_return_value;"""
 
 function_direct_body_template = """\
 %(file_scope)s PyObject *impl_%(function_identifier)s( %(direct_call_arg_spec)s )
@@ -97,17 +105,12 @@ function_direct_body_template = """\
     // Local variable declarations.
 %(function_locals)s
 
-    // Actual function code.
+    // Actual function body.
 %(function_body)s
 
 %(function_exit)s
 }
 """
 
-function_dict_setup = """\
-// Locals dictionary setup.
-PyObject *%(locals_dict)s = PyDict_New();
-"""
-
 from . import TemplateDebugWrapper # isort:skip
 TemplateDebugWrapper.checkDebug(globals())
