@@ -19,109 +19,91 @@
 
 """
 
-template_genfunc_yielder_decl_template = """\
-#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
-static PyObject *%(function_identifier)s_context( struct Nuitka_GeneratorObject *generator, PyObject *yield_return_value );
-#else
-static void %(function_identifier)s_context( struct Nuitka_GeneratorObject *generator );
-#endif
+template_genfunc_yielder_maker_decl = """\
+static PyObject *%(function_identifier)s_maker( void );
 """
 
 template_genfunc_yielder_body_template = """
-#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
 struct %(function_identifier)s_locals {
 %(function_local_types)s
 };
-#endif
 
-#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
 static PyObject *%(function_identifier)s_context( struct Nuitka_GeneratorObject *generator, PyObject *yield_return_value )
-#else
-static void %(function_identifier)s_context( struct Nuitka_GeneratorObject *generator )
-#endif
 {
     CHECK_OBJECT( (PyObject *)generator );
     assert( Nuitka_Generator_Check( (PyObject *)generator ) );
 
-    // Local variable initialization
-%(function_var_inits)s
+    // Heap access if used.
+%(heap_declaration)s
 
     // Dispatch to yield based on return label index:
 %(function_dispatch)s
 
-    // Actual function code.
+    // Local variable initialization
+%(function_var_inits)s
+
+    // Actual generator function body.
 %(function_body)s
 
 %(generator_exit)s
 }
+
+static PyObject *%(function_identifier)s_maker( void )
+{
+    return Nuitka_Generator_New(
+        %(function_identifier)s_context,
+        %(generator_module)s,
+        %(generator_name_obj)s,
+#if PYTHON_VERSION >= 350
+        %(generator_qualname_obj)s,
+#endif
+        %(code_identifier)s,
+        %(closure_count)d,
+        sizeof(struct %(function_identifier)s_locals)
+    );
+}
 """
 
+template_make_generator = """\
+%(to_name)s = %(generator_identifier)s_maker();
+%(closure_copy)s
+"""
+
+
 template_generator_exception_exit = """\
-%(function_cleanup)s\
-#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+%(function_cleanup)s
     return NULL;
-#else
-    generator->m_yielded = NULL;
-    return;
-#endif
 
     function_exception_exit:
 %(function_cleanup)s\
-    assert( exception_type );
-    RESTORE_ERROR_OCCURRED( exception_type, exception_value, exception_tb );
+    assert( %(exception_type)s );
+    RESTORE_ERROR_OCCURRED( %(exception_type)s, %(exception_value)s, %(exception_tb)s );
 
-#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
     return NULL;
-#else
-    generator->m_yielded = NULL;
-    return;
-#endif
 """
 
 template_generator_noexception_exit = """\
     // Return statement need not be present.
 %(function_cleanup)s\
 
-#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
     return NULL;
-#else
-    generator->m_yielded = NULL;
-    return;
-#endif
 """
 
+# TODO: Clarify if cannot get here function is to be used.
 template_generator_return_exit = """\
     // The above won't return, but we need to make it clear to the compiler
     // as well, or else it will complain and/or generate inferior code.
     assert(false);
-    return;
+    return NULL;
 
     function_return_exit:
 #if PYTHON_VERSION >= 300
-    generator->m_returned = tmp_return_value;
+    generator->m_returned = %(return_value)s;
 #endif
 
-#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
     return NULL;
-#else
-    generator->m_yielded = NULL;
-    return;
-#endif
 """
 
-template_generator_making = """\
-%(to_name)s = Nuitka_Generator_New(
-    %(generator_identifier)s_context,
-    %(generator_module)s,
-    %(generator_name_obj)s,
-#if PYTHON_VERSION >= 350
-    %(generator_qualname_obj)s,
-#endif
-    %(code_identifier)s,
-    %(closure_count)d
-);
-%(closure_copy)s
-"""
 
 
 from . import TemplateDebugWrapper # isort:skip
