@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -37,9 +37,11 @@ the ``os`` module like it's done in ``isStandardLibraryPath`` of this module.
 
 from __future__ import print_function
 
+import hashlib
 import imp
 import os
 import sys
+import zipfile
 from logging import warning
 
 from nuitka import Options
@@ -47,6 +49,7 @@ from nuitka.containers.oset import OrderedSet
 from nuitka.importing import StandardLibrary
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
+from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.FileOperations import listDir
 
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
@@ -78,9 +81,10 @@ def isPackageDir(dirname):
         extra packages provided via "*.pth" file tricks by "site.py" loading.
     """
 
-    return os.path.isdir(dirname) and \
+    return '.' not in os.path.basename(dirname) and \
+           os.path.isdir(dirname) and \
            (
-               python_version >= 330 or
+               python_version >= 300 or
                os.path.isfile(os.path.join(dirname, "__init__.py")) or
                isPreloadedPackagePath(dirname)
            )
@@ -137,7 +141,7 @@ def isWhiteListedImport(node):
 
 
 def warnAbout(importing, module_name, parent_package, level, tried_names):
-    # This probably should not be dealt with here.
+    # This probably should not be dealt with here, pylint: disable=too-many-branches
     if module_name == "":
         return
 
@@ -165,22 +169,23 @@ def warnAbout(importing, module_name, parent_package, level, tried_names):
             else:
                 level_desc = "%d package levels up" % level
 
-            if parent_package is not None:
-                warning(
-                    "%s: Cannot find '%s' in package '%s' %s (tried %s).",
-                    importing.getSourceReference().getAsString(),
-                    module_name,
-                    parent_package,
-                    level_desc,
-                    ','.join(tried_names)
-                )
-            else:
-                warning(
-                    "%s: Cannot find '%s' %s.",
-                    importing.getSourceReference().getAsString(),
-                    module_name,
-                    level_desc
-                )
+            if _debug_module_finding:
+                if parent_package is not None:
+                    warning(
+                        "%s: Cannot find '%s' in package '%s' %s (tried %s).",
+                        importing.getSourceReference().getAsString(),
+                        module_name,
+                        parent_package,
+                        level_desc,
+                        ','.join(tried_names)
+                    )
+                else:
+                    warning(
+                        "%s: Cannot find '%s' %s.",
+                        importing.getSourceReference().getAsString(),
+                        module_name,
+                        level_desc
+                    )
 
 
 def normalizePackageName(module_name):
@@ -351,7 +356,7 @@ def _findModuleInPath2(module_name, search_path):
                     )
                     break
             else:
-                if python_version >= 330:
+                if python_version >= 300:
                     candidates.add(
                         (entry, 2, package_directory)
                     )
@@ -394,12 +399,41 @@ def _findModuleInPath2(module_name, search_path):
     # Nothing found.
     raise ImportError
 
+_egg_files = {}
+
+def _unpackPathElement(path_entry):
+    if not path_entry:
+        return '.' # empty means current directory
+
+    if os.path.isfile(path_entry) and path_entry.lower().endswith(".egg"):
+        if path_entry not in _egg_files:
+            checksum = hashlib.md5(open(path_entry, "rb").read()).hexdigest()
+
+            target_dir = os.path.join(
+                getCacheDir(),
+                "egg-content",
+                checksum
+            )
+
+            zip_ref = zipfile.ZipFile(path_entry, 'r')
+            zip_ref.extractall(target_dir)
+            zip_ref.close()
+
+            _egg_files[path_entry] = target_dir
+
+        return _egg_files[path_entry]
+
+    return path_entry
 
 def getPackageSearchPath(package_name):
     assert main_path is not None
 
     if package_name is None:
-        return [os.getcwd(), main_path] + sys.path
+        return [os.getcwd(), main_path] + [
+            _unpackPathElement(path_element)
+            for path_element in
+            sys.path
+        ]
     elif '.' in package_name:
         parent_package_name, child_package_name = package_name.rsplit('.', 1)
 
@@ -457,7 +491,7 @@ def _findModuleInPath(module_name, package_name):
 
     # Free pass for built-in modules, the need not exist.
     if package_name is None and imp.is_builtin(module_name):
-        return None, module_name
+        return None
 
     search_path = getPackageSearchPath(package_name)
 
@@ -528,17 +562,17 @@ def _findModule2(module_name):
     # Need a real module name.
     assert module_name != ""
 
+    preloaded_path = getPreloadedPackagePath(module_name)
+
+    if preloaded_path is not None:
+        return preloaded_path[0]
+
     if '.' in module_name:
         package_part = module_name[ : module_name.rfind('.') ]
         module_name = module_name[ module_name.rfind('.') + 1 : ]
     else:
         package_part = None
 
-    preloaded_path = getPreloadedPackagePath(module_name)
-
-    if preloaded_path is not None:
-        return preloaded_path[0]
-
     return _findModuleInPath(
         module_name  = module_name,
         package_name = package_part
