@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -66,7 +66,7 @@ def getModuleWhiteList():
         "collections.abc",
 
         # test_compile.py
-        "__package__.module", "__mangled_mod",
+        "__package__.module", "__mangled_mod", "__package__",
 
         # test_ctypes
         "ctypes.test",
@@ -81,7 +81,7 @@ def getModuleWhiteList():
         "distutils.tests", "distutils.mwerkscompiler",
 
         # test_docxmlrpc.py
-        "xmlrpc.server",
+        "xmlrpc",
 
         # test_emails.py
         "email.test.test_email", "email.test.test_email_renamed",
@@ -145,9 +145,13 @@ def getModuleWhiteList():
         # test_macostools.py
         "macostools",
 
+        # test_msilib.py
+        "msilib",
+
         # test_namespace_pkgs.py
         "foo.one", "foo.two", "parent.child.one", "parent.child.two",
-        "parent.child.three", "bar.two", "a_test",
+        "parent.child.three", "bar.two", "a_test", "parent.child", "parent",
+        "bar",
 
         # test_new.py
         "Spam",
@@ -160,14 +164,20 @@ def getModuleWhiteList():
         # test_platform.py
         "gestalt",
 
+        # test_pickleable.py
+        "email.headerregistry",
+
         # test_pkg.py
         "t1", "t2", "t2.sub", "t2.sub.subsub", "t3.sub.subsub", "t5", "t6",
-        "t7", "t7.sub", "t7.sub.subsub", "t8",
+        "t7", "t7.sub", "t7.sub.subsub", "t8", "t3.sub", "t3",
 
         # test_pkgutil.py
         "foo", "foo.bar", "foo.baz", "zipimport", "pkg", "pkg.subpkg",
         "pkg.subpkg.c", "pkg.subpkg.d",
 
+        # test_policy.py
+        "email.policy",
+
         # test_urllib.py
         "urllib.parse",
 
@@ -177,6 +187,7 @@ def getModuleWhiteList():
         # test_repr.py
         """areallylongpackageandmodulenametotestreprtruncation.\
 areallylongpackageandmodulenametotestreprtruncation""",
+        "areallylongpackageandmodulenametotestreprtruncation",
 
         # test_robotparser.py
         "urllib.error", "urllib.robotparser",
@@ -184,6 +195,9 @@ areallylongpackageandmodulenametotestreprtruncation""",
         # test_runpy.py
         "test.script_helper",
 
+        # test_secrets.py
+        "secrets",
+
         # test_selectors.py
         "selectors",
 
@@ -226,15 +240,24 @@ areallylongpackageandmodulenametotestreprtruncation""",
         # test_unittest.py
         "unittest.test",
 
+        # test_wsgiref.py
+        "test.test_httpservers",
+
         # test_xml_etree.py
         "xml.parsers.expat.errors",
 
+        # test_xmlrpc.py
+        "xmlrpc.client",
+
         # test_zipimport_support.py
         "test_zipped_doctest", "zip_pkg",
 
         # test/test_zipimport_support.py
         "test.test_cmd_line_script",
 
+        # test_winconsoleio.py
+        "_testconsole",
+
         # Python3: modules that no longer exist
         "commands", "dummy_thread", "_dummy_thread", "httplib", "Queue", "sets",
 
@@ -259,9 +282,6 @@ areallylongpackageandmodulenametotestreprtruncation""",
         "elementree.ElementTree", "_gbdm", "resource", "crypt", "bz2", "dbm",
         "mmap", "Mailman",
 
-        # Nuitka tests
-        "test_common",
-
         # Mercurial test
         "statprof", "email.Generator", "email.Utils",
 
@@ -270,11 +290,36 @@ areallylongpackageandmodulenametotestreprtruncation""",
 
         # reportlab does use this if present only and warns about itself.
         "pyfribidi2", "macfs",
+
+        # psutils
+        "_psutil_windows",
+
+        # nose
+        "unittest2", "IronPython", "clr", "compiler.consts", "new",
+
+        # pkg_resources
+        "pkg_resources.extern", "ordereddict",
+
+        # appdirs
+        "com", "win32com",
+
+        # gtk
+        "gdk",
+
+        # six
+        "six.moves",
+
+        # Python3 namespace packages.
+        "_frozen_importlib_external",
     )
 
 
 def isWhiteListedNotExistingModule(module_name):
-    result = module_name in getModuleWhiteList()
+    result = False
+    for white_listed in getModuleWhiteList():
+        if module_name == white_listed or module_name.startswith(white_listed + '.'):
+            result = True
+            break
 
     if not result and module_name in sys.builtin_module_names:
         warning("""\
