@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,32 +32,31 @@ the traces.
 
 from nuitka.ModuleRegistry import getOwnerFromCodeName
 
-from .NodeBases import NodeBase, StatementChildrenHavingBase
+from .NodeBases import StatementBase, StatementChildHavingBase
 from .NodeMakingHelpers import (
     makeStatementExpressionOnlyReplacementNode,
     makeStatementsSequenceReplacementNode
 )
 
 
-class StatementAssignmentVariableName(StatementChildrenHavingBase):
+class StatementAssignmentVariableName(StatementChildHavingBase):
     """ Precursor of StatementAssignmentVariable used during tree building phase
 
     """
 
     kind = "STATEMENT_ASSIGNMENT_VARIABLE_NAME"
 
-    named_children = (
-        "source",
-    )
+    named_child = "source"
+    nice_child = "assignment source"
+
+    __slots__ = ("variable_name", "provider")
 
     def __init__(self, provider, variable_name, source, source_ref):
         assert source is not None, source_ref
 
-        StatementChildrenHavingBase.__init__(
+        StatementChildHavingBase.__init__(
             self,
-            values     = {
-                "source"       : source,
-            },
+            value      = source,
             source_ref = source_ref
         )
 
@@ -77,14 +76,18 @@ class StatementAssignmentVariableName(StatementChildrenHavingBase):
 
     def computeStatement(self, trace_collection):
         # Only for abc, pylint: disable=no-self-use
+
+        # These must not enter real optimization, they only live during the
+        # tree building.
         assert False
 
-    getAssignSource = StatementChildrenHavingBase.childGetter(
-        "source"
-    )
+    def getStatementNiceName(self):
+        return "variable assignment statement"
+
+    getAssignSource = StatementChildHavingBase.childGetter("source")
 
 
-class StatementDelVariableName(NodeBase):
+class StatementDelVariableName(StatementBase):
     """ Precursor of StatementDelVariable used during tree building phase
 
     """
@@ -94,7 +97,7 @@ class StatementDelVariableName(NodeBase):
     __slots__ = "variable_name", "provider", "tolerant"
 
     def __init__(self, provider, variable_name, tolerant, source_ref):
-        NodeBase.__init__(
+        StatementBase.__init__(
             self,
             source_ref = source_ref
         )
@@ -104,6 +107,10 @@ class StatementDelVariableName(NodeBase):
 
         self.tolerant = tolerant
 
+    def finalize(self):
+        del self.parent
+        del self.provider
+
     def getDetails(self):
         return {
             "variable_name" : self.variable_name,
@@ -116,10 +123,13 @@ class StatementDelVariableName(NodeBase):
 
     def computeStatement(self, trace_collection):
         # Only for abc, pylint: disable=no-self-use
+
+        # These must not enter real optimization, they only live during the
+        # tree building.
         assert False
 
 
-class StatementAssignmentVariable(StatementChildrenHavingBase):
+class StatementAssignmentVariable(StatementChildHavingBase):
     """ Assignment to a variable from an expression.
 
         All assignment forms that are not to attributes, slices, subscripts
@@ -135,11 +145,10 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
 
     kind = "STATEMENT_ASSIGNMENT_VARIABLE"
 
-    named_children = (
-        "source",
-    )
+    named_child = "source"
+    nice_child = "assignment source"
 
-    inplace_suspect = None
+    __slots__ = ("variable", "variable_version", "variable_trace", "inplace_suspect")
 
     def __init__(self, source, variable, source_ref, version = None):
         assert source is not None, source_ref
@@ -151,15 +160,14 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
         self.variable = variable
         self.variable_version = version
 
-        StatementChildrenHavingBase.__init__(
+        StatementChildHavingBase.__init__(
             self,
-            values     = {
-                "source"       : source,
-            },
+            value      = source,
             source_ref = source_ref
         )
 
         self.variable_trace = None
+        self.inplace_suspect = None
 
     def getDetail(self):
         if self.variable is not None:
@@ -215,10 +223,10 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
             source_ref = self.source_ref
         )
 
-    getAssignSource = StatementChildrenHavingBase.childGetter(
+    getAssignSource = StatementChildHavingBase.childGetter(
         "source"
     )
-    setAssignSource = StatementChildrenHavingBase.childSetter(
+    setAssignSource = StatementChildHavingBase.childSetter(
         "source"
     )
 
@@ -232,8 +240,8 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
         self.variable = variable
         self.variable_version = variable.allocateTargetNumber()
 
-    def getVariableVersion(self):
-        return self.variable_version
+    def getVariableTrace(self):
+        return self.variable_trace
 
     def markAsInplaceSuspect(self):
         self.inplace_suspect = True
@@ -245,7 +253,7 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
         return self.getAssignSource().mayRaiseException(exception_type)
 
     def computeStatement(self, trace_collection):
-        # This is very complex stuff, pylint: disable=too-many-branches
+        # This is very complex stuff, pylint: disable=too-many-branches,too-many-return-statements
 
         # TODO: Way too ugly to have global trace kinds just here, and needs to
         # be abstracted somehow. But for now we let it live here.
@@ -297,6 +305,8 @@ Side effects of assignments promoted to statements."""
                 node       = self
             )
 
+            del self.parent
+
             return result, "new_raise", """\
 Assignment raises exception in assigned value, removed assignment."""
 
@@ -333,32 +343,60 @@ Reduced assignment of %s from itself to mere access of it.""" % variable.getDesc
 Removed assignment of %s from itself which is known to be defined.""" % variable.getDescription()
 
 
-
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
         self.variable_trace = trace_collection.onVariableSet(
+            variable    = variable,
+            version     = self.variable_version,
             assign_node = self
         )
 
-        provider = self.getParentVariableProvider()
+        provider = trace_collection.getOwner()
 
         if variable.hasAccessesOutsideOf(provider) is False:
             last_trace = variable.getMatchingAssignTrace(self)
 
             if last_trace is not None:
-                if source.isCompileTimeConstant():
-                    # Can safely forward propagate only non-mutable constants.
-                    if not source.isMutable():
-                        if not last_trace.getNameUsageCount():
-                            self.variable_trace.setReplacementNode(
-                                lambda _usage : source.makeClone()
-                            )
+                if source.isCompileTimeConstant() and \
+                   not last_trace.hasLoopUsages():
+                    if not variable.isModuleVariable():
 
-                            propagated = True
+                        # Can safely forward propagate only non-mutable constants.
+                        if source.isMutable():
+                            # Something might be possible still, lets check for
+                            # ununused.
+                            if not last_trace.hasPotentialUsages() and \
+                               not last_trace.hasDefiniteUsages() and \
+                               not last_trace.getNameUsageCount():
+                                if not last_trace.getPrevious().isUninitTrace():
+                                    # TODO: We could well decide, if that's even necessary, but for now
+                                    # the "StatementDelVariable" is tasked with that.
+                                    result = StatementDelVariable(
+                                        variable   = self.variable,
+                                        version    = self.variable_version,
+                                        tolerant   = True,
+                                        source_ref = self.getSourceReference()
+                                    )
+                                else:
+                                    result = None
+
+                                return (
+                                    result,
+                                    "new_statements",
+                                    "Dropped dead assignment statement to '%s'." % (
+                                       self.getVariableName()
+                                    )
+                                )
                         else:
-                            propagated = False
+                            if not last_trace.getNameUsageCount():
+                                self.variable_trace.setReplacementNode(
+                                    lambda _usage : source.makeClone()
+                                )
+
+                                propagated = True
+                            else:
+                                propagated = False
 
-                        if not variable.isModuleVariable():
                             if not last_trace.hasPotentialUsages() and not last_trace.getNameUsageCount():
                                 if not last_trace.getPrevious().isUninitTrace():
                                     # TODO: We could well decide, if that's even necessary, but for now
@@ -380,10 +418,6 @@ Removed assignment of %s from itself which is known to be defined.""" % variable
                                        self.getVariableName()
                                     )
                                 )
-                    else:
-                        # Something might be possible still.
-
-                        pass
                 else:
                     # More cases thinkable.
                     pass
@@ -400,8 +434,11 @@ Removed assignment of %s from itself which is known to be defined.""" % variable
         else:
             return None
 
+    def getStatementNiceName(self):
+        return "variable assignment statement"
 
-class StatementDelVariable(NodeBase):
+
+class StatementDelVariable(StatementBase):
     """ Deleting a variable.
 
         All del forms that are not to attributes, slices, subscripts
@@ -423,26 +460,31 @@ class StatementDelVariable(NodeBase):
     def __init__(self, tolerant, source_ref, variable, version = None):
         if type(tolerant) is str:
             tolerant = tolerant == "True"
-
         assert tolerant is True or tolerant is False, repr(tolerant)
 
         if variable is not None:
             if version is None:
                 version = variable.allocateTargetNumber()
 
-        self.variable = variable
-        self.variable_version = version
-
-        NodeBase.__init__(
+        StatementBase.__init__(
             self,
             source_ref = source_ref
         )
 
+        self.variable = variable
+        self.variable_version = version
+
         self.variable_trace = None
         self.previous_trace = None
 
         self.tolerant = tolerant
 
+    def finalize(self):
+        del self.parent
+        del self.variable
+        del self.variable_trace
+        del self.previous_trace
+
     def getDetail(self):
         if self.variable is not None:
             return "to variable %s" % self.variable
@@ -507,18 +549,19 @@ class StatementDelVariable(NodeBase):
     def getVariableName(self):
         return self.variable.getName()
 
+    def getVariableTrace(self):
+        return self.variable_trace
+
+    def getPreviousVariableTrace(self):
+        return self.previous_trace
+
     def getVariable(self):
         return self.variable
 
     def setVariable(self, variable):
-        assert self.variable_version is None
-
         self.variable = variable
         self.variable_version = variable.allocateTargetNumber()
 
-    def getVariableVersion(self):
-        return self.variable_version
-
     def computeStatement(self, trace_collection):
         variable = self.variable
 
@@ -543,7 +586,7 @@ class StatementDelVariable(NodeBase):
             trace_collection.onExceptionRaiseExit(BaseException)
 
         # Record the deletion, needs to start a new version then.
-        trace_collection.onVariableDel(
+        self.variable_trace = trace_collection.onVariableDel(
             variable = variable,
             version  = self.variable_version
         )
@@ -553,10 +596,6 @@ class StatementDelVariable(NodeBase):
         # Any code could be run, note that.
         trace_collection.onControlFlowEscape(self)
 
-        # Need to fetch the potentially invalidated variable. A "del" on a
-        # or shared value, may easily assign the global variable in "__del__".
-        self.variable_trace = trace_collection.getVariableCurrentTrace(variable)
-
         return self, None, None
 
     def mayHaveSideEffects(self):
@@ -567,12 +606,10 @@ class StatementDelVariable(NodeBase):
             return False
         else:
             if self.variable_trace is not None:
-                variable = self.getVariable()
-
                 # Temporary variables deletions won't raise, just because we
                 # don't create them that way. We can avoid going through SSA in
                 # these cases.
-                if variable.isTempVariable():
+                if self.variable.isTempVariable():
                     return False
 
                 # If SSA knows, that's fine.
@@ -583,7 +620,7 @@ class StatementDelVariable(NodeBase):
             return True
 
 
-class StatementReleaseVariable(NodeBase):
+class StatementReleaseVariable(StatementBase):
     """ Releasing a variable.
 
         Just release the value, which of course is not to be used afterwards.
@@ -599,7 +636,7 @@ class StatementReleaseVariable(NodeBase):
     def __init__(self, variable, source_ref):
         assert variable is not None, source_ref
 
-        NodeBase.__init__(
+        StatementBase.__init__(
             self,
             source_ref = source_ref
         )
@@ -608,6 +645,11 @@ class StatementReleaseVariable(NodeBase):
 
         self.variable_trace = None
 
+    def finalize(self):
+        del self.variable
+        del self.variable_trace
+        del self.parent
+
     def getDetail(self):
         return "of variable %s" % self.variable
 
@@ -639,8 +681,8 @@ class StatementReleaseVariable(NodeBase):
     def getVariable(self):
         return self.variable
 
-    def getVariableVersion(self):
-        return self.variable_trace.getVersion()
+    def getVariableTrace(self):
+        return self.variable_trace
 
     def setVariable(self, variable):
         self.variable = variable
