@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,23 +29,34 @@ from nuitka.Builtins import (
     builtin_anon_names,
     builtin_exception_names,
     builtin_exception_values,
-    builtin_names
+    builtin_names,
+    builtin_type_names
 )
-from nuitka.optimizations import BuiltinOptimization
+from nuitka.Options import getPythonFlags
 from nuitka.PythonVersions import python_version
+from nuitka.specs import BuiltinParameterSpecs
 
-from .ConstantRefNodes import ExpressionConstantRef
+from .ConstantRefNodes import makeConstantRefNode
 from .ExceptionNodes import ExpressionBuiltinMakeException
-from .NodeBases import CompileTimeConstantExpressionMixin, NodeBase
+from .ExpressionBases import CompileTimeConstantExpressionBase
 
 
-class ExpressionBuiltinRefBase(CompileTimeConstantExpressionMixin, NodeBase):
+class ExpressionBuiltinRefBase(CompileTimeConstantExpressionBase):
+    # Base classes can be abstract, pylint: disable=abstract-method
+
+    __slots__ = ("builtin_name",)
+
     def __init__(self, builtin_name, source_ref):
-        NodeBase.__init__(self, source_ref = source_ref)
-        CompileTimeConstantExpressionMixin.__init__(self)
+        CompileTimeConstantExpressionBase.__init__(
+            self,
+            source_ref = source_ref
+        )
 
         self.builtin_name = builtin_name
 
+    def finalize(self):
+        del self.parent
+
     def getDetails(self):
         return {
             "builtin_name" : self.builtin_name
@@ -64,19 +75,48 @@ class ExpressionBuiltinRefBase(CompileTimeConstantExpressionMixin, NodeBase):
         return False
 
     def getStrValue(self):
-        return ExpressionConstantRef(
+        return makeConstantRefNode(
             constant      = str(self.getCompileTimeConstant()),
             user_provided = True,
             source_ref    = self.getSourceReference(),
         )
 
+_debug_value = "no_asserts" not in getPythonFlags()
+
+def makeExpressionBuiltinRef(builtin_name, source_ref):
+    assert builtin_name in builtin_names, builtin_name
+
+    quick_names = {
+        "None"      : None,
+        "True"      : True,
+        "False"     : False,
+        "__debug__" : _debug_value,
+        "Ellipsis"  : Ellipsis,
+    }
+
+    if builtin_name in quick_names:
+        return makeConstantRefNode(
+            constant   = quick_names[builtin_name],
+            source_ref = source_ref
+        )
+    elif builtin_name in builtin_type_names:
+        return makeConstantRefNode(
+            constant   = __builtins__[builtin_name],
+            source_ref = source_ref
+        )
+    else:
+        return ExpressionBuiltinRef(
+            builtin_name = builtin_name,
+            source_ref   = source_ref
+        )
+
 
 class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_REF"
 
-    def __init__(self, builtin_name, source_ref):
-        assert builtin_name in builtin_names, builtin_name
+    __slots__ = ()
 
+    def __init__(self, builtin_name, source_ref):
         ExpressionBuiltinRefBase.__init__(
             self,
             builtin_name = builtin_name,
@@ -87,44 +127,27 @@ class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
         return True
 
     def getCompileTimeConstant(self):
-        return __builtins__[ self.builtin_name ]
-
-    def computeExpression(self, constraint_collection):
-        quick_names = {
-            "None"      : None,
-            "True"      : True,
-            "False"     : False,
-            "__debug__" : __debug__,
-            "Ellipsis"  : Ellipsis,
-        }
-
-        if self.builtin_name in quick_names:
-            new_node = ExpressionConstantRef(
-                constant   = quick_names[self.builtin_name],
-                source_ref = self.getSourceReference()
-            )
-
-            return new_node, "new_constant", """\
-Built-in constant '%s' resolved.""" % self.builtin_name
+        return __builtins__[self.builtin_name]
 
+    def computeExpressionRaw(self, trace_collection):
         return self, None, None
 
     def computeExpressionCall(self, call_node, call_args, call_kw,
-                              constraint_collection):
+                              trace_collection):
         from nuitka.optimizations.OptimizeBuiltinCalls import computeBuiltinCall
 
         # Anything may happen. On next pass, if replaced, we might be better
         # but not now.
-        constraint_collection.onExceptionRaiseExit(BaseException)
+        trace_collection.onExceptionRaiseExit(BaseException)
 
         new_node, tags, message = computeBuiltinCall(
-            call_node = call_node,
-            called    = self
+            builtin_name = self.builtin_name,
+            call_node    = call_node
         )
 
         if self.builtin_name in ("dir", "eval", "exec", "execfile", "locals", "vars"):
             # Just inform the collection that all has escaped.
-            constraint_collection.onLocalsUsage()
+            trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
         return new_node, tags, message
 
@@ -136,23 +159,11 @@ Built-in constant '%s' resolved.""" % self.builtin_name
         return None
 
 
-class ExpressionBuiltinOriginalRef(ExpressionBuiltinRef):
-    kind = "EXPRESSION_BUILTIN_ORIGINAL_REF"
-
-    def isCompileTimeConstant(self):
-        # TODO: Actually the base class should not be constant and this
-        # one should be.
-        return False
-
-    def computeExpression(self, constraint_collection):
-
-        # Needs whole program analysis, we don't really know much about it.
-        return self, None, None
-
-
 class ExpressionBuiltinAnonymousRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_ANONYMOUS_REF"
 
+    __slots__ = ()
+
     def __init__(self, builtin_name, source_ref):
         assert builtin_name in builtin_anon_names, builtin_name
 
@@ -166,9 +177,9 @@ class ExpressionBuiltinAnonymousRef(ExpressionBuiltinRefBase):
         return True
 
     def getCompileTimeConstant(self):
-        return builtin_anon_names[ self.builtin_name ]
+        return builtin_anon_names[self.builtin_name]
 
-    def computeExpression(self, constraint_collection):
+    def computeExpressionRaw(self, trace_collection):
         return self, None, None
 
     def getStringValue(self):
@@ -178,8 +189,10 @@ class ExpressionBuiltinAnonymousRef(ExpressionBuiltinRefBase):
 class ExpressionBuiltinExceptionRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_EXCEPTION_REF"
 
+    __slots__ = ()
+
     def __init__(self, exception_name, source_ref):
-        assert exception_name in builtin_exception_names
+        assert exception_name in builtin_exception_names, exception_name
 
         ExpressionBuiltinRefBase.__init__(
             self,
@@ -203,17 +216,17 @@ class ExpressionBuiltinExceptionRef(ExpressionBuiltinRefBase):
     def getCompileTimeConstant(self):
         return builtin_exception_values[self.builtin_name]
 
-    def computeExpression(self, constraint_collection):
+    def computeExpressionRaw(self, trace_collection):
         # Not much that can be done here.
         return self, None, None
 
     def computeExpressionCall(self, call_node, call_args, call_kw,
-                              constraint_collection):
+                              trace_collection):
         exception_name = self.getExceptionName()
 
         # TODO: Keyword only arguments of it, are not properly handled yet by
         # the built-in call code.
-        if exception_name == "ImportError" and python_version >= 330:
+        if exception_name == "ImportError" and python_version >= 300:
             if call_kw is not None and \
                (not call_kw.isExpressionConstantRef() or call_kw.getConstant() != {}):
                 return call_node, None, None
@@ -225,10 +238,10 @@ class ExpressionBuiltinExceptionRef(ExpressionBuiltinRefBase):
                 source_ref     = source_ref
             )
 
-        new_node = BuiltinOptimization.extractBuiltinArgs(
+        new_node = BuiltinParameterSpecs.extractBuiltinArgs(
             node          = call_node,
             builtin_class = createBuiltinMakeException,
-            builtin_spec  = BuiltinOptimization.makeBuiltinParameterSpec(
+            builtin_spec  = BuiltinParameterSpecs.makeBuiltinExceptionParameterSpec(
                 exception_name = exception_name
             )
         )
