@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,10 +22,8 @@ whose implementation lives here. The creation itself also lives here.
 
 """
 
-from .Checkers import checkStatementsSequenceOrNone
 from .ExpressionBases import ExpressionChildrenHavingBase
 from .FunctionNodes import ExpressionFunctionEntryPointBase
-from .IndicatorMixins import MarkLocalsDictIndicatorMixin
 
 
 class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
@@ -37,7 +35,7 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
 
     getCoroutineRef = ExpressionChildrenHavingBase.childGetter("coroutine_ref")
 
-    def __init__(self, coroutine_ref, code_object, source_ref):
+    def __init__(self, coroutine_ref, source_ref):
         assert coroutine_ref.getFunctionBody().isExpressionCoroutineObjectBody()
 
         ExpressionChildrenHavingBase.__init__(
@@ -48,18 +46,8 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
-        self.code_object = code_object
-
         self.variable_closure_traces = None
 
-    def getDetails(self):
-        return {
-            "code_object" : self.code_object
-        }
-
-    def getCodeObject(self):
-        return self.code_object
-
     def getDetailsForDisplay(self):
         return {
             "coroutine" : self.getCoroutineRef().getFunctionBody().getCodeName()
@@ -72,7 +60,9 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
             trace.addClosureUsage()
 
-            self.variable_closure_traces.append(trace)
+            self.variable_closure_traces.append(
+                (closure_variable, trace)
+            )
 
         # TODO: Coroutine body may know something too.
         return self, None, None
@@ -84,40 +74,25 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
         return False
 
     def getClosureVariableVersions(self):
-        return [
-            (trace.getVariable(), trace.getVersion())
-            for trace in self.variable_closure_traces
-        ]
+        return self.variable_closure_traces
 
 
-class ExpressionCoroutineObjectBody(MarkLocalsDictIndicatorMixin, ExpressionFunctionEntryPointBase):
-    # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, pylint: disable=R0901
+class ExpressionCoroutineObjectBody(ExpressionFunctionEntryPointBase):
     kind = "EXPRESSION_COROUTINE_OBJECT_BODY"
 
-    named_children = (
-        "body",
-    )
-
-    checkers = {
-        # TODO: Is "None" really an allowed value.
-        "body" : checkStatementsSequenceOrNone
-    }
-
     qualname_setup = None
 
-    def __init__(self, provider, name, flags, source_ref):
+    def __init__(self, provider, name, code_object, flags, source_ref):
         ExpressionFunctionEntryPointBase.__init__(
             self,
             provider    = provider,
             name        = name,
+            code_object = code_object,
             code_prefix = "coroutine",
             flags       = flags,
             source_ref  = source_ref
         )
 
-        MarkLocalsDictIndicatorMixin.__init__(self)
-
         self.needs_generator_return_exit = False
 
     def getFunctionName(self):
@@ -160,14 +135,23 @@ class ExpressionAsyncWait(ExpressionChildrenHavingBase):
 
         self.exception_preserving = False
 
-    def markAsExceptionPreserving(self):
-        self.exception_preserving = True
-
-    def isExceptionPreserving(self):
-        return self.exception_preserving
+    @staticmethod
+    def isExpressionAsyncWait():
+        return True
 
     def computeExpression(self, trace_collection):
         # TODO: Might be predictable based awaitable analysis or for constants.
+
+        trace_collection.onExceptionRaiseExit(BaseException)
+
         return self, None, None
 
     getValue = ExpressionChildrenHavingBase.childGetter("expression")
+
+
+class ExpressionAsyncWaitEnter(ExpressionAsyncWait):
+    kind = "EXPRESSION_ASYNC_WAIT_ENTER"
+
+
+class ExpressionAsyncWaitExit(ExpressionAsyncWait):
+    kind = "EXPRESSION_ASYNC_WAIT_EXIT"
