@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,8 +30,12 @@ classes.
 """
 
 from nuitka import Options, Variables
-from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
 from nuitka.PythonVersions import python_version
+from nuitka.specs.ParameterSpecs import (
+    ParameterSpec,
+    TooManyArguments,
+    matchCall
+)
 from nuitka.tree.Extractions import updateVariableUsage
 
 from .Checkers import checkStatementsSequenceOrNone
@@ -39,15 +43,16 @@ from .CodeObjectSpecs import CodeObjectSpec
 from .ExpressionBases import (
     CompileTimeConstantExpressionBase,
     ExpressionBase,
+    ExpressionChildHavingBase,
     ExpressionChildrenHavingBase
 )
 from .FutureSpecs import fromFlags
 from .IndicatorMixins import (
-    MarkLocalsDictIndicatorMixin,
+    EntryPointMixin,
     MarkUnoptimizedFunctionIndicatorMixin
 )
+from .LocalsScopes import getLocalsDictHandle, setLocalsDictType
 from .NodeBases import (
-    ChildrenHavingMixin,
     ClosureGiverNodeMixin,
     ClosureTakerMixin,
     SideEffectsFromChildrenMixin
@@ -57,22 +62,26 @@ from .NodeMakingHelpers import (
     makeRaiseExceptionReplacementExpressionFromInstance,
     wrapExpressionWithSideEffects
 )
-from .ParameterSpecs import ParameterSpec, TooManyArguments, matchCall
 
 
 class MaybeLocalVariableUsage(Exception):
     pass
 
-class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
-                                 ExpressionChildrenHavingBase):
 
-    def __init__(self, provider, name, code_prefix, flags, source_ref,
-                 body = None):
-        ExpressionChildrenHavingBase.__init__(
+class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
+                                 ExpressionChildHavingBase):
+
+    named_child = "body"
+
+    checker = checkStatementsSequenceOrNone
+
+    def __init__(self, provider, name, body, code_prefix, flags, source_ref):
+        while provider.isExpressionOutlineBody():
+            provider = provider.getParentVariableProvider()
+
+        ExpressionChildHavingBase.__init__(
             self,
-            values     = {
-                "body" : body # Might be None initially in some cases.
-            },
+            value      = body, # Might be None initially in some cases.
             source_ref = source_ref
         )
 
@@ -103,15 +112,20 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         # Non-local declarations.
         self.non_local_declarations = []
 
-        # Register ourselves immediately with the module.
-        provider.getParentModule().addFunction(self)
-
-        self.trace_collection = None
-
     @staticmethod
     def isExpressionFunctionBodyBase():
         return True
 
+    def getEntryPoint(self):
+        """ Entry point for code.
+
+            Normally ourselves. Only outlines will refer to their parent which
+            technically owns them.
+
+        """
+
+        return self
+
     def getContainingClassDictCreation(self):
         current = self
 
@@ -143,11 +157,6 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
 
         return False
 
-    def isLocalsUpdatedMode(self):
-        return python_version >= 300 or \
-               self.isEarlyClosure() or \
-               self.isUnoptimized()
-
     def hasVariableName(self, variable_name):
         return variable_name in self.providing or variable_name in self.temp_variables
 
@@ -179,13 +188,27 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
             if variable.getOwner() is self
         )
 
+    def getOutlineLocalVariables(self):
+        outlines = self.getTraceCollection().getOutlineFunctions()
+
+        if outlines is None:
+            return ()
+
+        result = []
+
+        for outline in outlines:
+            result.extend(outline.getUserLocalVariables())
+
+        return tuple(result)
+
     def removeClosureVariable(self, variable):
-        assert variable in self.providing.values(), (self.providing, variable)
+        # Do not remove parameter variables of ours.
+        assert not variable.isParameterVariable() or variable.getOwner() is not self
 
-        del self.providing[variable.getName()]
+        variable_name = variable.getName()
 
-        assert not variable.isParameterVariable() or \
-               variable.getOwner() is not self
+        if variable_name in self.providing:
+            del self.providing[variable.getName()]
 
         self.taken.remove(variable)
 
@@ -213,8 +236,11 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
             new_variable = new_variable
         )
 
+    def hasClosureVariable(self, variable):
+        return variable in self.taken
+
     def removeUserVariable(self, variable):
-        assert variable in self.providing.values(), (self.providing, variable)
+        assert variable in self.providing.values(), (self, self.providing, variable)
 
         del self.providing[variable.getName()]
 
@@ -232,7 +258,7 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         return result
 
     def getVariableForReference(self, variable_name):
-        # print( "REF func", self.getCodeName(), variable_name )
+        # print( "REF func", self, variable_name )
 
         if self.hasProvidedVariable(variable_name):
             result = self.getProvidedVariable(variable_name)
@@ -246,12 +272,16 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
             if not result.isModuleVariable():
                 self.registerProvidedVariable(result)
 
+            entry_point = self.getEntryPoint()
+
             # For "exec" containing/star import containing, we raise this exception to indicate
             # that instead of merely a variable, to be assigned, we need to replace with locals
             # dict access.
-            if python_version < 300 and not self.isExpressionClassBody() and \
+            if python_version < 300 and \
+               not entry_point.isExpressionClassBody() and \
+               not entry_point.isPythonMainModule() and \
                result.isModuleVariable() and \
-               self.isUnoptimized():
+               entry_point.isUnoptimized():
                 raise MaybeLocalVariableUsage
 
         return result
@@ -325,13 +355,85 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         else:
             return self.getBody().mayRaiseException(exception_type)
 
+    getBody = ExpressionChildHavingBase.childGetter("body")
+    setBody = ExpressionChildHavingBase.childSetter("body")
 
-class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
-                             MarkUnoptimizedFunctionIndicatorMixin,
-                             ExpressionFunctionBodyBase):
-    # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, leading to many methods, pylint: disable=R0901
 
+class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBase):
+    def __init__(self, provider, name, code_object, code_prefix, flags,
+                 source_ref):
+        ExpressionFunctionBodyBase.__init__(
+            self,
+            provider    = provider,
+            name        = name,
+            code_prefix = code_prefix,
+            flags       = flags,
+            body        = None,
+            source_ref  = source_ref
+        )
+
+        EntryPointMixin.__init__(self)
+
+        self.code_object = code_object
+
+        provider.getParentModule().addFunction(self)
+
+        if "has_exec" in flags or python_version >= 300:
+            self.locals_dict_name = "locals_%s" % (
+                self.getCodeName(),
+            )
+
+            if "has_exec" in flags:
+                setLocalsDictType(self.locals_dict_name, "python2_function_exec")
+            else:
+                setLocalsDictType(self.locals_dict_name, "python3_function")
+
+        else:
+            # TODO: There should be a locals scope for non-dict/mapping too.
+            self.locals_dict_name = None
+
+    def getFunctionLocalsScope(self):
+        if self.locals_dict_name is None:
+            return None
+        else:
+            return getLocalsDictHandle(self.locals_dict_name)
+
+    def getCodeObject(self):
+        return self.code_object
+
+    def computeFunctionRaw(self, trace_collection):
+        from nuitka.optimizations.TraceCollections import \
+            TraceCollectionFunction
+
+        trace_collection = TraceCollectionFunction(
+            parent        = trace_collection,
+            function_body = self
+        )
+        old_collection = self.setTraceCollection(trace_collection)
+
+        self.computeFunction(trace_collection)
+
+        trace_collection.updateVariablesFromCollection(old_collection)
+
+    def computeFunction(self, trace_collection):
+        statements_sequence = self.getBody()
+
+        if statements_sequence is not None and \
+           not statements_sequence.getStatements():
+            statements_sequence.setStatements(None)
+            statements_sequence = None
+
+        if statements_sequence is not None:
+            result = statements_sequence.computeStatementsSequence(
+                trace_collection = trace_collection
+            )
+
+            if result is not statements_sequence:
+                self.setBody(result)
+
+
+class ExpressionFunctionBody(MarkUnoptimizedFunctionIndicatorMixin,
+                             ExpressionFunctionEntryPointBase):
     kind = "EXPRESSION_FUNCTION_BODY"
 
     named_children = (
@@ -346,26 +448,18 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
     if python_version >= 340:
         qualname_setup = None
 
-    def __init__(self, provider, name, doc, parameters, flags, source_ref,
-                 body = None):
-        while provider.isExpressionOutlineBody():
-            provider = provider.getParentVariableProvider()
-
-        if name == "<listcontraction>":
-            assert python_version >= 300
-
-        ExpressionFunctionBodyBase.__init__(
+    def __init__(self, provider, name, code_object, doc, parameters, flags,
+                 source_ref):
+        ExpressionFunctionEntryPointBase.__init__(
             self,
             provider    = provider,
             name        = name,
+            code_object = code_object,
             code_prefix = "function",
             flags       = flags,
-            body        = body,
             source_ref  = source_ref
         )
 
-        MarkLocalsDictIndicatorMixin.__init__(self)
-
         MarkUnoptimizedFunctionIndicatorMixin.__init__(self, flags)
 
         self.doc = doc
@@ -390,11 +484,13 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
 
     def getDetails(self):
         return {
-            "name"       : self.getFunctionName(),
-            "ref_name"   : self.getCodeName(),
-            "parameters" : self.getParameters(),
-            "provider"   : self.provider.getCodeName(),
-            "doc"        : self.doc
+            "name"        : self.getFunctionName(),
+            "ref_name"    : self.getCodeName(),
+            "parameters"  : self.getParameters(),
+            "code_object" : self.code_object,
+            "provider"    : self.provider.getCodeName(),
+            "doc"         : self.doc,
+            "flags"       : self.flags
         }
 
     def getDetailsForDisplay(self):
@@ -406,6 +502,9 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
 
         result.update(self.parameters.getDetails())
 
+        if self.code_object:
+            result.update(self.code_object.getDetails())
+
         if self.doc is not None:
             result["doc"] = self.doc
 
@@ -416,17 +515,21 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
         assert provider is not None
 
         parameter_spec_args = {}
+        code_object_args = {}
         other_args = {}
 
         for key, value in args.items():
             if key.startswith("ps_"):
                 parameter_spec_args[key] = value
+            elif key.startswith("co_"):
+                code_object_args[key] = value
             elif key == "code_flags":
-                source_ref.future_spec = fromFlags(value)
+                code_object_args["future_spec"] = fromFlags(args["code_flags"])
             else:
                 other_args[key] = value
 
         parameters = ParameterSpec(**parameter_spec_args)
+        code_object = CodeObjectSpec(**code_object_args)
 
         # The empty doc string and no doc string are distinguished by presence. The
         # most common case is going to be not present.
@@ -434,9 +537,10 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
             other_args["doc"] = None
 
         return cls(
-            provider   = provider,
-            parameters = parameters,
-            source_ref = source_ref,
+            provider    = provider,
+            parameters  = parameters,
+            code_object = code_object,
+            source_ref  = source_ref,
             **other_args
         )
 
@@ -452,9 +556,6 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
     def getParameters(self):
         return self.parameters
 
-    getBody = ChildrenHavingMixin.childGetter("body")
-    setBody = ChildrenHavingMixin.childSetter("body")
-
     def needsCreation(self):
         return self.needs_creation
 
@@ -499,6 +600,7 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
     def needsExceptionReturnValue(self):
         return self.return_exception
 
+
 def convertNoneConstantOrEmptyDictToNone(node):
     if node is None:
         return None
@@ -536,7 +638,7 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
         "kw_defaults" : convertNoneConstantOrEmptyDictToNone,
     }
 
-    def __init__(self, function_ref, code_object, defaults, kw_defaults,
+    def __init__(self, function_ref, defaults, kw_defaults,
                  annotations, source_ref):
         assert kw_defaults is None or kw_defaults.isExpression()
         assert annotations is None or annotations.isExpression()
@@ -553,98 +655,21 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
             source_ref = source_ref
         )
 
-        self.code_object = code_object
-
         self.variable_closure_traces = None
 
     def getName(self):
         return self.getFunctionRef().getName()
 
-    def getCodeObject(self):
-        return self.code_object
-
-    def getDetails(self):
-        return {
-            "code_object" : self.code_object
-        }
-
-    def getDetailsForDisplay(self):
-        if self.code_object:
-            return self.code_object.getDetails()
-        else:
-            return {}
-
-    @classmethod
-    def fromXML(cls, provider, source_ref, **args):
-        code_object_args = {}
-        other_args = {}
-
-        for key, value in args.items():
-            if key.startswith("co_"):
-                code_object_args[key] = value
-            elif key == "code_flags":
-                code_object_args["future_spec"] = fromFlags(args["code_flags"])
-            else:
-                other_args[key] = value
-
-        if code_object_args:
-            code_object = CodeObjectSpec(**code_object_args)
-        else:
-            code_object = None
-
-        return cls(
-            code_object = code_object,
-            source_ref  = source_ref,
-            **other_args
-        )
-
     def computeExpression(self, trace_collection):
-        defaults = self.getDefaults()
-
-        side_effects = []
-
-        for default in defaults:
-            if default.willRaiseException(BaseException):
-                result = wrapExpressionWithSideEffects(
-                    side_effects = side_effects,
-                    old_node     = self,
-                    new_node     = default
-                )
-
-                return result, "new_raise", "Default value contains raise."
-
-        kw_defaults = self.getKwDefaults()
-
-        if kw_defaults is not None:
-            if kw_defaults.willRaiseException(BaseException):
-                result = wrapExpressionWithSideEffects(
-                    side_effects = side_effects,
-                    old_node     = self,
-                    new_node     = kw_defaults
-                )
-
-                return result, "new_raise", "Keyword default values contain raise."
-
-            side_effects.append(kw_defaults)
-
-        annotations = self.getAnnotations()
-
-        if annotations is not None and annotations.willRaiseException(BaseException):
-            result = wrapExpressionWithSideEffects(
-                side_effects = side_effects,
-                old_node     = self,
-                new_node     = annotations
-            )
-
-            return result, "new_raise", "Annotation values contain raise."
-
         self.variable_closure_traces = []
 
         for closure_variable in self.getFunctionRef().getFunctionBody().getClosureVariables():
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
             trace.addClosureUsage()
 
-            self.variable_closure_traces.append(trace)
+            self.variable_closure_traces.append(
+                (closure_variable, trace)
+            )
 
         # TODO: Function body may know something too.
         return self, None, None
@@ -709,6 +734,7 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
                 star_list_arg = call_spec.getStarListArgumentName(),
                 star_dict_arg = call_spec.getStarDictArgumentName(),
                 num_defaults  = call_spec.getDefaultCount(),
+                num_posonly   = call_spec.getPositionalOnlyCount(),
                 positional    = args_tuple,
                 pairs         = ()
             )
@@ -779,6 +805,8 @@ error""" % self.getName()
         return 20
 
     def createOutlineFromCall(self, provider, values):
+        from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
+
         return convertFunctionCallToOutline(
             provider     = provider,
             function_ref = self.getFunctionRef(),
@@ -786,11 +814,7 @@ error""" % self.getName()
         )
 
     def getClosureVariableVersions(self):
-        return [
-            (trace.getVariable(), trace.getVersion())
-            for trace in self.variable_closure_traces
-        ]
-
+        return self.variable_closure_traces
 
 
 class ExpressionFunctionRef(ExpressionBase):
@@ -810,6 +834,10 @@ class ExpressionFunctionRef(ExpressionBase):
         self.function_body = function_body
         self.code_name = code_name
 
+    def finalize(self):
+        del self.parent
+        del self.function_body
+
     def getName(self):
         return self.function_body.getName()
 
@@ -845,38 +873,10 @@ class ExpressionFunctionRef(ExpressionBase):
         from nuitka.ModuleRegistry import addUsedModule
         addUsedModule(owning_module)
 
-        owning_module.addUsedFunction(function_body)
+        needs_visit = owning_module.addUsedFunction(function_body)
 
-        from nuitka.optimizations.TraceCollections import \
-            TraceCollectionFunction
-
-        # TODO: Doesn't this mean, we can do this multiple times by doing it
-        # in the reference. We should do it in the body, and there we should
-        # limit us to only doing it once per module run, e.g. being based on
-        # presence in used functions of the module already.
-        old_collection = function_body.trace_collection
-
-        function_body.trace_collection = TraceCollectionFunction(
-            parent        = trace_collection,
-            function_body = function_body
-        )
-
-        statements_sequence = function_body.getBody()
-
-        if statements_sequence is not None and \
-           not statements_sequence.getStatements():
-            function_body.setStatements(None)
-            statements_sequence = None
-
-        if statements_sequence is not None:
-            result = statements_sequence.computeStatementsSequence(
-                trace_collection = function_body.trace_collection
-            )
-
-            if result is not statements_sequence:
-                function_body.setBody(result)
-
-        function_body.trace_collection.updateFromCollection(old_collection)
+        if needs_visit:
+            function_body.computeFunctionRaw(trace_collection)
 
         # TODO: Function collection may now know something.
         return self, None, None
@@ -885,6 +885,9 @@ class ExpressionFunctionRef(ExpressionBase):
         # Using a function has no side effects.
         return False
 
+    def mayRaiseException(self, exception_type):
+        return False
+
 
 class ExpressionFunctionCall(ExpressionChildrenHavingBase):
     """ Shared function call.
@@ -946,7 +949,9 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
             trace.addClosureUsage()
 
-            self.variable_closure_traces.append(trace)
+            self.variable_closure_traces.append(
+                (closure_variable, trace)
+            )
 
         return self, None, None
 
@@ -968,16 +973,14 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
     getArgumentValues = ExpressionChildrenHavingBase.childGetter("values")
 
     def getClosureVariableVersions(self):
-        return [
-            (trace.getVariable(), trace.getVersion())
-            for trace in self.variable_closure_traces
-        ]
+        return self.variable_closure_traces
+
 
 # Needed for Python3.3 and higher
 class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionBase):
     kind = "EXPRESSION_FUNCTION_QUALNAME_REF"
 
-    __slots__ = "function_body",
+    __slots__ = ("function_body",)
 
     def __init__(self, function_body, source_ref):
         CompileTimeConstantExpressionBase.__init__(
@@ -987,6 +990,10 @@ class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionBase):
 
         self.function_body = function_body
 
+    def finalize(self):
+        del self.parent
+        del self.function_body
+
     def computeExpressionRaw(self, trace_collection):
         result = makeConstantReplacementNode(
             node     = self,
