@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -41,8 +41,13 @@ from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.importing.Whitelisting import getModuleWhiteList
 from nuitka.utils.FileOperations import relpath
 
-from .ExpressionBases import ExpressionBase, ExpressionChildrenHavingBase
-from .NodeBases import StatementChildrenHavingBase
+from .ExpressionBases import (
+    ExpressionBase,
+    ExpressionChildHavingBase,
+    ExpressionChildrenHavingBase
+)
+from .LocalsScopes import GlobalsDictHandle
+from .NodeBases import StatementChildHavingBase
 from .shapes.BuiltinTypeShapes import ShapeTypeBuiltinModule, ShapeTypeModule
 
 
@@ -55,7 +60,7 @@ class ExpressionImportModuleHard(ExpressionBase):
 
     kind = "EXPRESSION_IMPORT_MODULE_HARD"
 
-    __slots__ = "module_name",
+    __slots__ = ("module_name",)
 
     def __init__(self, module_name, source_ref):
         ExpressionBase.__init__(
@@ -65,6 +70,9 @@ class ExpressionImportModuleHard(ExpressionBase):
 
         self.module_name = module_name
 
+    def finalize(self):
+        del self.parent
+
     def getDetails(self):
         return {
             "module_name" : self.module_name
@@ -107,6 +115,9 @@ class ExpressionImportModuleNameHard(ExpressionBase):
         self.module_name = module_name
         self.import_name = import_name
 
+    def finalize(self):
+        del self.parent
+
     def getDetails(self):
         return {
             "module_name" : self.module_name,
@@ -163,15 +174,15 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
         self.recurse_attempted = False
 
         # The module actually referenced in that import.
-        self.imported_module = None
+        self.imported_module_desc = None
 
         # The fromlist imported modules if any.
-        self.import_list_modules = []
+        self.import_list_modules_desc = []
 
         # For "package.sub_package.module" we also need to import the package,
         # because the imported_module not be found, as it's not a module, e.g.
         # in the case of "os.path" or "six.moves".
-        self.package_modules = None
+        self.package_modules_desc = None
 
         self.finding = None
 
@@ -233,10 +244,10 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
                     warning(
                         """\
 Not recursing to '%(full_path)s' (%(filename)s), please specify \
---recurse-none (do not warn), \
---recurse-all (recurse to all), \
---recurse-not-to=%(full_path)s (ignore it), \
---recurse-to=%(full_path)s (recurse to it) to change.""" % {
+--nofollow-imports (do not warn), \
+--follow-imports (recurse to all), \
+--nofollow-import-to=%(full_path)s (ignore it), \
+--follow-import-to=%(full_path)s (recurse to it) to change.""" % {
                             "full_path" : module_fullpath,
                             "filename"  : module_filename
                         }
@@ -274,13 +285,18 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
         )
 
         if module_filename is not None:
-            self.imported_module = self._consider(
+            imported_module = self._consider(
                 trace_collection = trace_collection,
                 module_filename  = module_filename,
                 module_package   = module_package
             )
 
-            if self.imported_module is not None:
+            if imported_module is not None:
+                self.imported_module_desc = (
+                    imported_module.getFullName(),
+                    imported_module.getFilename()
+                )
+
                 import_list = self.getFromList()
 
                 if import_list is not None:
@@ -291,7 +307,7 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                         import_list = None
 
                 if import_list and \
-                   self.imported_module.isCompiledPythonPackage():
+                   imported_module.isCompiledPythonPackage():
                     for import_item in import_list:
                         if import_item == '*':
                             continue
@@ -299,7 +315,7 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                         module_package, module_filename, _finding = findModule(
                             importing      = self,
                             module_name    = import_item,
-                            parent_package = self.imported_module.getFullName(),
+                            parent_package = imported_module.getFullName(),
                             level          = -1, # Relative import, so child is used.
                             warn           = False
                         )
@@ -312,12 +328,15 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                             )
 
                             if sub_imported_module is not None:
-                                self.import_list_modules.append(
-                                    sub_imported_module.getFullName()
+                                self.import_list_modules_desc.append(
+                                    (
+                                        sub_imported_module.getFullName(),
+                                        sub_imported_module.getFilename()
+                                    )
                                 )
         else:
-            while "." in module_name:
-                module_name = ".".join(module_name.split(".")[:-1])
+            while '.' in module_name:
+                module_name = '.'.join(module_name.split('.')[:-1])
 
                 module_package, module_filename, _finding = findModule(
                     importing      = self,
@@ -335,24 +354,38 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                     )
 
                     if package_module is not None:
-                        if self.package_modules is None:
-                            self.package_modules = []
+                        if self.package_modules_desc is None:
+                            self.package_modules_desc = []
 
-                        self.package_modules.append(package_module)
+                        self.package_modules_desc.append(
+                            (
+                                package_module.getFullName(),
+                                package_module.getFilename()
+                            )
+                        )
 
 
     def _addUsedModules(self, trace_collection):
         if self.finding != "not-found":
-            if self.imported_module is not None:
-                trace_collection.onUsedModule(self.imported_module.getFullName())
+            if self.imported_module_desc is not None:
+                trace_collection.onUsedModule(
+                    module_name    = self.imported_module_desc[0],
+                    module_relpath = self.imported_module_desc[1]
+                )
 
-            for import_list_module in self.import_list_modules:
-                trace_collection.onUsedModule(import_list_module)
+            for import_list_module_desc in self.import_list_modules_desc:
+                trace_collection.onUsedModule(
+                    import_list_module_desc[0],
+                    import_list_module_desc[1]
+                )
 
         # These are added in any case.
-        if self.package_modules is not None:
-            for package_module in self.package_modules:
-                trace_collection.onUsedModule(package_module.getFullName())
+        if self.package_modules_desc is not None:
+            for package_module_desc in self.package_modules_desc:
+                trace_collection.onUsedModule(
+                    package_module_desc[0],
+                    package_module_desc[1]
+                )
 
 
 
@@ -436,25 +469,36 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
         return self.type_shape
 
 
-class StatementImportStar(StatementChildrenHavingBase):
+class StatementImportStar(StatementChildHavingBase):
     kind = "STATEMENT_IMPORT_STAR"
 
-    named_children = ("module",)
+    named_child = "module"
 
-    def __init__(self, module_import, source_ref):
-        StatementChildrenHavingBase.__init__(
+    __slots__ = ("target_scope",)
+
+    def __init__(self, target_scope, module_import, source_ref):
+        StatementChildHavingBase.__init__(
             self,
-            values     = {
-                "module" : module_import
-            },
+            value      = module_import,
             source_ref = source_ref
         )
 
-    getSourceModule = StatementChildrenHavingBase.childGetter("module")
+        self.target_scope = target_scope
+
+        # TODO: Abstract these things.
+        if type(self.target_scope) is GlobalsDictHandle:
+            self.target_scope.markAsEscaped()
+
+    getSourceModule = StatementChildHavingBase.childGetter("module")
+
+    def getTargetDictScope(self):
+        return self.target_scope
 
     def computeStatement(self, trace_collection):
         trace_collection.onExpression(self.getSourceModule())
 
+        trace_collection.onLocalsDictEscaped(self.target_scope)
+
         # Need to invalidate everything, and everything could be assigned to
         # something else now.
         trace_collection.removeAllKnowledge()
@@ -470,33 +514,39 @@ class StatementImportStar(StatementChildrenHavingBase):
         # really can be that way.
         return True
 
+    def getStatementNiceName(self):
+        return "star import statement"
 
-class ExpressionImportName(ExpressionChildrenHavingBase):
+
+class ExpressionImportName(ExpressionChildHavingBase):
     kind = "EXPRESSION_IMPORT_NAME"
 
-    named_children = (
-        "module",
-    )
+    named_child = "module"
 
-    def __init__(self, module, import_name, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+    __slots__ = ("import_name", "level")
+
+    def __init__(self, module, import_name, level, source_ref):
+        ExpressionChildHavingBase.__init__(
             self,
-            values     = {
-                "module" : module
-            },
+            value      = module,
             source_ref = source_ref
         )
 
         self.import_name = import_name
+        self.level = level
 
         assert module is not None
 
     def getImportName(self):
         return self.import_name
 
+    def getImportLevel(self):
+        return self.level
+
     def getDetails(self):
         return {
-            "import_name" : self.getImportName()
+            "import_name" : self.getImportName(),
+            "level"       : self.level
         }
 
     def getDetail(self):
