@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -213,16 +213,19 @@ class ExpressionOutlineFunctionBodyBase(ExpressionFunctionBodyBase):
 
         self.temp_scope = None
 
-    getBody = ExpressionFunctionBodyBase.childGetter("body")
-    setBody = ExpressionFunctionBodyBase.childSetter("body")
-
     def isExpressionOutlineFunctionBodyBase(self):
         return True
 
     def getDetails(self):
         return {
-            "name" : self.name,
-            "provider"   : self.provider.getCodeName()
+            "name"     : self.name,
+            "provider" : self.provider
+        }
+
+    def getDetailsForDisplay(self):
+        return {
+            "name"     : self.name,
+            "provider" : self.provider.getCodeName()
         }
 
     def computeExpressionRaw(self, trace_collection):
@@ -332,18 +335,16 @@ class ExpressionOutlineFunctionBodyBase(ExpressionFunctionBodyBase):
         )
 
 
+# TODO: No other uses of the base class, maybe ExpressionOutlineBody should be
+# moved to be so, or that level should be removed.
 class ExpressionOutlineFunction(ExpressionOutlineFunctionBodyBase):
     kind = "EXPRESSION_OUTLINE_FUNCTION"
 
-    named_children = (
-        "body",
-    )
-
     def isEarlyClosure(self):
         return self.provider.isEarlyClosure()
 
-    def hasLocalsDict(self):
-        return self.provider.hasLocalsDict()
+    def isUnoptimized(self):
+        return self.provider.isUnoptimized()
 
-    def markAsLocalsDict(self):
-        return self.provider.markAsLocalsDict()
+    def getFunctionLocalsScope(self):
+        return self.provider.getFunctionLocalsScope()
