@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,8 +24,8 @@ and its expressions, changing the meaning of course dramatically.
 
 from nuitka import Builtins, Variables
 from nuitka.ModuleRegistry import getOwnerFromCodeName
+from nuitka.PythonVersions import python_version
 
-from .ConstantRefNodes import makeConstantRefNode
 from .DictionaryNodes import (
     ExpressionDictOperationGet,
     ExpressionDictOperationIn,
@@ -34,6 +34,12 @@ from .DictionaryNodes import (
     StatementDictOperationSet
 )
 from .ExpressionBases import ExpressionBase
+from .ModuleAttributeNodes import (
+    ExpressionModuleAttributeLoaderRef,
+    ExpressionModuleAttributeNameRef,
+    ExpressionModuleAttributePackageRef,
+    ExpressionModuleAttributeSpecRef
+)
 from .NodeMakingHelpers import makeRaiseExceptionReplacementExpression
 from .shapes.StandardShapes import ShapeUnknown
 
@@ -45,9 +51,11 @@ class ExpressionVariableNameRef(ExpressionBase):
 
     kind = "EXPRESSION_VARIABLE_NAME_REF"
 
-    __slots__ = ("variable_name",)
+    __slots__ = "variable_name", "provider"
+
+    def __init__(self, provider, variable_name, source_ref):
+        assert not provider.isExpressionOutlineBody(), source_ref
 
-    def __init__(self, variable_name, source_ref):
         ExpressionBase.__init__(
             self,
             source_ref = source_ref
@@ -55,9 +63,20 @@ class ExpressionVariableNameRef(ExpressionBase):
 
         self.variable_name = variable_name
 
+        self.provider = provider
+
+    def finalize(self):
+        del self.parent
+        del self.provider
+
+    @staticmethod
+    def isExpressionVariableNameRef():
+        return True
+
     def getDetails(self):
         return {
-            "variable_name" : self.variable_name
+            "variable_name" : self.variable_name,
+            "provider"      : self.provider
         }
 
     def getVariableName(self):
@@ -66,6 +85,25 @@ class ExpressionVariableNameRef(ExpressionBase):
     def computeExpressionRaw(self, trace_collection):
         return self, None, None
 
+    @staticmethod
+    def needsFallback():
+        return True
+
+
+class ExpressionVariableLocalNameRef(ExpressionVariableNameRef):
+    """ These are used before the actual variable object is known from VariableClosure.
+
+        The special thing about this as opposed to ExpressionVariableNameRef is that
+        these must remain local names and cannot fallback to outside scopes. This is
+        used for __annotations__.
+
+    """
+    kind = "EXPRESSION_VARIABLE_LOCAL_NAME_REF"
+
+    @staticmethod
+    def needsFallback():
+        return False
+
 
 class ExpressionVariableRefBase(ExpressionBase):
     # Base classes can be abstract, pylint: disable=abstract-method
@@ -81,192 +119,60 @@ class ExpressionVariableRefBase(ExpressionBase):
         self.variable = variable
         self.variable_trace = None
 
+    def finalize(self):
+        del self.parent
+        del self.variable
+        del self.variable_trace
+
     def getVariableName(self):
         return self.variable.getName()
 
     def getVariable(self):
         return self.variable
 
-    def getVariableVersion(self):
-        return self.variable_trace.getVersion()
-
-
-class ExpressionVariableRef(ExpressionVariableRefBase):
-    kind = "EXPRESSION_VARIABLE_REF"
-
-    __slots__ = ()
-
-    def __init__(self, variable, source_ref):
-        assert variable is not None
-
-        ExpressionVariableRefBase.__init__(
-            self,
-            variable   = variable,
-            source_ref = source_ref
-        )
-
-    def getDetails(self):
-        return {
-            "variable"      : self.variable
-        }
-
-    def getDetailsForDisplay(self):
-        return {
-            "variable_name" : self.variable.getName(),
-            "owner"         : self.variable.getOwner().getCodeName()
-        }
-
-    @classmethod
-    def fromXML(cls, provider, source_ref, **args):
-        assert cls is ExpressionVariableRef, cls
-
-        owner = getOwnerFromCodeName(args["owner"])
-        variable = owner.getProvidedVariable(args["variable_name"])
-
-        return cls(
-            variable   = variable,
-            source_ref = source_ref
-        )
-
-    def getDetail(self):
-        if self.variable is None:
-            return self.variable_name
-        else:
-            return repr(self.variable)
-
-    @staticmethod
-    def isTargetVariableRef():
-        return False
-
-    def getVariableName(self):
-        return self.variable_name
-
-    def getVariable(self):
-        return self.variable
-
-    def setVariable(self, variable):
-        assert isinstance(variable, Variables.Variable), repr(variable)
-
-        self.variable = variable
+    def getVariableTrace(self):
+        return self.variable_trace
 
     def getTypeShape(self):
-        if self.variable_trace.isAssignTrace():
-            return self.variable_trace.getAssignNode().getAssignSource().getTypeShape()
-        else:
+        if self.variable_trace is None:
             return ShapeUnknown
+        else:
+            return self.variable_trace.getTypeShape()
 
-    def computeExpressionRaw(self, trace_collection):
-        variable = self.variable
-        assert variable is not None
+    def computeExpressionComparisonIn(self, in_node, value_node, trace_collection):
+        tags = None
+        message = None
 
-        self.variable_trace = trace_collection.getVariableCurrentTrace(
-            variable = variable
-        )
+        # Any code could be run, note that.
+        trace_collection.onControlFlowEscape(in_node)
 
-        replacement = self.variable_trace.getReplacementNode(self)
-
-        if replacement is not None:
-            trace_collection.signalChange(
-                "new_expression",
-                self.source_ref,
-                "Value propagated for '%s' from '%s'." % (
-                    variable.getName(),
-                    replacement.getSourceReference().getAsString()
-                )
+        if self.variable_trace.hasShapeDictionaryExact():
+            tags = "new_expression"
+            message = """\
+Check '%s' on dictionary lowered to dictionary '%s'.""" % (
+                in_node.comparator,
+                in_node.comparator
             )
 
-            # Need to compute the replacement still.
-            return replacement.computeExpressionRaw(trace_collection)
-
-        if not self.variable_trace.mustHaveValue():
-            # TODO: This could be way more specific surely.
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
-
-        if variable.isModuleVariable() and \
-           variable.hasDefiniteWrites() is False:
-            variable_name = self.variable.getName()
-
-            if variable_name in Builtins.builtin_exception_names:
-                from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
-
-                new_node = ExpressionBuiltinExceptionRef(
-                    exception_name = self.variable.getName(),
-                    source_ref     = self.getSourceReference()
+            if in_node.comparator == "In":
+                in_node = ExpressionDictOperationIn(
+                    key        = value_node,
+                    dict_arg   = self,
+                    source_ref = in_node.getSourceReference()
                 )
-
-                change_tags = "new_builtin_ref"
-                change_desc = """\
-Module variable '%s' found to be built-in exception reference.""" % (
-                    variable_name
-                )
-            elif variable_name in Builtins.builtin_names and \
-                 variable_name != "pow":
-                from .BuiltinRefNodes import makeExpressionBuiltinRef
-
-                new_node = makeExpressionBuiltinRef(
-                    builtin_name = variable_name,
-                    source_ref   = self.getSourceReference()
-                )
-
-                change_tags = "new_builtin_ref"
-                change_desc = """\
-Module variable '%s' found to be built-in reference.""" % (
-                    variable_name
-                )
-            elif variable_name == "__name__":
-                new_node = makeConstantRefNode(
-                    constant   = variable.getOwner().getParentModule().\
-                                   getFullName(),
-                    source_ref = self.getSourceReference()
-                )
-
-                change_tags = "new_constant"
-                change_desc = """\
-Replaced read-only module attribute '__name__' with constant value."""
             else:
-                self.variable_trace.addUsage()
+                in_node = ExpressionDictOperationNOTIn(
+                    key        = value_node,
+                    dict_arg   = self,
+                    source_ref = in_node.getSourceReference()
+                )
 
-                # Probably should give a warning once about it.
-                new_node = self
-                change_tags = None
-                change_desc = None
 
-            return new_node, change_tags, change_desc
+        # Any exception may be raised.
+        if in_node.mayRaiseException(BaseException):
+            trace_collection.onExceptionRaiseExit(BaseException)
 
-        self.variable_trace.addUsage()
-
-        if self.variable_trace.mustNotHaveValue():
-            assert self.variable.isLocalVariable(), self.variable
-
-            variable_name = self.variable.getName()
-
-            result = makeRaiseExceptionReplacementExpression(
-                expression      = self,
-                exception_type  = "UnboundLocalError",
-                exception_value = """\
-local variable '%s' referenced before assignment""" % variable_name
-            )
-
-            return result, "new_raise", "Variable access of not initialized variable '%s'" % variable_name
-
-        return self, None, None
-
-    def computeExpressionCall(self, call_node, call_args, call_kw,
-                              trace_collection):
-
-        trace_collection.onExceptionRaiseExit(BaseException)
-
-        trace_collection.onControlFlowEscape(self)
-
-        if not Variables.complete and \
-           self.variable.getName() in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
-           self.variable.isModuleVariable():
-            # Just inform the collection that all escaped.
-            trace_collection.onLocalsUsage(self.getParentVariableProvider())
-
-        return call_node, None, None
+        return in_node, tags, message
 
     def computeExpressionSetSubscript(self, set_node, subscript, value_node,
                                       trace_collection):
@@ -348,40 +254,221 @@ Subscript look-up to dictionary lowered to dictionary look-up."""
 
         return lookup_node, tags, message
 
-    def computeExpressionComparisonIn(self, in_node, value_node, trace_collection):
-        tags = None
-        message = None
 
-        # Any code could be run, note that.
-        trace_collection.onControlFlowEscape(in_node)
+_hard_names = ("dir", "eval", "exec", "execfile", "locals", "vars", "super")
 
-        if self.variable_trace.hasShapeDictionaryExact():
-            tags = "new_expression"
-            message = """\
-Check '%s' on dictionary lowered to dictionary '%s'.""" % (
-                in_node.comparator,
-                in_node.comparator
+class ExpressionVariableRef(ExpressionVariableRefBase):
+    kind = "EXPRESSION_VARIABLE_REF"
+
+    __slots__ = ()
+
+    def __init__(self, variable, source_ref):
+        assert variable is not None
+
+        ExpressionVariableRefBase.__init__(
+            self,
+            variable   = variable,
+            source_ref = source_ref
+        )
+
+    def getDetails(self):
+        return {
+            "variable"      : self.variable
+        }
+
+    def getDetailsForDisplay(self):
+        return {
+            "variable_name" : self.variable.getName(),
+            "owner"         : self.variable.getOwner().getCodeName()
+        }
+
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        assert cls is ExpressionVariableRef, cls
+
+        owner = getOwnerFromCodeName(args["owner"])
+        variable = owner.getProvidedVariable(args["variable_name"])
+
+        return cls(
+            variable   = variable,
+            source_ref = source_ref
+        )
+
+    def getDetail(self):
+        if self.variable is None:
+            return self.variable_name
+        else:
+            return repr(self.variable)
+
+    @staticmethod
+    def isTargetVariableRef():
+        return False
+
+    def getVariableName(self):
+        return self.variable_name
+
+    def getVariable(self):
+        return self.variable
+
+    def setVariable(self, variable):
+        assert isinstance(variable, Variables.Variable), repr(variable)
+
+        self.variable = variable
+
+    def computeExpressionRaw(self, trace_collection):
+        # Terribly detailed, pylint: disable=too-many-branches,too-many-statements
+
+        variable = self.variable
+        assert variable is not None
+
+        self.variable_trace = trace_collection.getVariableCurrentTrace(
+            variable = variable
+        )
+
+        replacement = self.variable_trace.getReplacementNode(self)
+
+        if replacement is not None:
+            trace_collection.signalChange(
+                "new_expression",
+                self.source_ref,
+                "Value propagated for '%s' from '%s'." % (
+                    variable.getName(),
+                    replacement.getSourceReference().getAsString()
+                )
             )
 
-            if in_node.comparator == "In":
-                in_node = ExpressionDictOperationIn(
-                    key        = value_node,
-                    dict_arg   = self,
-                    source_ref = in_node.getSourceReference()
+            # Need to compute the replacement still.
+            return replacement.computeExpressionRaw(trace_collection)
+
+        if not self.variable_trace.mustHaveValue():
+            # TODO: This could be way more specific surely.
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
+        if variable.isModuleVariable() and \
+           variable.hasDefiniteWrites() is False:
+            variable_name = self.variable.getName()
+
+            if variable_name in Builtins.builtin_exception_names:
+                if not self.variable.getOwner().getModuleDictScope().isEscaped():
+                    from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
+
+                    new_node = ExpressionBuiltinExceptionRef(
+                        exception_name = self.variable.getName(),
+                        source_ref     = self.getSourceReference()
+                    )
+
+                    change_tags = "new_builtin_ref"
+                    change_desc = """\
+Module variable '%s' found to be built-in exception reference.""" % (
+                        variable_name
+                    )
+                else:
+                    self.variable_trace.addUsage()
+
+                    new_node = self
+                    change_tags = None
+                    change_desc = None
+
+            elif variable_name in Builtins.builtin_names:
+                if variable_name in _hard_names or not self.variable.getOwner().getModuleDictScope().isEscaped():
+                    from .BuiltinRefNodes import makeExpressionBuiltinRef
+
+                    new_node = makeExpressionBuiltinRef(
+                        builtin_name = variable_name,
+                        source_ref   = self.getSourceReference()
+                    )
+
+                    change_tags = "new_builtin_ref"
+                    change_desc = """\
+Module variable '%s' found to be built-in reference.""" % (
+                        variable_name
+                    )
+                else:
+                    self.variable_trace.addUsage()
+
+                    new_node = self
+                    change_tags = None
+                    change_desc = None
+            elif variable_name == "__name__":
+                new_node = ExpressionModuleAttributeNameRef(
+                    variable   = variable,
+                    source_ref = self.getSourceReference()
                 )
+
+                change_tags = "new_expression"
+                change_desc = """\
+Replaced read-only module attribute '__name__' with module attribute reference."""
+            elif variable_name == "__package__":
+                new_node = ExpressionModuleAttributePackageRef(
+                    variable   = variable,
+                    source_ref = self.getSourceReference()
+                )
+
+                change_tags = "new_expression"
+                change_desc = """\
+Replaced read-only module attribute '__package__' with module attribute reference."""
+            elif variable_name == "__loader__" and python_version >= 300:
+                new_node = ExpressionModuleAttributeLoaderRef(
+                    variable   = variable,
+                    source_ref = self.getSourceReference()
+                )
+
+                change_tags = "new_expression"
+                change_desc = """\
+Replaced read-only module attribute '__loader__' with module attribute reference."""
+            elif variable_name == "__spec__" and python_version >= 340:
+                new_node = ExpressionModuleAttributeSpecRef(
+                    variable   = variable,
+                    source_ref = self.getSourceReference()
+                )
+
+                change_tags = "new_expression"
+                change_desc = """\
+Replaced read-only module attribute '__spec__' with module attribute reference."""
             else:
-                in_node = ExpressionDictOperationNOTIn(
-                    key        = value_node,
-                    dict_arg   = self,
-                    source_ref = in_node.getSourceReference()
-                )
+                self.variable_trace.addUsage()
 
+                # Probably should give a warning once about it.
+                new_node = self
+                change_tags = None
+                change_desc = None
 
-        # Any exception may be raised.
-        if in_node.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(BaseException)
+            return new_node, change_tags, change_desc
 
-        return in_node, tags, message
+        self.variable_trace.addUsage()
+
+        if self.variable_trace.mustNotHaveValue():
+            assert self.variable.isLocalVariable(), self.variable
+
+            variable_name = self.variable.getName()
+
+            result = makeRaiseExceptionReplacementExpression(
+                expression      = self,
+                exception_type  = "UnboundLocalError",
+                exception_value = """\
+local variable '%s' referenced before assignment""" % variable_name
+            )
+
+            return result, "new_raise", "Variable access of not initialized variable '%s'" % variable_name
+
+        return self, None, None
+
+    def computeExpressionCall(self, call_node, call_args, call_kw,
+                              trace_collection):
+
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        trace_collection.onControlFlowEscape(self)
+
+        if not Variables.complete and \
+           self.variable.getName() in _hard_names and \
+           self.variable.isModuleVariable():
+            # Just inform the collection that all escaped.
+            trace_collection.onLocalsUsage(self.getParentVariableProvider())
+
+        return call_node, None, None
 
     def hasShapeDictionaryExact(self):
         return self.variable_trace.hasShapeDictionaryExact()
@@ -396,9 +483,8 @@ Check '%s' on dictionary lowered to dictionary '%s'.""" % (
         return not self.variable_trace.mustHaveValue()
 
     def mayRaiseException(self, exception_type):
-        variable_trace = self.variable_trace
-
-        return variable_trace is None or not self.variable_trace.mustHaveValue()
+        return self.variable_trace is None or \
+               not self.variable_trace.mustHaveValue()
 
 
 class ExpressionTempVariableRef(ExpressionVariableRefBase):
@@ -444,14 +530,6 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
     def isTargetVariableRef():
         return False
 
-    def getTypeShape(self):
-        if self.variable_trace is None:
-            return ShapeUnknown
-        elif self.variable_trace.isAssignTrace():
-            return self.variable_trace.getAssignNode().getAssignSource().getTypeShape()
-        else:
-            return ShapeUnknown
-
     def computeExpressionRaw(self, trace_collection):
         self.variable_trace = trace_collection.getVariableCurrentTrace(
             variable = self.variable
@@ -527,126 +605,3 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
     def isKnownToBeIterableAtMax(self, count):
         # TODO: See through the variable current trace.
         return None
-
-
-class ExpressionLocalsVariableRef(ExpressionBase):
-    kind = "EXPRESSION_LOCALS_VARIABLE_REF"
-
-    __slots__ = "variable_name", "fallback_variable", "variable_trace"
-
-    def __init__(self, variable_name, fallback_variable, source_ref):
-        self.variable_name = variable_name
-
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
-
-        self.fallback_variable = fallback_variable
-
-        self.variable_trace = None
-
-    def getDetails(self):
-        return {
-            "variable_name" : self.variable_name,
-        }
-
-    def getVariableName(self):
-        return self.variable_name
-
-    def getFallbackVariable(self):
-        return self.fallback_variable
-
-    def getFallbackVariableVersion(self):
-        return self.variable_trace.getVersion()
-
-    def computeExpressionRaw(self, trace_collection):
-        # TODO: Use dictionary tracings for locals dict
-
-        self.variable_trace = trace_collection.getVariableCurrentTrace(
-            variable = self.fallback_variable
-        )
-
-        if self.fallback_variable.isModuleVariable() and \
-           self.fallback_variable.hasDefiniteWrites() is False:
-            if self.variable_name in Builtins.builtin_exception_names:
-                from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
-
-                new_node = ExpressionBuiltinExceptionRef(
-                    exception_name = self.variable_name,
-                    source_ref     = self.getSourceReference()
-                )
-
-                change_tags = "new_builtin_ref"
-                change_desc = """\
-Module variable '%s' found to be built-in exception reference.""" % (
-                    self.variable_name
-                )
-            elif self.variable_name in Builtins.builtin_names and \
-                 self.variable_name != "pow":
-                from .BuiltinRefNodes import makeExpressionBuiltinRef
-
-                new_node = makeExpressionBuiltinRef(
-                    builtin_name = self.variable_name,
-                    source_ref   = self.getSourceReference()
-                )
-
-                change_tags = "new_builtin_ref"
-                change_desc = """\
-Module variable '%s' found to be built-in reference.""" % (
-                    self.variable_name
-                )
-            elif self.variable_name == "__name__":
-                new_node = makeConstantRefNode(
-                    constant   = self.getParentModule().getFullName(),
-                    source_ref = self.getSourceReference()
-                )
-
-                change_tags = "new_constant"
-                change_desc = """\
-Replaced read-only module attribute '__name__' with constant value."""
-            elif self.variable_name == "__package__":
-                new_node = makeConstantRefNode(
-                    constant   = self.getParentModule().getPackage(),
-                    source_ref = self.getSourceReference()
-                )
-
-                change_tags = "new_constant"
-                change_desc = """\
-Replaced read-only module attribute '__package__' with constant value."""
-            else:
-                # Probably should give a warning once about it.
-                new_node = self
-                change_tags = None
-                change_desc = None
-
-            if new_node is not self:
-                return new_node, change_tags, change_desc
-
-        # TODO: This could be way more specific surely, using dictionary tracing
-        if not self.variable_trace.mustHaveValue():
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
-
-        self.variable_trace.addUsage()
-
-        return self, None, None
-
-    def computeExpressionCall(self, call_node, call_args, call_kw,
-                              trace_collection):
-        trace_collection.onExceptionRaiseExit(BaseException)
-
-        trace_collection.onControlFlowEscape(self)
-
-        if not Variables.complete and \
-           self.variable_name in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
-           self.fallback_variable.isModuleVariable():
-            # Just inform the collection that all escaped.
-            trace_collection.onLocalsUsage(self.getParentVariableProvider())
-
-        return call_node, None, None
-
-
-    def mayRaiseException(self, exception_type):
-        return self.variable_trace is None or not self.variable_trace.mustHaveValue()
