@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,6 +28,7 @@ from logging import debug, info
 
 from nuitka import ModuleRegistry, Options, Variables
 from nuitka.importing import ImportCache
+from nuitka.nodes.LocalsScopes import LocalsDictHandle, getLocalsDictHandles
 from nuitka.plugins.Plugins import Plugins
 from nuitka.Tracing import printLine
 from nuitka.utils import MemoryUsage
@@ -37,6 +38,7 @@ from .BytecodeDemotion import demoteCompiledModuleToBytecode
 from .Tags import TagSet
 
 _progress = Options.isShowProgress()
+_is_verbose = Options.isVerbose()
 
 def _attemptRecursion(module):
     new_modules = module.attemptRecursion()
@@ -58,13 +60,16 @@ def signalChange(tags, source_ref, message):
 
     """
     if message is not None:
-        debug(
-            "{source_ref} : {tags} : {message}".format(
-                source_ref = source_ref.getAsString(),
-                tags       = tags,
-                message    = message() if inspect.isfunction(message) else message
+        # Try hard to not call a delayed evaluation of node descriptions.
+
+        if _is_verbose:
+            debug(
+                "{source_ref} : {tags} : {message}".format(
+                    source_ref = source_ref.getAsString(),
+                    tags       = tags,
+                    message    = message() if inspect.isfunction(message) else message
+                )
             )
-        )
 
     tag_set.onSignal(tags)
 
@@ -134,8 +139,11 @@ def optimizeUncompiledPythonModule(module):
             )
         )
 
-    for used_module_name in module.getUsedModules():
-        used_module = ImportCache.getImportedModuleByName(used_module_name)
+    for used_module_name, used_module_path in module.getUsedModules():
+        used_module = ImportCache.getImportedModuleByNameAndPath(
+            used_module_name,
+            used_module_path
+        )
         ModuleRegistry.addUsedModule(used_module)
 
     package_name = module.getPackage()
@@ -207,6 +215,10 @@ def areEmptyTraces(variable_traces):
                 # them as well.
                 empty = False
                 break
+
+        elif variable_trace.isLoopTrace():
+            empty = False
+            break
         else:
             assert False, variable_trace
 
@@ -243,6 +255,57 @@ def optimizeUnusedClosureVariables(function_body):
     return changed
 
 
+def optimizeLocalsDictsHandles():
+    # Lots of cases, pylint: disable=too-many-branches
+
+    changed = False
+
+    locals_scopes = getLocalsDictHandles()
+
+    for locals_scope_name, locals_scope in locals_scopes.items():
+        # Limit to Python2 classes for now:
+        if type(locals_scope) is not LocalsDictHandle:
+            continue
+
+        if locals_scope.isMarkedForPropagation():
+            locals_scope.finalize()
+
+            del locals_scopes[locals_scope_name]
+
+            assert locals_scope not in locals_scopes
+            continue
+
+        propagate = True
+
+        for variable in locals_scope.variables.values():
+            for variable_trace in variable.traces:
+                if variable_trace.isAssignTrace():
+                    # For assign traces we want the value to not have a side effect,
+                    # then we can push it down the line. TODO: Once temporary
+                    # variables and dictionary building allows for unset values
+                    # remove this
+                    if variable_trace.getAssignNode().getAssignSource().mayHaveSideEffects():
+                        propagate = False
+                        break
+                elif variable_trace.isUninitTrace():
+                    if variable_trace.previous is not None:
+                        propagate = False
+                        break
+                elif variable_trace.isMergeTrace():
+                    propagate = False
+                    break
+                elif variable_trace.isUnknownTrace():
+                    propagate = False
+                    break
+                else:
+                    assert False, (variable, variable_trace)
+
+        if propagate:
+            locals_scope.markForLocalsDictPropagation()
+
+    return changed
+
+
 def optimizeUnusedUserVariables(function_body):
     changed = False
 
@@ -289,30 +352,43 @@ def optimizeUnusedTempVariables(provider):
         return False
 
 
+def optimizeUnusedAssignments(provider):
+    for _trace_collection in provider.getTraceCollections():
+        # TODO: Find things to do here.
+        pass
+
+    return False
+
+
 def optimizeVariables(module):
     changed = False
 
     try:
-        if Variables.complete:
-            try:
-                for function_body in module.getUsedFunctions():
+        try:
+            for function_body in module.getUsedFunctions():
+                if Variables.complete:
                     if optimizeUnusedUserVariables(function_body):
                         changed = True
 
                     if optimizeUnusedClosureVariables(function_body):
                         changed = True
 
-                    if optimizeUnusedTempVariables(function_body):
-                        changed = True
-            except Exception:
-                print("Problem with", function_body)
-                raise#
+                if optimizeUnusedTempVariables(function_body):
+                    changed = True
+        except Exception:
+            print("Problem with", function_body)
+            raise#
 
         if optimizeUnusedUserVariables(module):
             changed = True
 
         if optimizeUnusedTempVariables(module):
             changed = True
+
+#        TODO: Global optimizations could go here maybe, so far we can do all
+#        the things in assign nodes themselves based on last trace.
+#        if optimizeUnusedAssignments(module):
+#            changed = True
     except Exception:
         print("Problem with", module)
         raise
@@ -415,6 +491,10 @@ def makeOptimizationPass(initial_pass):
 
             current_module.setFunctions(used_functions)
 
+    if Variables.complete:
+        if optimizeLocalsDictsHandles():
+            finished = False
+
     return finished
 
 
@@ -455,17 +535,17 @@ def _checkXMLPersistence():
 
 
 
-def optimize():
+def optimize(output_filename):
     Graphs.startGraph()
 
     # First pass.
     if _progress:
         info("PASS 1:")
 
-    makeOptimizationPass(False)
+    makeOptimizationPass(initial_pass = True)
     Variables.complete = True
 
-    finished = makeOptimizationPass(False)
+    finished = makeOptimizationPass(initial_pass = False)
 
     if Options.isExperimental("check_xml_persistence"):
         _checkXMLPersistence()
@@ -482,6 +562,6 @@ def optimize():
 
     # Second, "endless" pass.
     while not finished:
-        finished = makeOptimizationPass(True)
+        finished = makeOptimizationPass(initial_pass = False)
 
-    Graphs.endGraph()
+    Graphs.endGraph(output_filename)
