@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,42 +29,59 @@ from logging import debug
 from nuitka import Tracing, Variables
 from nuitka.__past__ import iterItems  # Python3 compatibility.
 from nuitka.containers.oset import OrderedSet
-from nuitka.importing.ImportCache import (
-    getImportedModuleByName,
-    isImportedModuleByName
-)
+from nuitka.importing.ImportCache import getImportedModuleByNameAndPath
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.nodes.NodeMakingHelpers import getComputationResult
+from nuitka.nodes.shapes.BuiltinTypeShapes import (
+    ShapeTypeDict,
+    ShapeTypeInt,
+    ShapeTypeIntOrLong,
+    ShapeTypeLong
+)
 from nuitka.PythonVersions import python_version
+from nuitka.tree.SourceReading import readSourceLine
+from nuitka.utils.FileOperations import relpath
 from nuitka.utils.InstanceCounters import counted_del, counted_init
 
-from .VariableTraces import (
-    VariableTraceAssign,
-    VariableTraceInit,
-    VariableTraceLoopMerge,
-    VariableTraceMerge,
-    VariableTraceUninit,
-    VariableTraceUnknown
+from .ValueTraces import (
+    ValueTraceAssign,
+    ValueTraceInit,
+    ValueTraceLoopComplete,
+    ValueTraceLoopInitial,
+    ValueTraceMerge,
+    ValueTraceUninit,
+    ValueTraceUnknown
 )
 
 signalChange = None
 
 
 class CollectionTracingMixin(object):
+    """ This contains for logic for maintaining active traces.
+
+        They are kept for "variable" and versions.
+    """
+
     def __init__(self):
-        # For functions, when we are in here, the currently active one,
+        # Currently active values in the tracing.
         self.variable_actives = {}
 
     def getVariableCurrentTrace(self, variable):
+        """ Get the current value trace associated to this variable
+
+            It is also created on the fly if necessary. We create them
+            lazy so to keep the tracing branches minimal where possible.
+        """
+
         return self.getVariableTrace(
             variable = variable,
-            version  = self.getCurrentVariableVersion(variable)
+            version  = self._getCurrentVariableVersion(variable)
         )
 
     def markCurrentVariableTrace(self, variable, version):
         self.variable_actives[variable] = version
 
-    def getCurrentVariableVersion(self, variable):
+    def _getCurrentVariableVersion(self, variable):
         try:
             return self.variable_actives[variable]
         except KeyError:
@@ -90,28 +107,40 @@ class CollectionTracingMixin(object):
             self.addVariableTrace(
                 variable = variable,
                 version  = version,
-                trace    = VariableTraceUnknown(
+                trace    = ValueTraceUnknown(
                     owner    = self.owner,
-                    variable = variable,
-                    version  = version,
                     previous = current
                 )
             )
 
             self.markCurrentVariableTrace(variable, version)
 
-    def markActiveVariableAsLoopMerge(self, variable):
+    def markActiveVariableAsLoopMerge(self, variable, shapes, initial):
         current = self.getVariableCurrentTrace(
             variable = variable,
         )
 
         version = variable.allocateTargetNumber()
 
-        result = VariableTraceLoopMerge(
-            variable = variable,
-            version  = version,
-            previous = current
-        )
+        current_shape = current.getTypeShape()
+
+        if current_shape not in shapes:
+            shapes = set(shapes)
+            shapes.add(current_shape)
+
+        if python_version < 300:
+            if ShapeTypeIntOrLong in shapes:
+                if ShapeTypeInt in shapes:
+                    shapes.discard(ShapeTypeInt)
+                if ShapeTypeLong in shapes:
+                    shapes.discard(ShapeTypeLong)
+
+        # print(initial, shapes)
+
+        if initial:
+            result = ValueTraceLoopInitial(current, shapes)
+        else:
+            result = ValueTraceLoopComplete(current, shapes)
 
         self.addVariableTrace(
             variable = variable,
@@ -125,10 +154,15 @@ class CollectionTracingMixin(object):
 
     def markActiveVariablesAsUnknown(self):
         for variable in self.getActiveVariables():
+            if variable.isTempVariable():
+                continue
+
             self.markActiveVariableAsUnknown(variable)
 
 
 class CollectionStartpointMixin(object):
+    # Many things are traced
+
     def __init__(self):
         # Variable assignments performed in here, last issued number, only used
         # to determine the next number that should be used for a new assignment.
@@ -234,11 +268,7 @@ class CollectionStartpointMixin(object):
     def addVariableMergeMultipleTrace(self, variable, traces):
         version = variable.allocateTargetNumber()
 
-        trace_merge = VariableTraceMerge(
-            variable = variable,
-            version  = version,
-            traces   = traces
-        )
+        trace_merge = ValueTraceMerge(traces)
 
         self.addVariableTrace(variable, version, trace_merge)
 
@@ -260,11 +290,9 @@ class CollectionStartpointMixin(object):
 
         Tracing.printSeparator()
 
-    def _initVariableUnknown(self, variable):
-        trace = VariableTraceUnknown(
+    def initVariableUnknown(self, variable):
+        trace = ValueTraceUnknown(
             owner    = self.owner,
-            variable = variable,
-            version  = 0,
             previous = None
         )
 
@@ -277,11 +305,7 @@ class CollectionStartpointMixin(object):
         return trace
 
     def _initVariableInit(self, variable):
-        trace = VariableTraceInit(
-            owner    = self.owner,
-            variable = variable,
-            version  = 0
-        )
+        trace = ValueTraceInit(self.owner)
 
         self.addVariableTrace(
             variable = variable,
@@ -292,10 +316,8 @@ class CollectionStartpointMixin(object):
         return trace
 
     def _initVariableUninit(self, variable):
-        trace = VariableTraceUninit(
+        trace = ValueTraceUninit(
             owner    = self.owner,
-            variable = variable,
-            version  = 0,
             previous = None
         )
 
@@ -343,14 +365,17 @@ class CollectionStartpointMixin(object):
         elif variable.isLocalVariable():
             result = self._initVariableUninit(variable)
         elif variable.isModuleVariable():
-            result = self._initVariableUnknown(variable)
+            result = self.initVariableUnknown(variable)
         elif variable.isTempVariable():
             result = self._initVariableUninit(variable)
+        elif variable.isLocalsDictVariable():
+            if variable.getOwner().getTypeShape() is ShapeTypeDict:
+                result = self._initVariableUninit(variable)
+            else:
+                result = self.initVariableUnknown(variable)
         else:
             assert False, variable
 
-        assert result.getVariable() is variable
-
         return result
 
     def addOutlineFunction(self, outline):
@@ -362,6 +387,20 @@ class CollectionStartpointMixin(object):
     def getOutlineFunctions(self):
         return self.outline_functions
 
+    def onLocalsDictEscaped(self, locals_scope):
+        # TODO: Limit to the scope.
+        if locals_scope is not None:
+            for variable in locals_scope.variables.values():
+                self.markActiveVariableAsUnknown(variable)
+
+
+        for variable in self.getActiveVariables():
+            if variable.isTempVariable() or variable.isModuleVariable():
+                continue
+
+            self.markActiveVariableAsUnknown(variable)
+
+
 
 class TraceCollectionBase(CollectionTracingMixin):
     __del__ = counted_del()
@@ -378,19 +417,22 @@ class TraceCollectionBase(CollectionTracingMixin):
         self.value_states = {}
 
     def __repr__(self):
-        return "<%s for %s %d>" % (
+        return "<%s for %s at 0x%x>" % (
             self.__class__.__name__,
             self.name,
             id(self)
         )
 
+    def getOwner(self):
+        return self.owner
+
     @staticmethod
     def signalChange(tags, source_ref, message):
         # This is monkey patched from another module.
         signalChange(tags, source_ref, message)
 
-    def onUsedModule(self, module_name):
-        return self.parent.onUsedModule(module_name)
+    def onUsedModule(self, module_name, module_relpath):
+        return self.parent.onUsedModule(module_name, module_relpath)
 
     @staticmethod
     def mustAlias(a, b):
@@ -421,13 +463,9 @@ class TraceCollectionBase(CollectionTracingMixin):
 
                 self.markActiveVariableAsUnknown(variable)
 
-            elif python_version >= 300 or variable.isSharedTechnically() is not False:
-                # print variable
-
-                # TODO: Could be limited to shared variables that are actually
-                # written to. Most of the time, that won't be the case.
-
-                self.markActiveVariableAsUnknown(variable)
+            elif variable.isLocalVariable():
+                if python_version >= 300 and variable.hasWritesOutsideOf(self.owner) is not False:
+                    self.markActiveVariableAsUnknown(variable)
 
     def removeAllKnowledge(self):
         self.markActiveVariablesAsUnknown()
@@ -444,16 +482,10 @@ class TraceCollectionBase(CollectionTracingMixin):
     def addVariableMergeMultipleTrace(self, variable, traces):
         return self.parent.addVariableMergeMultipleTrace(variable, traces)
 
-    def onVariableSet(self, assign_node):
-        version = assign_node.getVariableVersion()
-        variable = assign_node.getVariable()
-
-        # TODO: The variable, version and assign_node are redundant to pass.
-        variable_trace = VariableTraceAssign(
+    def onVariableSet(self, variable, version, assign_node):
+        variable_trace = ValueTraceAssign(
             owner       = self.owner,
             assign_node = assign_node,
-            variable    = variable,
-            version     = version,
             previous    = self.getVariableCurrentTrace(
                 variable = variable
             )
@@ -475,10 +507,8 @@ class TraceCollectionBase(CollectionTracingMixin):
         # remember the delete of the current
         old_trace = self.getVariableCurrentTrace(variable)
 
-        variable_trace = VariableTraceUninit(
+        variable_trace = ValueTraceUninit(
             owner    = self.owner,
-            variable = variable,
-            version  = version,
             previous = old_trace
         )
 
@@ -492,8 +522,11 @@ class TraceCollectionBase(CollectionTracingMixin):
         # Make references point to it.
         self.markCurrentVariableTrace(variable, version)
 
+        return variable_trace
 
     def onLocalsUsage(self, locals_owner):
+        self.onLocalsDictEscaped(locals_owner.getFunctionLocalsScope())
+
         result = []
 
         include_closure = locals_owner.isExpressionFunctionBody() and \
@@ -509,11 +542,10 @@ class TraceCollectionBase(CollectionTracingMixin):
                 )
 
                 variable_trace.addNameUsage()
-
                 result.append(
                     (
                         variable,
-                        variable_trace.getVersion()
+                        variable_trace
                     )
                 )
 
@@ -527,7 +559,9 @@ class TraceCollectionBase(CollectionTracingMixin):
             return None
 
         assert expression.isExpression(), expression
-        assert expression.parent, expression
+
+        parent = expression.parent
+        assert parent, expression
 
         # Now compute this expression, allowing it to replace itself with
         # something else as part of a local peep hole optimization.
@@ -548,13 +582,7 @@ class TraceCollectionBase(CollectionTracingMixin):
             )
 
         if new_node is not expression:
-            expression.replaceWith(new_node)
-
-            if new_node.isExpressionVariableRef() or \
-               new_node.isExpressionTempVariableRef():
-                # Remember the reference for constraint collection.
-                assert new_node.variable_trace.hasDefiniteUsages()
-
+            parent.replaceChild(expression, new_node)
 
         return new_node
 
@@ -576,8 +604,10 @@ class TraceCollectionBase(CollectionTracingMixin):
             return new_statement
         except Exception:
             Tracing.printError(
-                "Problem with statement at %s:" %
-                statement.getSourceReference().getAsString()
+                "Problem with statement at %s:\n-> %s" % (
+                    statement.source_ref.getAsString(),
+                    readSourceLine(statement.source_ref)
+                )
             )
             raise
 
@@ -702,6 +732,9 @@ class TraceCollectionBase(CollectionTracingMixin):
             catch_exceptions = catch_exceptions
         )
 
+    def onLocalsDictEscaped(self, locals_scope):
+        self.parent.onLocalsDictEscaped(locals_scope)
+
     def getCompileTimeComputationResult(self, node, computation, description):
         new_node, change_tags, message = getComputationResult(
             node        = node,
@@ -751,7 +784,7 @@ class TraceCollectionBranch(TraceCollectionBase):
             )
 
             if result is not branch:
-                branch.replaceWith(result)
+                branch.parent.replaceChild(branch, result)
         else:
             self.onExpression(
                 expression = branch
@@ -759,12 +792,14 @@ class TraceCollectionBranch(TraceCollectionBase):
 
     def initVariable(self, variable):
         variable_trace = self.parent.initVariable(variable)
-        assert variable_trace.getVersion() == 0
 
         self.variable_actives[variable] = 0
 
         return variable_trace
 
+    def onLocalsDictEscaped(self, locals_scope):
+        return self.parent.onLocalsDictEscaped(locals_scope)
+
     def dumpTraces(self):
         Tracing.printSeparator()
         self.parent.dumpTraces()
@@ -783,7 +818,6 @@ class TraceCollectionFunction(CollectionStartpointMixin,
                               TraceCollectionBase):
     def __init__(self, parent, function_body):
         assert function_body.isExpressionFunctionBody() or \
-               function_body.isExpressionClassBody() or \
                function_body.isExpressionGeneratorObjectBody() or \
                function_body.isExpressionCoroutineObjectBody() or \
                function_body.isExpressionAsyncgenObjectBody(), function_body
@@ -803,32 +837,49 @@ class TraceCollectionFunction(CollectionStartpointMixin,
                 self.variable_actives[parameter_variable] = 0
 
         for closure_variable in function_body.getClosureVariables():
-            self._initVariableUnknown(closure_variable)
+            self.initVariableUnknown(closure_variable)
             self.variable_actives[closure_variable] = 0
 
+        # TODO: Have special function type for exec functions stuff.
+        locals_scope = function_body.getFunctionLocalsScope()
+
+        if locals_scope is not None:
+            if not locals_scope.isMarkedForPropagation():
+                for locals_dict_variable in locals_scope.variables.values():
+                    self._initVariableUninit(locals_dict_variable)
+            else:
+                function_body.locals_scope = None
+
 
 class TraceCollectionModule(CollectionStartpointMixin,
                             TraceCollectionBase):
     def __init__(self, module):
+        assert module.isCompiledPythonModule(), module
+
         CollectionStartpointMixin.__init__(self)
 
         TraceCollectionBase.__init__(
             self,
             owner  = module,
-            name   = "module",
+            name   = "module:" + module.getFullName(),
             parent = None
         )
 
         self.used_modules = OrderedSet()
 
-    def onUsedModule(self, module_name):
+    def onUsedModule(self, module_name, module_relpath):
         assert type(module_name) is str, module_name
 
-        self.used_modules.add(module_name)
+        # TODO: Make users provide this through a method that has already
+        # done this.
+        module_relpath = relpath(module_relpath)
 
-        if isImportedModuleByName(module_name):
-            module = getImportedModuleByName(module_name)
-            addUsedModule(module)
+        self.used_modules.add(
+            (module_name, module_relpath)
+        )
+
+        module = getImportedModuleByNameAndPath(module_name, module_relpath)
+        addUsedModule(module)
 
     def getUsedModules(self):
         return self.used_modules
