@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -60,9 +60,21 @@ class NuitkaPluginBase(object):
     plugin_name = None
 
     def getPluginOptionBool(self, option_name, default_value):
-        plugin_options = Options.getPluginOptions(self.plugin_name)
+        plugin_options = self.getPluginOptions()
 
-        return plugin_options.get(option_name, default_value)
+        if option_name in plugin_options and "no" + option_name in plugin_options:
+            sys.exit("Error, conflicting options values given.")
+
+        if option_name in plugin_options:
+            return True
+
+        if "no" + option_name in plugin_options:
+            return False
+
+        return default_value
+
+    def getPluginOptions(self):
+        return Options.getPluginOptions(self.plugin_name)
 
     def considerImplicitImports(self, module, signal_change):
         """ Consider module imports.
@@ -74,12 +86,10 @@ class NuitkaPluginBase(object):
             calls, as the "signal_change" part of this API is not to be cared
             about. Most prominently "getImplicitImports()".
         """
-        for full_name in self.getImplicitImports(module):
+        for full_name, required in self.getImplicitImports(module):
             module_name = full_name.split('.')[-1]
             module_package = '.'.join(full_name.split('.')[:-1]) or None
 
-            required = self.isRequiredImplicitImport(module, full_name)
-
             module_filename = self.locateModule(
                 importing      = module,
                 module_name    = module_name,
@@ -170,6 +180,7 @@ class NuitkaPluginBase(object):
         trigger_module = CompiledPythonModule(
             name         = module_name,
             package_name = module.getPackage(),
+            is_top       = False,
             mode         = mode,
             future_spec  = None,
             source_ref   = source_ref
@@ -302,7 +313,7 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return ()
 
-    def suppressBuiltinImportWarning(self, module_name, source_ref):
+    def suppressBuiltinImportWarning(self, module, source_ref):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return False
 
