@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,10 +23,26 @@ import os
 
 from nuitka import Options
 from nuitka.plugins.PluginBase import NuitkaPluginBase
-from nuitka.utils.Utils import isFile, joinpath, normpath
 
 known_data_files = {
-    "nose.core" : ("../usage.txt",),
+    # Key is the package name to trigger it
+    # Value is a tuple of 2 element tuples, thus trailing commas, where
+    # the target path can be specified (None is just default, i.e. the
+    # package directory) and the filename relative to the source package
+    # directory
+
+    "nose.core" : (
+        (None, "usage.txt"),
+    ),
+    "scrapy"    : (
+        (None, "VERSION"),
+    ),
+    "requests"  : (
+        ("certifi", "../certifi/cacert.pem"),
+    ),
+    "importlib_resources" : (
+        (None, "version.txt"),
+    )
 }
 
 class NuitkaPluginDataFileCollector(NuitkaPluginBase):
@@ -38,38 +54,22 @@ class NuitkaPluginDataFileCollector(NuitkaPluginBase):
 
     def considerDataFiles(self, module):
         if module.getFullName() in known_data_files:
-            for filename in known_data_files[module.getFullName()]:
-                source_path = joinpath(
-                    module.getCompileTimeFilename(),
+            for target_dir, filename in known_data_files[module.getFullName()]:
+                source_path = os.path.join(
+                    os.path.dirname(module.getCompileTimeFilename()),
                     filename
                 )
 
-                # so we can ".." out from what is a filename, we have to
-                # normalize the result, or else checking for existance or
-                # opening it will fail.
-                source_path = normpath(source_path)
+                if os.path.isfile(source_path):
+                    if target_dir is None:
+                        target_dir = module.getFullName().replace('.', os.path.sep)
 
-                if isFile(source_path):
                     yield (
                         source_path,
-                        normpath(
-                            joinpath(
-                                module.getFullName().replace('.', os.path.sep),
+                        os.path.normpath(
+                            os.path.join(
+                                target_dir,
                                 filename
                             )
                         )
                     )
-
-
-class NuitkaPluginDetectorDataFileCollector(NuitkaPluginBase):
-    plugin_name = "data-files"
-
-    @staticmethod
-    def isRelevant():
-        return Options.isStandaloneMode()
-
-    def considerDataFiles(self, module):
-        if module.getFullName() in known_data_files:
-            self.warnUnusedPlugin("Data files are not automatically collected.")
-
-        return ()
