@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,16 +32,11 @@ from nuitka.utils.Utils import getOS
 
 
 class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
-    def isRequiredImplicitImport(self, module, full_name):
-        if full_name == "_tkinter":
-            return False
-
-        if module.isPythonShlibModule():
-            if full_name in module.getUsedModules():
-                return False
-
-        return True
+    def __init__(self):
+        NuitkaPluginBase.__init__(self)
 
+        self.pkg_utils_externals = None
+        self.opengl_plugins = None
 
     def getImplicitImports(self, module):
         # Many variables, branches, due to the many cases, pylint: disable=too-many-branches,too-many-statements
@@ -49,16 +44,20 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
 
         if module.isPythonShlibModule():
             for used_module in module.getUsedModules():
-                yield used_module
+                yield used_module[0], False
 
         # TODO: Move this out to some kind of configuration format.
         elements = full_name.split('.')
 
         if elements[0] in ("PyQt4", "PyQt5"):
             if python_version < 300:
-                yield "atexit"
+                yield "atexit", True
 
-            yield "sip"
+            # These are alternatives now:
+            # TODO: One day it should avoid including both.
+            yield "sip", False
+            if elements[0] == "PyQt5":
+                yield "PyQt5.sip", False
 
             child = elements[1] if len(elements) > 1 else None
 
@@ -71,16 +70,16 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                          "QtQuick", "QtWebChannel", "QtWebSockets", "QtX11Extras",
                          "_QOpenGLFunctions_2_0", "_QOpenGLFunctions_2_1",
                          "_QOpenGLFunctions_4_1_Core"):
-                yield elements[0] + ".QtCore"
+                yield elements[0] + ".QtCore", True
 
             if child in ("QtDeclarative", "QtWebKit", "QtXmlPatterns", "QtQml",
                          "QtPrintSupport", "QtWebKitWidgets", "QtMultimedia",
                          "QtMultimediaWidgets", "QtQuick", "QtQuickWidgets",
                          "QtWebSockets"):
-                yield elements[0] + ".QtNetwork"
+                yield elements[0] + ".QtNetwork", True
 
             if child == "QtScriptTools":
-                yield elements[0] + ".QtScript"
+                yield elements[0] + ".QtScript", True
 
             if child in ("QtWidgets", "QtDeclarative", "QtDesigner", "QtHelp",
                          "QtScriptTools", "QtSvg", "QtTest", "QtWebKit",
@@ -88,111 +87,187 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                          "QtMultimediaWidgets", "QtOpenGL", "QtQuick",
                          "QtQuickWidgets", "QtSql", "_QOpenGLFunctions_2_0",
                          "_QOpenGLFunctions_2_1", "_QOpenGLFunctions_4_1_Core"):
-                yield elements[0] + ".QtGui"
+                yield elements[0] + ".QtGui", True
 
             if full_name in ("PyQt5.QtDesigner", "PyQt5.QtHelp", "PyQt5.QtTest",
                              "PyQt5.QtPrintSupport", "PyQt5.QtSvg", "PyQt5.QtOpenGL",
                              "PyQt5.QtWebKitWidgets", "PyQt5.QtMultimediaWidgets",
                              "PyQt5.QtQuickWidgets", "PyQt5.QtSql"):
-                yield "PyQt5.QtWidgets"
+                yield "PyQt5.QtWidgets", True
 
             if full_name in ("PyQt5.QtPrintSupport",):
-                yield "PyQt5.QtSvg"
+                yield "PyQt5.QtSvg", True
 
             if full_name in ("PyQt5.QtWebKitWidgets",):
-                yield "PyQt5.QtWebKit"
-                yield "PyQt5.QtPrintSupport"
+                yield "PyQt5.QtWebKit", True
+                yield "PyQt5.QtPrintSupport", True
 
             if full_name in ("PyQt5.QtMultimediaWidgets",):
-                yield "PyQt5.QtMultimedia"
+                yield "PyQt5.QtMultimedia", True
 
             if full_name in ("PyQt5.QtQuick", "PyQt5.QtQuickWidgets"):
-                yield "PyQt5.QtQml"
+                yield "PyQt5.QtQml", True
 
-            if full_name == "PyQt5.QtQuickWidgets":
-                yield "PyQt5.QtQuick"
+            if full_name in ("PyQt5.QtQuickWidgets", "PyQt5.QtQml"):
+                yield "PyQt5.QtQuick", True
 
+            if full_name == "PyQt5.Qt":
+                yield "PyQt5.QtCore", True
+                yield "PyQt5.QtDBus", True
+                yield "PyQt5.QtGui", True
+                yield "PyQt5.QtNetwork", True
+                yield "PyQt5.QtNetworkAuth", False
+                yield "PyQt5.QtSensors", False
+                yield "PyQt5.QtSerialPort", False
+                yield "PyQt5.QtMultimedia", True
+                yield "PyQt5.QtQml", False
+                yield "PyQt5.QtWidgets", True
+        elif full_name == "sip" and python_version < 300:
+            yield "enum", False
         elif full_name == "PySide.QtDeclarative":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "PySide.QtHelp":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "PySide.QtOpenGL":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "PySide.QtScriptTools":
-            yield "PySide.QtScript"
-            yield "PySide.QtGui"
+            yield "PySide.QtScript", True
+            yield "PySide.QtGui", True
         elif full_name == "PySide.QtSql":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "PySide.QtSvg":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "PySide.QtTest":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "PySide.QtUiTools":
-            yield "PySide.QtGui"
-            yield "PySide.QtXml"
+            yield "PySide.QtGui", True
+            yield "PySide.QtXml", True
         elif full_name == "PySide.QtWebKit":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "PySide.phonon":
-            yield "PySide.QtGui"
+            yield "PySide.QtGui", True
         elif full_name == "lxml.etree":
-            yield "gzip"
-            yield "lxml._elementpath"
+            yield "gzip", True
+            yield "lxml._elementpath", True
         elif full_name == "gtk._gtk":
-            yield "pangocairo"
-            yield "pango"
-            yield "cairo"
-            yield "gio"
-            yield "atk"
+            yield "pangocairo", True
+            yield "pango", True
+            yield "cairo", True
+            yield "gio", True
+            yield "atk", True
         elif full_name == "atk":
-            yield "gobject"
+            yield "gobject", True
         elif full_name == "gtkunixprint":
-            yield "gobject"
-            yield "cairo"
-            yield "gtk"
+            yield "gobject", True
+            yield "cairo", True
+            yield "gtk", True
         elif full_name == "pango":
-            yield "gobject"
+            yield "gobject", True
         elif full_name == "pangocairo":
-            yield "pango"
-            yield "cairo"
+            yield "pango", True
+            yield "cairo", True
         elif full_name == "reportlab.rl_config":
-            yield "reportlab.rl_settings"
+            yield "reportlab.rl_settings", True
         elif full_name == "socket":
-            yield "_socket"
+            yield "_socket", False
         elif full_name == "ctypes":
-            yield "_ctypes"
+            yield "_ctypes", True
         elif full_name == "gi._gi":
-            yield "gi._error"
+            yield "gi._error", True
         elif full_name == "gi._gi_cairo":
-            yield "cairo"
+            yield "cairo", True
         elif full_name == "cairo._cairo":
-            yield "gi._gobject"
+            yield "gi._gobject", False
         elif full_name in ("Tkinter", "tkinter"):
-            yield "_tkinter"
+            yield "_tkinter", False
         elif full_name in ("cryptography.hazmat.bindings._openssl",
                            "cryptography.hazmat.bindings._constant_time",
                            "cryptography.hazmat.bindings._padding"):
-            yield "_cffi_backend"
+            yield "_cffi_backend", True
         elif full_name.startswith("cryptography._Cryptography_cffi_"):
-            yield "_cffi_backend"
+            yield "_cffi_backend", True
         elif full_name == "bcrypt._bcrypt":
-            yield "_cffi_backend"
+            yield "_cffi_backend", True
         elif full_name == "nacl._sodium":
-            yield "_cffi_backend"
+            yield "_cffi_backend", True
         elif full_name == "_dbus_glib_bindings":
-            yield "_dbus_bindings"
+            yield "_dbus_bindings", True
         elif full_name == "_mysql":
-            yield "_mysql_exceptions"
+            yield "_mysql_exceptions", True
         elif full_name == "lxml.objectify":
-            yield "lxml.etree"
+            yield "lxml.etree", True
         elif full_name == "_yaml":
-            yield "yaml"
+            yield "yaml", True
         elif full_name == "apt_inst":
-            yield "apt_pkg"
+            yield "apt_pkg", True
         elif full_name == "PIL._imagingtk":
-            yield "PIL._tkinter_finder"
+            yield "PIL._tkinter_finder", True
+        elif full_name == "pkg_resources.extern":
+            if self.pkg_utils_externals is None:
+                for line in open(module.getCompileTimeFilename()):
+                    if line.startswith("names"):
+                        line = line.split('=')[-1].strip()
+                        parts = line.split(',')
+
+                        self.pkg_utils_externals = [
+                            part.strip("' ")
+                            for part in
+                            parts
+                        ]
+
+                        break
+                else:
+                    self.pkg_utils_externals = ()
+
+            for pkg_util_external in self.pkg_utils_externals:
+                yield "pkg_resources._vendor." + pkg_util_external, False
+        elif full_name == "pkg_resources._vendor.packaging":
+            yield "pkg_resources._vendor.packaging.version", True
+            yield "pkg_resources._vendor.packaging.specifiers", True
+            yield "pkg_resources._vendor.packaging.requirements", True
+        elif full_name == "uvloop.loop":
+            yield "uvloop._noop", True
+        elif full_name == "fitz.fitz":
+            yield "fitz._fitz", True
+        elif full_name == "pandas._libs":
+            yield "pandas._libs.tslibs.np_datetime", False
+            yield "pandas._libs.tslibs.nattype", False
+        elif full_name == "pandas.core.window":
+            yield "pandas._libs.skiplist", False
+        elif full_name == "zmq.backend":
+            yield "zmq.backend.cython", True
+        elif full_name == "OpenGL":
+            if self.opengl_plugins is None:
+                self.opengl_plugins = []
+
+                for line in open(module.getCompileTimeFilename()):
+                    if line.startswith("PlatformPlugin("):
+                        os_part, plugin_name_part = line[15:-1].split(',')
+                        os_part = os_part.strip("' ")
+                        plugin_name_part = plugin_name_part.strip(") '")
+                        plugin_name_part = plugin_name_part[:plugin_name_part.rfind('.')]
+                        if os_part == "nt":
+                            if getOS() == "Windows":
+                                self.opengl_plugins.append(plugin_name_part)
+                        elif os_part.startswith("linux"):
+                            if getOS() == "Linux":
+                                self.opengl_plugins.append(plugin_name_part)
+                        elif os_part.startswith("darwin"):
+                            if getOS() == "MacOS":
+                                self.opengl_plugins.append(plugin_name_part)
+                        elif os_part.startswith(("posix", "osmesa", "egl")):
+                            if getOS() != "Windows":
+                                self.opengl_plugins.append(plugin_name_part)
+                        else:
+                            assert False, os_part
+
+            for opengl_plugin in self.opengl_plugins:
+                yield opengl_plugin, True
+
+
+    # We don't care about line length here, pylint: disable=line-too-long
 
     module_aliases = {
-        # Python2
         "six.moves.builtins" : "__builtin__" if python_version < 300 else "builtins",
         "six.moves.configparser" : "ConfigParser" if python_version < 300 else "configparser",
         "six.moves.copyreg" : "copy_reg" if python_version < 300 else "copyreg",
@@ -243,28 +318,28 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         "requests.packages.urllib3.contrib.ntlmpool" : "urllib3.contrib.ntlmpool",
         "requests.packages.urllib3.contrib.socks" : "urllib3.contrib.socks",
         "requests.packages.urllib3.exceptions" : "urllib3.exceptions",
-        'requests.packages.urllib3._collections' : 'urllib3._collections',
+        "requests.packages.urllib3._collections" : "urllib3._collections",
         "requests.packages.chardet" : "chardet",
         "requests.packages.idna"    : "idna",
-        'requests.packages.urllib3.packages' : 'urllib3.packages',
-        'requests.packages.urllib3.packages.ordered_dict' : 'urllib3.packages.ordered_dict',
-        'requests.packages.urllib3.packages.ssl_match_hostname' : 'urllib3.packages.ssl_match_hostname',
-        'requests.packages.urllib3.packages.ssl_match_hostname._implementation' : 'urllib3.packages.ssl_match_hostname._implementation',
-        'requests.packages.urllib3.connectionpool' : 'urllib3.connectionpool',
-        'requests.packages.urllib3.connection' : 'urllib3.connection',
-        'requests.packages.urllib3.filepost' : 'urllib3.filepost',
-        'requests.packages.urllib3.request' : 'urllib3.request',
-        'requests.packages.urllib3.response' : 'urllib3.response',
-        'requests.packages.urllib3.fields' : 'urllib3.fields',
-        'requests.packages.urllib3.poolmanager' : 'urllib3.poolmanager',
-        'requests.packages.urllib3.util' : 'urllib3.util',
-        'requests.packages.urllib3.util.connection' : 'urllib3.util.connection',
-        'requests.packages.urllib3.util.request' : 'urllib3.util.request',
-        'requests.packages.urllib3.util.response' : 'urllib3.util.response',
-        'requests.packages.urllib3.util.retry' : 'urllib3.util.retry',
-        'requests.packages.urllib3.util.ssl_' : 'urllib3.util.ssl_',
-        'requests.packages.urllib3.util.timeout' : 'urllib3.util.timeout',
-        'requests.packages.urllib3.util.url' : 'urllib3.util.url',
+        "requests.packages.urllib3.packages" : "urllib3.packages",
+        "requests.packages.urllib3.packages.ordered_dict" : "urllib3.packages.ordered_dict",
+        "requests.packages.urllib3.packages.ssl_match_hostname" : "urllib3.packages.ssl_match_hostname",
+        "requests.packages.urllib3.packages.ssl_match_hostname._implementation" : "urllib3.packages.ssl_match_hostname._implementation",
+        "requests.packages.urllib3.connectionpool" : "urllib3.connectionpool",
+        "requests.packages.urllib3.connection" : "urllib3.connection",
+        "requests.packages.urllib3.filepost" : "urllib3.filepost",
+        "requests.packages.urllib3.request" : "urllib3.request",
+        "requests.packages.urllib3.response" : "urllib3.response",
+        "requests.packages.urllib3.fields" : "urllib3.fields",
+        "requests.packages.urllib3.poolmanager" : "urllib3.poolmanager",
+        "requests.packages.urllib3.util" : "urllib3.util",
+        "requests.packages.urllib3.util.connection" : "urllib3.util.connection",
+        "requests.packages.urllib3.util.request" : "urllib3.util.request",
+        "requests.packages.urllib3.util.response" : "urllib3.util.response",
+        "requests.packages.urllib3.util.retry" : "urllib3.util.retry",
+        "requests.packages.urllib3.util.ssl_" : "urllib3.util.ssl_",
+        "requests.packages.urllib3.util.timeout" : "urllib3.util.timeout",
+        "requests.packages.urllib3.util.url" : "urllib3.util.url",
     }
 
     def onModuleSourceCode(self, module_name, source_code):
@@ -293,11 +368,11 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
     def considerExtraDlls(self, dist_dir, module):
         full_name = module.getFullName()
 
-        if getOS() == "Linux" and full_name == "uuid":
+        if full_name == "uuid" and getOS() == "Linux":
             uuid_dll_path = locateDLL("uuid")
             dist_dll_path = os.path.join(dist_dir, os.path.basename(uuid_dll_path))
 
-            shutil.copy(uuid_dll_path, dist_dir)
+            shutil.copy(uuid_dll_path, dist_dll_path)
 
             return (
                 (uuid_dll_path, dist_dll_path, None),
@@ -308,6 +383,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
     unworthy_namespaces = (
         "setuptools",      # Not performance relevant.
         "distutils",       # Not performance relevant.
+        "wheel",           # Not performance relevant.
         "pkg_resources",   # Not performance relevant.
         "numpy.distutils", # Largely unused, and a lot of modules.
         "numpy.f2py",      # Mostly unused, only numpy.distutils import it.
