@@ -23,16 +23,23 @@
 
 from __future__ import print_function
 
+import hashlib
 import os
+import pickle
 import re
 import subprocess
 import sys
 import tempfile
 import time
 
-from nuitka.tools.testing.Common import addToPythonPath, withPythonPathChange
+from nuitka.tools.testing.Common import (
+    addToPythonPath,
+    getTestingCPythonOutputsCacheDir,
+    withPythonPathChange
+)
 from nuitka.tools.testing.OutputComparison import compareOutput
 from nuitka.Tracing import my_print
+from nuitka.utils.Timing import StopWatch
 
 
 def displayOutput(stdout, stderr):
@@ -45,9 +52,6 @@ def displayOutput(stdout, stderr):
         my_print(stderr)
 
 
-
-
-
 def checkNoPermissionError(output):
     # Forms of permission errors.
     for candidate in (b"Permission denied:",
@@ -61,9 +65,85 @@ def checkNoPermissionError(output):
                  b".*(@test|totest|xx|Error 145)", output):
         return False
 
+    # Give those a retry as well.
+    if b"clcache.__main__.CacheLockException" in output:
+        return False
+
     return True
 
 
+def _getCPythonResults(cpython_cmd):
+    stop_watch = StopWatch()
+
+    # Try a coupile of times for permission denied, on Windows it can
+    # be transient.
+    for _i in range(5):
+        stop_watch.start()
+
+        with withPythonPathChange(os.getcwd()):
+            process = subprocess.Popen(
+                args   = cpython_cmd,
+                stdout = subprocess.PIPE,
+                stderr = subprocess.PIPE
+            )
+
+        stdout_cpython, stderr_cpython = process.communicate()
+        exit_cpython = process.returncode
+
+        stop_watch.stop()
+
+        if checkNoPermissionError(stdout_cpython) and \
+           checkNoPermissionError(stderr_cpython):
+            break
+
+        my_print("Retrying CPython due to permission problems after delay.")
+        time.sleep(2)
+
+    cpython_time = stop_watch.delta()
+
+    return cpython_time, stdout_cpython, stderr_cpython, exit_cpython
+
+
+def getCPythonResults(cpython_cmd, cpython_cached):
+    cached = False
+    if cpython_cached:
+        # TODO: Hashing stuff and creating cache filename is duplicate code
+        # and should be shared.
+        hash_input = " -- ".join(cpython_cmd)
+        if str is not bytes:
+            hash_input = hash_input.encode("utf8")
+
+        command_hash = hashlib.md5(hash_input)
+
+        for element in cpython_cmd:
+            if os.path.exists(element):
+                with open(element, "rb") as element_file:
+                    command_hash.update(element_file.read())
+
+        cache_filename = os.path.join(
+            getTestingCPythonOutputsCacheDir(),
+            command_hash.hexdigest()
+        )
+
+        if os.path.exists(cache_filename):
+            with open(cache_filename, "rb") as cache_file:
+                cpython_time, stdout_cpython, stderr_cpython, exit_cpython = \
+                  pickle.load(cache_file)
+                cached = True
+
+    if not cached:
+        cpython_time, stdout_cpython, stderr_cpython, exit_cpython = _getCPythonResults(cpython_cmd)
+
+        if cpython_cached:
+            with open(cache_filename, "wb") as cache_file:
+                pickle.dump(
+                    (cpython_time, stdout_cpython, stderr_cpython, exit_cpython),
+                    cache_file
+                )
+
+    return cpython_time, stdout_cpython, stderr_cpython, exit_cpython
+
+
 def main():
     # Of course many cases to deal with, pylint: disable=too-many-branches,too-many-locals,too-many-statements
     from nuitka.utils.Execution import check_output
@@ -101,9 +181,10 @@ def main():
     timing             = hasArg("timing")
     coverage_mode      = hasArg("coverage")
     original_file      = hasArg("original_file")
+    runtime_file       = hasArg("runtime_file")
     no_warnings        = not hasArg("warnings")
     full_compat        = not hasArg("improved")
-
+    cpython_cached     = hasArg("cpython_cache")
     syntax_errors      = hasArg("syntax_errors")
 
     plugins_enabled = []
@@ -127,6 +208,13 @@ def main():
             recurse_not.append(arg[len("recurse_not:"):])
             del args[count]
 
+    recurse_to = []
+
+    for count, arg in reversed(tuple(enumerate(args))):
+        if arg.startswith("recurse_to:"):
+            recurse_to.append(arg[len("recurse_to:"):])
+            del args[count]
+
     if args:
         sys.exit("Error, non understood mode(s) '%s'," % ','.join(args))
 
@@ -278,6 +366,9 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
     if original_file:
         extra_options.append("--file-reference-choice=original")
 
+    if runtime_file:
+        extra_options.append("--file-reference-choice=runtime")
+
     if full_compat:
         extra_options.append("--full-compat")
 
@@ -301,16 +392,16 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
         addToPythonPath(os.path.dirname(os.path.abspath(filename)))
 
     if keep_python_path or binary_python_path:
-        extra_options.append("--keep-pythonpath")
+        extra_options.append("--execute-with-pythonpath")
 
     if recurse_none:
-        extra_options.append("--recurse-none")
+        extra_options.append("--nofollow-imports")
 
     if recurse_all:
-        extra_options.append("--recurse-all")
+        extra_options.append("--follow-imports")
 
     if recurse_not:
-        extra_options.extend("--recurse-not-to=" + v for v in recurse_not)
+        extra_options.extend("--nofollow-import-to=" + v for v in recurse_not)
 
     if coverage_mode:
         extra_options.append("--must-not-re-execute")
@@ -326,13 +417,13 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
     if not two_step_execution:
         if module_mode:
             nuitka_cmd = nuitka_call + extra_options + \
-              ["--execute", "--module", filename]
+              ["--run", "--module", filename]
         elif standalone_mode:
             nuitka_cmd = nuitka_call + extra_options + \
-              ["--execute", "--standalone", filename]
+              ["--run", "--standalone", filename]
         else:
             nuitka_cmd = nuitka_call + extra_options + \
-              ["--execute", filename]
+              ["--run", filename]
 
         if no_site:
             nuitka_cmd.insert(len(nuitka_cmd) - 1, "--python-flag=-S")
@@ -375,7 +466,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
             exe_filename = exe_filename[:-3]
 
         exe_filename = exe_filename.replace(')', "").replace('(', "")
-        exe_filename += ".exe"
+        exe_filename += (".exe" if os.name == "nt" else ".bin")
 
         nuitka_cmd2 = [
             os.path.join(output_dir, exe_filename)
@@ -387,29 +478,10 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
         my_print("CPython command:", *cpython_cmd)
 
     if comparison_mode:
-        start_time = time.time()
-
-        # Try a coupile of times for permission denied, on Windows it can
-        # be transient.
-        for _i in range(5):
-            with withPythonPathChange(os.getcwd()):
-                process = subprocess.Popen(
-                    args   = cpython_cmd,
-                    stdout = subprocess.PIPE,
-                    stderr = subprocess.PIPE
-                )
-
-            stdout_cpython, stderr_cpython = process.communicate()
-            exit_cpython = process.returncode
-
-            if checkNoPermissionError(stdout_cpython) and \
-               checkNoPermissionError(stderr_cpython):
-                break
-
-            my_print("Retrying CPython due to permission problems after delay.")
-            time.sleep(2)
-
-        cpython_time = time.time() - start_time
+        cpython_time, stdout_cpython, stderr_cpython, exit_cpython = getCPythonResults(
+            cpython_cmd    = cpython_cmd,
+            cpython_cached = cpython_cached
+        )
 
     if comparison_mode and not silent_mode:
         displayOutput(stdout_cpython, stderr_cpython)
@@ -434,7 +506,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
         if trace_command:
             my_print("Going to output directory", os.getcwd())
 
-    start_time = time.time()
+    stop_watch = StopWatch()
+    stop_watch.start()
 
     if not two_step_execution:
         if trace_command:
@@ -512,8 +585,8 @@ Stderr was:
                 exit_nuitka = exit_nuitka1
                 stdout_nuitka, stderr_nuitka = stdout_nuitka1, stderr_nuitka1
 
-
-    nuitka_time = time.time() - start_time
+    stop_watch.stop()
+    nuitka_time = stop_watch.delta()
 
     if not silent_mode:
         displayOutput(stdout_nuitka, stderr_nuitka)
