@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -36,15 +36,23 @@ from nuitka.nodes.AttributeNodes import (
 )
 from nuitka.nodes.BuiltinIteratorNodes import (
     ExpressionBuiltinIter1,
+    ExpressionBuiltinIterForUnpack,
     StatementSpecialUnpackCheck
 )
+from nuitka.nodes.BuiltinLenNodes import ExpressionBuiltinLen
 from nuitka.nodes.BuiltinNextNodes import ExpressionSpecialUnpack
 from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinList
-from nuitka.nodes.ComparisonNodes import ExpressionComparisonIsNOT
-from nuitka.nodes.ConditionalNodes import StatementConditional
+from nuitka.nodes.ComparisonNodes import makeComparisonExpression
+from nuitka.nodes.ConditionalNodes import makeStatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantEllipsisRef
 from nuitka.nodes.ContainerOperationNodes import ExpressionListOperationPop
-from nuitka.nodes.OperatorNodes import makeExpressionOperationBinaryInplace
+from nuitka.nodes.NodeMakingHelpers import (
+    makeRaiseExceptionExpressionFromTemplate
+)
+from nuitka.nodes.OperatorNodes import (
+    makeBinaryOperationNode,
+    makeExpressionOperationBinaryInplace
+)
 from nuitka.nodes.SliceNodes import (
     ExpressionBuiltinSlice,
     ExpressionSliceLookup,
@@ -58,25 +66,25 @@ from nuitka.nodes.SubscriptNodes import (
 )
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
-    ExpressionVariableNameRef,
-    ExpressionVariableRef
+    ExpressionVariableLocalNameRef,
+    ExpressionVariableNameRef
 )
 from nuitka.PythonVersions import python_version
 
-from .Helpers import (
+from .ReformulationImportStatements import getFutureSpec
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .SyntaxErrors import raiseSyntaxError
+from .TreeHelpers import (
+    buildAnnotationNode,
     buildNode,
     getKind,
     makeConstantRefNode,
     makeSequenceCreationOrConstant,
     makeStatementsSequence,
-    makeStatementsSequenceFromStatement,
     makeStatementsSequenceFromStatements,
     makeStatementsSequenceOrStatement,
     mangleName
 )
-from .ReformulationImportStatements import getFutureSpec
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
-from .SyntaxErrors import raiseSyntaxError
 
 
 def buildExtSliceNode(provider, node, source_ref):
@@ -123,10 +131,11 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                                          source_ref):
     # This is using many variable names on purpose, so as to give names to the
     # unpacked detail values, and has many branches due to the many cases
-    # dealt with, pylint: disable=too-many-branches,too-many-locals
+    # dealt with, pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     if kind == "Name":
         return StatementAssignmentVariableName(
+            provider      = provider,
             variable_name = detail,
             source        = source,
             source_ref    = source_ref
@@ -203,15 +212,19 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
         statements = []
 
         for element_index, element in enumerate(detail):
-            element_var = element_vars[element_index]
-
-            if starred_list_var is not None:
-                if element[0] == "Starred":
+            if element[0] == "Starred":
+                if starred_index is not None:
                     raiseSyntaxError(
                         "two starred expressions in assignment",
                         source_ref.atColumnNumber(0)
                     )
 
+                starred_index = element_index
+
+        for element_index, element in enumerate(detail):
+            element_var = element_vars[element_index]
+
+            if starred_list_var is not None:
                 statements.insert(
                     starred_index+1,
                     StatementAssignmentVariable(
@@ -236,14 +249,16 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                                 source_ref = source_ref
                             ),
                             count      = element_index + 1,
-                            expected   = len(detail),
+                            expected   = starred_index or len(detail),
+                            starred    = starred_index is not None,
+
                             source_ref = source_ref
                         ),
                         source_ref = source_ref
                     )
                 )
             else:
-                starred_index = element_index
+                assert starred_index == element_index
                 starred_list_var = element_var
 
                 statements.append(
@@ -271,12 +286,61 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                     source_ref = source_ref
                 )
             )
+        else:
+            statements.insert(
+                starred_index+1,
+                makeStatementConditional(
+                    condition  = makeComparisonExpression(
+                        comparator = "Lt",
+                        left       = ExpressionBuiltinLen(
+                            value      = ExpressionTempVariableRef(
+                                variable   = starred_list_var,
+                                source_ref = source_ref
+                            ),
+                            source_ref = source_ref
+                        ),
+                        right      = makeConstantRefNode(
+                            constant   = len(statements)-starred_index-1,
+                            source_ref = source_ref
+                        ),
+                        source_ref = source_ref
 
+                    ),
+                    yes_branch = makeRaiseExceptionExpressionFromTemplate(
+                        exception_type = "ValueError",
+                        template       = """\
+not enough values to unpack (expected at least %d, got %%d)""" % (len(statements) - 1),
+                        template_args  = makeBinaryOperationNode(
+                            operator   = "Add",
+                            left       = ExpressionBuiltinLen(
+                                value      = ExpressionTempVariableRef(
+                                    variable   = starred_list_var,
+                                    source_ref = source_ref
+                                ),
+                                source_ref = source_ref
+                            ),
+                            right      = makeConstantRefNode(
+                                constant   = starred_index,
+                                source_ref = source_ref
+                            ),
+                            source_ref = source_ref
+                        ),
+                        source_ref     = source_ref
+                    ).asStatement(),
+                    no_branch  = None,
+                    source_ref = source_ref
+                )
+            )
+
+        if python_version >= 370:
+            iter_creation_class = ExpressionBuiltinIterForUnpack
+        else:
+            iter_creation_class = ExpressionBuiltinIter1
 
         statements = [
             StatementAssignmentVariable(
                 variable   = source_iter_var,
-                source     = ExpressionBuiltinIter1(
+                source     = iter_creation_class(
                     value      = source,
                     source_ref = source_ref
                 ),
@@ -366,7 +430,7 @@ def buildAssignmentStatements(provider, node, source, source_ref,
     )
 
     return buildAssignmentStatementsFromDecoded(
-        provider   = temp_provider,
+        provider   = provider,
         kind       = kind,
         detail     = detail,
         source     = source,
@@ -526,6 +590,7 @@ def buildAnnAssignNode(provider, node, source_ref):
     """ Python3.6 annotation assignment.
 
     """
+    # There are many cases to deal with here.
 
     if provider.isCompiledPythonModule() or provider.isExpressionClassBody():
         provider.markAsNeedsAnnotationsDictionary()
@@ -547,7 +612,7 @@ def buildAnnAssignNode(provider, node, source_ref):
         )
 
         # Only name referencing annotations are effective right now.
-        if statements[-1].isStatementAssignmentVariable():
+        if statements[-1].isStatementAssignmentVariableName():
             variable_name = statements[-1].getVariableName()
         else:
             variable_name = None
@@ -567,16 +632,24 @@ def buildAnnAssignNode(provider, node, source_ref):
     # Only annotations for modules and classes are really made, for functions
     # they are ignored like comments.
     if variable_name is not None:
-        if provider.isExpressionFunctionBody():
-            provider.getVariableForAssignment(variable_name)
-        else:
-            annotation = buildNode(provider, node.annotation, source_ref)
+        if provider.isExpressionClassBody() or provider.isCompiledPythonModule():
+            annotation = buildAnnotationNode(provider, node.annotation, source_ref)
+
+            # TODO: As CPython core considers this implementation detail, and it seems
+            # mostly useless to support having this as a closure taken name after a
+            # __del__ on annotations, we might do this except in full compat mode. It
+            # will produce only noise for all annotations in classes otherwise.
+            if python_version < 370:
+                ref_class = ExpressionVariableLocalNameRef
+            else:
+                ref_class = ExpressionVariableNameRef
 
             statements.append(
                 StatementAssignmentSubscript(
-                    expression = ExpressionVariableRef(
-                        variable   = provider.getVariableForAssignment("__annotations__"),
-                        source_ref = source_ref
+                    expression = ref_class(
+                        provider      = provider,
+                        variable_name = "__annotations__",
+                        source_ref    = source_ref
                     ),
                     subscript  = makeConstantRefNode(
                         constant   = variable_name,
@@ -586,6 +659,10 @@ def buildAnnAssignNode(provider, node, source_ref):
                     source_ref = source_ref
                 )
             )
+        else:
+            # Functions.
+            if node.simple:
+                provider.getVariableForAssignment(variable_name)
 
     return makeStatementsSequence(
         statements = statements,
@@ -594,12 +671,13 @@ def buildAnnAssignNode(provider, node, source_ref):
     )
 
 
-def buildDeleteStatementFromDecoded(kind, detail, source_ref):
+def buildDeleteStatementFromDecoded(provider, kind, detail, source_ref):
     if kind in ("Name", "Name_Exception"):
         # Note: Name_Exception is a "del" for exception handlers that doesn't
         # insist on the variable being defined, user code may do it too, and
         # that will be fine, so make that tolerant.
         return StatementDelVariableName(
+            provider      = provider,
             variable_name = detail,
             tolerant      = kind == "Name_Exception",
             source_ref    = source_ref
@@ -649,6 +727,7 @@ def buildDeleteStatementFromDecoded(kind, detail, source_ref):
         for sub_node in detail:
             result.append(
                 buildDeleteStatementFromDecoded(
+                    provider   = provider,
                     kind       = sub_node[0],
                     detail     = sub_node[1],
                     source_ref = source_ref
@@ -682,6 +761,7 @@ def buildDeleteNode(provider, node, source_ref):
 
         statements.append(
             buildDeleteStatementFromDecoded(
+                provider   = provider,
                 kind       = kind,
                 detail     = detail,
                 source_ref = source_ref
@@ -694,11 +774,12 @@ def buildDeleteNode(provider, node, source_ref):
     )
 
 
-def _buildInplaceAssignVariableNode(variable_name, operator, expression,
-                                    source_ref):
+def _buildInplaceAssignVariableNode(provider, variable_name, operator,
+                                    expression, source_ref):
     inplace_node = makeExpressionOperationBinaryInplace(
         operator   = operator,
         left       = ExpressionVariableNameRef(
+            provider      = provider,
             variable_name = variable_name,
             source_ref    = source_ref
         ),
@@ -710,6 +791,7 @@ def _buildInplaceAssignVariableNode(variable_name, operator, expression,
 
     return (
         StatementAssignmentVariableName(
+            provider      = provider,
             variable_name = variable_name,
             source        = inplace_node,
             source_ref    = source_ref
@@ -746,37 +828,18 @@ def _buildInplaceAssignAttributeNode(provider, lookup_source, attribute_name,
         source_ref = source_ref
     )
 
-    # Third, copy it over, if the reference values change, i.e. IsNot is true.
-    copy_back_from_tmp = StatementConditional(
-        condition  = ExpressionComparisonIsNOT(
-            left       = ExpressionTempVariableRef(
-                variable   = tmp_variable1,
-                source_ref = source_ref
-            ),
-            right      = ExpressionTempVariableRef(
+    # Third, copy it back.
+    copy_back_from_tmp = makeTryFinallyStatement(
+        provider   = provider,
+        tried      = StatementAssignmentAttribute(
+            expression     = lookup_source.makeClone(),
+            attribute_name = attribute_name,
+            source         = ExpressionTempVariableRef(
                 variable   = tmp_variable2,
                 source_ref = source_ref
             ),
-            source_ref = source_ref
+            source_ref     = source_ref
         ),
-        yes_branch = makeStatementsSequenceFromStatement(
-            statement = StatementAssignmentAttribute(
-                expression     = lookup_source.makeClone(),
-                attribute_name = attribute_name,
-                source         = ExpressionTempVariableRef(
-                    variable   = tmp_variable2,
-                    source_ref = source_ref
-                ),
-                source_ref     = source_ref
-            )
-        ),
-        no_branch  = None,
-        source_ref = source_ref
-    )
-
-    copy_back_from_tmp = makeTryFinallyStatement(
-        provider   = provider,
-        tried      = copy_back_from_tmp,
         final      = StatementReleaseVariable(
             variable   = tmp_variable2,
             source_ref = source_ref
@@ -1047,6 +1110,7 @@ def buildInplaceAssignNode(provider, node, source_ref):
 
     if kind == "Name":
         statements = _buildInplaceAssignVariableNode(
+            provider      = provider,
             variable_name = detail,
             operator      = operator,
             expression    = expression,
