@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,16 +26,11 @@ from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
-from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
-from nuitka.nodes.BuiltinRefNodes import (
-    ExpressionBuiltinAnonymousRef,
-    ExpressionBuiltinExceptionRef
-)
-from nuitka.nodes.CallNodes import ExpressionCallEmpty
+from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinExceptionRef
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
-    StatementConditional
+    makeStatementConditional
 )
 from nuitka.nodes.ConstantRefNodes import (
     ExpressionConstantNoneRef,
@@ -45,7 +40,6 @@ from nuitka.nodes.ExceptionNodes import StatementRaiseException
 from nuitka.nodes.ExecEvalNodes import StatementExec, StatementLocalsDictSync
 from nuitka.nodes.GlobalsLocalsNodes import ExpressionBuiltinGlobals
 from nuitka.nodes.NodeMakingHelpers import makeExpressionBuiltinLocals
-from nuitka.nodes.TypeNodes import ExpressionBuiltinIsinstance
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
@@ -145,7 +139,7 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
             source     = locals_node,
             source_ref = source_ref,
         ),
-        StatementConditional(
+        makeStatementConditional(
             condition  = ExpressionComparisonIs(
                 left       = ExpressionTempVariableRef(
                     variable   = locals_keeper_variable,
@@ -156,17 +150,15 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
                 ),
                 source_ref = source_ref
             ),
-            yes_branch = makeStatementsSequenceFromStatement(
-                StatementAssignmentVariable(
-                    variable   = locals_keeper_variable,
-                    source     = locals_default,
-                    source_ref = source_ref,
-                )
+            yes_branch = StatementAssignmentVariable(
+                variable   = locals_keeper_variable,
+                source     = locals_default,
+                source_ref = source_ref
             ),
             no_branch  = None,
             source_ref = source_ref
         ),
-        StatementConditional(
+        makeStatementConditional(
             condition  = ExpressionComparisonIs(
                 left       = ExpressionTempVariableRef(
                     variable   = globals_keeper_variable,
@@ -177,14 +169,12 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
                 ),
                 source_ref = source_ref
             ),
-            yes_branch = makeStatementsSequenceFromStatement(
-                StatementAssignmentVariable(
-                    variable   = globals_keeper_variable,
-                    source     = ExpressionBuiltinGlobals(
-                        source_ref = source_ref
-                    ),
-                    source_ref = source_ref,
-                )
+            yes_branch = StatementAssignmentVariable(
+                variable   = globals_keeper_variable,
+                source     = ExpressionBuiltinGlobals(
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
             ),
             no_branch  = None,
             source_ref = source_ref
@@ -222,8 +212,6 @@ def buildExecNode(provider, node, source_ref):
     exec_locals = node.locals
     body = node.body
 
-    orig_globals = exec_globals
-
     # Handle exec(a,b,c) to be same as exec a, b, c
     if exec_locals is None and exec_globals is None and \
        getKind(body) == "Tuple":
@@ -251,10 +239,6 @@ exec: arg 1 must be a string, file, or code object""",
                 source_ref      = source_ref
             )
 
-    if not provider.isCompiledPythonModule():
-        if orig_globals is None:
-            provider.markAsUnqualifiedExecContaining(source_ref)
-
     temp_scope = provider.allocateTempScope("exec")
 
     locals_value  = buildNode(provider, exec_locals, source_ref, True)
@@ -322,7 +306,7 @@ exec: arg 1 must be a string, file, or code object""",
             ),
             source_ref = source_ref
         ),
-        StatementConditional(
+        makeStatementConditional(
             condition  = ExpressionComparisonIs(
                 left       = ExpressionTempVariableRef(
                     variable   = globals_keeper_variable,
@@ -341,7 +325,7 @@ exec: arg 1 must be a string, file, or code object""",
                     ),
                     source_ref = source_ref,
                 ),
-                StatementConditional(
+                makeStatementConditional(
                     condition  = ExpressionComparisonIs(
                         left       = ExpressionTempVariableRef(
                             variable   = locals_keeper_variable,
@@ -375,7 +359,7 @@ exec: arg 1 must be a string, file, or code object""",
                 ),
             ),
             no_branch  = makeStatementsSequenceFromStatements(
-                StatementConditional(
+                makeStatementConditional(
                     condition  = ExpressionComparisonIs(
                         left       = ExpressionTempVariableRef(
                             variable   = locals_keeper_variable,
@@ -402,40 +386,6 @@ exec: arg 1 must be a string, file, or code object""",
             ),
             source_ref = source_ref
         ),
-        # Source needs some special treatment for not done for "eval", if it's a
-        # file object, then  must be read.
-        StatementConditional(
-            condition  = ExpressionBuiltinIsinstance(
-                instance   = ExpressionTempVariableRef(
-                    variable   = source_variable,
-                    source_ref = source_ref
-                ),
-                classes    = ExpressionBuiltinAnonymousRef(
-                    builtin_name = "file",
-                    source_ref   = source_ref,
-                ),
-                source_ref = source_ref
-            ),
-            yes_branch = makeStatementsSequenceFromStatement(
-                statement = StatementAssignmentVariable(
-                    variable   = source_variable,
-                    source     = ExpressionCallEmpty(
-                        called     = ExpressionAttributeLookup(
-                            source         = ExpressionTempVariableRef(
-                                variable   = source_variable,
-                                source_ref = source_ref
-                            ),
-                            attribute_name = "read",
-                            source_ref     = source_ref
-                        ),
-                        source_ref = source_ref
-                    ),
-                    source_ref = source_ref
-                )
-            ),
-            no_branch  = None,
-            source_ref = source_ref
-        ),
         makeTryFinallyStatement(
             provider   = provider,
             tried      = StatementExec(
@@ -453,7 +403,7 @@ exec: arg 1 must be a string, file, or code object""",
                 ),
                 source_ref  = source_ref
             ),
-            final      = StatementConditional(
+            final      = makeStatementConditional(
                 condition  = ExpressionComparisonIs(
                     left       = ExpressionTempVariableRef(
                         variable   = plain_indicator_variable,
@@ -465,14 +415,12 @@ exec: arg 1 must be a string, file, or code object""",
                     ),
                     source_ref = source_ref
                 ),
-                yes_branch = makeStatementsSequenceFromStatement(
-                    statement = StatementLocalsDictSync(
-                        locals_arg = ExpressionTempVariableRef(
-                            variable   = locals_keeper_variable,
-                            source_ref = source_ref,
-                        ),
-                        source_ref = source_ref.atInternal()
-                    )
+                yes_branch = StatementLocalsDictSync(
+                    locals_arg = ExpressionTempVariableRef(
+                        variable   = locals_keeper_variable,
+                        source_ref = source_ref,
+                    ),
+                    source_ref = source_ref
                 ),
                 no_branch  = None,
                 source_ref = source_ref
