@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -33,7 +33,6 @@ from nuitka.nodes.GlobalsLocalsNodes import ExpressionBuiltinGlobals
 from nuitka.nodes.ImportNodes import (
     ExpressionBuiltinImport,
     ExpressionImportModuleHard,
-    ExpressionImportModuleNameHard,
     ExpressionImportName,
     StatementImportStar
 )
@@ -125,6 +124,8 @@ def _enableFutureFeature(node, object_name, source_ref):
     elif object_name in ("nested_scopes", "generators", "with_statement"):
         # These are enabled in all cases already.
         pass
+    elif object_name == "annotations" and python_version >= 370:
+        future_spec.enableFutureAnnotations()
     else:
         raiseSyntaxError(
             "future feature %s is not defined" % object_name,
@@ -135,7 +136,8 @@ def _enableFutureFeature(node, object_name, source_ref):
 def buildImportFromNode(provider, node, source_ref):
     # "from .. import .." statements. This may trigger a star import, or
     # multiple names being looked up from the given module variable name.
-    # This is pretty complex, pylint: disable=too-many-branches,too-many-locals
+    # This is pretty complex.
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     module_name = node.module if node.module is not None else ""
     level = node.level
@@ -147,7 +149,9 @@ def buildImportFromNode(provider, node, source_ref):
         level = None
 
     if level is not None:
-        level = makeConstantRefNode(level, source_ref, True)
+        level_obj = makeConstantRefNode(level, source_ref, True)
+    else:
+        level_obj = None
 
     # Importing from "__future__" module may enable flags to the parser,
     # that we need to know about, handle that.
@@ -197,12 +201,15 @@ def buildImportFromNode(provider, node, source_ref):
             import_locals = makeConstantRefNode({}, source_ref, True)
 
         return StatementImportStar(
+            target_scope  = provider.getModuleDictScope()
+                              if provider.isCompiledPythonModule() else
+                            provider.getFunctionLocalsScope(),
             module_import = ExpressionBuiltinImport(
                 name        = makeConstantRefNode(module_name, source_ref, True),
                 globals_arg = import_globals,
                 locals_arg  = import_locals,
                 fromlist    = makeConstantRefNode(('*',), source_ref, True),
-                level       = level,
+                level       = level_obj,
                 source_ref  = source_ref
             ),
             source_ref    = source_ref
@@ -219,7 +226,7 @@ def buildImportFromNode(provider, node, source_ref):
                 globals_arg = ExpressionBuiltinGlobals(source_ref),
                 locals_arg  = makeConstantRefNode(None, source_ref, True),
                 fromlist    = makeConstantRefNode(tuple(import_names), source_ref, True),
-                level       = level,
+                level       = level_obj,
                 source_ref  = source_ref
             )
 
@@ -259,10 +266,12 @@ def buildImportFromNode(provider, node, source_ref):
 
             import_statements.append(
                 StatementAssignmentVariableName(
+                    provider      = provider,
                     variable_name = mangleName(target_name, provider),
                     source        = ExpressionImportName(
                         module      = imported_from_module,
                         import_name = import_name,
+                        level       = level,
                         source_ref  = source_ref
                     ),
                     source_ref    = source_ref
@@ -333,6 +342,7 @@ def buildImportModulesNode(provider, node, source_ref):
                 import_node = ExpressionImportName(
                     module      = import_node,
                     import_name = import_name,
+                    level       = None,
                     source_ref  = source_ref
                 )
 
@@ -342,6 +352,7 @@ def buildImportModulesNode(provider, node, source_ref):
 
         import_nodes.append(
             StatementAssignmentVariableName(
+                provider      = provider,
                 variable_name = mangleName(
                     local_name
                       if local_name is not None else
