@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -36,10 +36,13 @@ from nuitka.nodes.CallNodes import (
     ExpressionCallNoKeywords
 )
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
-from nuitka.nodes.ConditionalNodes import StatementConditional
+from nuitka.nodes.ConditionalNodes import makeStatementConditional
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
-from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
+from nuitka.nodes.CoroutineNodes import (
+    ExpressionAsyncWaitEnter,
+    ExpressionAsyncWaitExit
+)
 from nuitka.nodes.ExceptionNodes import (
     ExpressionCaughtExceptionTracebackRef,
     ExpressionCaughtExceptionTypeRef,
@@ -50,6 +53,7 @@ from nuitka.nodes.StatementNodes import (
     StatementsSequence
 )
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.YieldNodes import ExpressionYieldFromWaitable
 from nuitka.PythonVersions import python_version
 
 from .ReformulationAssignmentStatements import buildAssignmentStatements
@@ -61,10 +65,8 @@ from .TreeHelpers import (
     buildNode,
     buildStatementsNode,
     getKind,
-    makeConditionalStatement,
     makeReraiseExceptionStatement,
-    makeStatementsSequence,
-    makeStatementsSequenceFromStatement
+    makeStatementsSequence
 )
 
 
@@ -183,16 +185,25 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
 
     # For "async with", await the entered value and exit value must be awaited.
     if not sync:
-        enter_value = ExpressionAsyncWait(
-            expression = enter_value,
+        enter_value = ExpressionYieldFromWaitable(
+            expression = ExpressionAsyncWaitEnter(
+                expression = enter_value,
+                source_ref = source_ref
+            ),
             source_ref = source_ref
         )
-        exit_value_exception = ExpressionAsyncWait(
-            expression = exit_value_exception,
+        exit_value_exception = ExpressionYieldFromWaitable(
+            expression = ExpressionAsyncWaitExit(
+                expression = exit_value_exception,
+                source_ref = source_ref
+            ),
             source_ref = source_ref
         )
-        exit_value_no_exception = ExpressionAsyncWait(
-            expression = exit_value_no_exception,
+        exit_value_no_exception = ExpressionYieldFromWaitable(
+            ExpressionAsyncWaitExit(
+                expression = exit_value_no_exception,
+                source_ref = source_ref
+            ),
             source_ref = source_ref
         )
 
@@ -262,7 +273,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                             ),
                             source_ref = source_ref
                         ),
-                        makeConditionalStatement(
+                        makeStatementConditional(
                             condition  = exit_value_exception,
                             no_branch  = makeReraiseExceptionStatement(
                                 source_ref = with_exit_source_ref
@@ -276,7 +287,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                 public_exc     = python_version >= 270,
                 source_ref     = source_ref
             ),
-            final      = StatementConditional(
+            final      = makeStatementConditional(
                 condition  = ExpressionComparisonIs(
                     left       = ExpressionTempVariableRef(
                         variable   = tmp_indicator_variable,
@@ -288,11 +299,9 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                     ),
                     source_ref = source_ref
                 ),
-                yes_branch = makeStatementsSequenceFromStatement(
-                    statement = StatementExpressionOnly(
-                        expression = exit_value_no_exception,
-                        source_ref = source_ref
-                    )
+                yes_branch = StatementExpressionOnly(
+                    expression = exit_value_no_exception,
+                    source_ref = source_ref
                 ),
                 no_branch  = None,
                 source_ref = source_ref
