@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,7 +25,6 @@ from logging import warning
 from nuitka import Constants, Options, Tracing
 from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
-from nuitka.nodes.ConditionalNodes import StatementConditional
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import (
     ExpressionMakeList,
@@ -36,7 +35,7 @@ from nuitka.nodes.DictionaryNodes import (
     ExpressionKeyValuePair,
     ExpressionMakeDict
 )
-from nuitka.nodes.ExceptionNodes import StatementRaiseException
+from nuitka.nodes.ExceptionNodes import StatementReraiseException
 from nuitka.nodes.FrameNodes import (
     StatementsFrameAsyncgen,
     StatementsFrameCoroutine,
@@ -47,7 +46,6 @@ from nuitka.nodes.FrameNodes import (
 from nuitka.nodes.ImportNodes import ExpressionBuiltinImport
 from nuitka.nodes.NodeBases import NodeBase
 from nuitka.nodes.NodeMakingHelpers import mergeStatements
-from nuitka.nodes.OperatorNodes import ExpressionOperationNOT
 from nuitka.nodes.StatementNodes import StatementsSequence
 from nuitka.PythonVersions import (
     needsSetLiteralReverseInsertion,
@@ -72,7 +70,7 @@ def extractDocFromBody(node):
        getKind(node.body[0]) == "Expr" and \
        getKind(node.body[0].value) == "Str":
 
-        if "no_asserts" not in Options.getPythonFlags():
+        if "no_docstrings" not in Options.getPythonFlags():
             doc = body[0].value.s
 
         body = body[1:]
@@ -105,12 +103,31 @@ def detectFunctionBodyKind(nodes, start_value = None):
 
     flags = set()
 
+    def _checkCoroutine(field):
+        """ Check only for co-routine nature of the field and only update that.
+
+        """
+        # TODO: This is clumsy code, trying to achieve what non-local does for
+        # Python2 as well.
+
+        old = set(indications)
+        indications.clear()
+
+        _check(field)
+
+        if "Coroutine" in indications:
+            old.add("Coroutine")
+
+        indications.clear()
+        indications.update(old)
+        del old
+
     def _check(node):
         node_class = node.__class__
 
         if node_class is ast.Yield:
             indications.add("Generator")
-        elif python_version >= 330 and node_class is ast.YieldFrom:  # @UndefinedVariable
+        elif python_version >= 300 and node_class is ast.YieldFrom:  # @UndefinedVariable
             indications.add("Generator")
         elif python_version >= 350 and node_class in (ast.Await, ast.AsyncWith):  # @UndefinedVariable
             indications.add("Coroutine")
@@ -159,10 +176,24 @@ def detectFunctionBodyKind(nodes, start_value = None):
                     assert False, (name, field, ast.dump(node))
         elif node_class is ast.GeneratorExp:
             for name, field in ast.iter_fields(node):
-                if name in ("name", "body", "comparators", "elt"):
+                if name == "name":
                     pass
+                elif name in ("body", "comparators", "elt"):
+                    if python_version >= 370:
+                        _checkCoroutine(field)
                 elif name == "generators":
                     _check(field[0].iter)
+
+                    # New syntax in 3.7 allows these to be present in functions not
+                    # declared with "async def", so we need to check them, but
+                    # only if top level.
+                    if python_version >= 370 and node in nodes:
+                        for gen in field:
+                            if gen.is_async:
+                                indications.add("Coroutine")
+                                break
+                            elif _checkCoroutine(gen):
+                                break
                 else:
                     assert False, (name, field, ast.dump(node))
         elif node_class is ast.ListComp and python_version >= 300:
@@ -312,6 +343,37 @@ def buildNodeList(provider, nodes, source_ref, allow_none = False):
         return []
 
 
+_host_node = None
+
+def buildAnnotationNode(provider, node, source_ref):
+    if python_version >= 370 and \
+       provider.getParentModule().getFutureSpec().isFutureAnnotations():
+
+        # Using global value for cache, to avoid creating it over and over,
+        # avoiding the pylint: disable=global-statement
+        global _host_node
+
+        if _host_node is None:
+            _host_node = ast.parse("x:1")
+
+        _host_node.body[0].annotation = node
+
+        r = compile(_host_node, "<annotations>", "exec", 1048576, dont_inherit = True)
+
+        # Using exec here, to compile the ast node tree back to string,
+        # there is no accessible "ast.unparse", and this works as a hack
+        # to convert our node to a string annotation, pylint: disable=exec-used
+        m = {}
+        exec(r, m)  # @UndefinedVariable
+
+        return makeConstantRefNode(
+            constant   = m["__annotations__"]['x'],
+            source_ref = source_ref
+        )
+
+    return buildNode(provider, node, source_ref)
+
+
 def makeModuleFrame(module, statements, source_ref):
     assert module.isCompiledPythonModule()
 
@@ -377,6 +439,9 @@ def buildFrameNode(provider, nodes, code_object, source_ref):
     if not statements:
         return None
 
+    if provider.isExpressionOutlineFunction():
+        provider = provider.getParentVariableProvider()
+
     if provider.isExpressionFunctionBody() or \
        provider.isExpressionClassBody():
         result = StatementsFrameFunction(
@@ -665,14 +730,13 @@ def getStatementsPrepended(statement_sequence, statements):
 
 
 def makeReraiseExceptionStatement(source_ref):
+    # TODO: Remove the statement sequence packaging and have users do it themselves
+    # in factory functions instead.
+
     return StatementsSequence(
         statements = (
-            StatementRaiseException(
-                exception_type  = None,
-                exception_value = None,
-                exception_trace = None,
-                exception_cause = None,
-                source_ref      = source_ref
+            StatementReraiseException(
+                source_ref = source_ref
             ),
         ),
         source_ref = source_ref
@@ -707,34 +771,6 @@ def mangleName(variable_name, owner):
             )
 
 
-def makeConditionalStatement(condition, yes_branch, no_branch, source_ref):
-    """ Create conditional statement, with yes_branch not being empty.
-
-        May have to invert condition to achieve that.
-    """
-
-    if yes_branch is None:
-        condition = ExpressionOperationNOT(
-            operand    = condition,
-            source_ref = condition.getSourceReference()
-        )
-
-        yes_branch, no_branch = no_branch, yes_branch
-
-    if not yes_branch.isStatementsSequence():
-        yes_branch = makeStatementsSequenceFromStatement(yes_branch)
-
-    if no_branch is not None and not no_branch.isStatementsSequence():
-        no_branch = makeStatementsSequenceFromStatement(no_branch)
-
-    return StatementConditional(
-        condition  = condition,
-        yes_branch = yes_branch,
-        no_branch  = no_branch,
-        source_ref = source_ref
-    )
-
-
 def makeCallNode(called, *args, **kwargs):
     source_ref = args[-1]
 
