@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,11 +29,14 @@ from nuitka.nodes.AssignNodes import (
     StatementDelVariable
 )
 from nuitka.nodes.FunctionNodes import MaybeLocalVariableUsage
-from nuitka.nodes.NodeMakingHelpers import makeConstantReplacementNode
-from nuitka.nodes.VariableRefNodes import (
+from nuitka.nodes.LocalsDictNodes import (
     ExpressionLocalsVariableRef,
-    ExpressionVariableRef
+    ExpressionLocalsVariableRefORFallback,
+    StatementLocalsDictOperationDel,
+    StatementLocalsDictOperationSet
 )
+from nuitka.nodes.NodeMakingHelpers import makeConstantReplacementNode
+from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
 from nuitka.PythonVersions import (
     getErrorMessageExecWithNestedFunction,
     python_version
@@ -90,31 +93,37 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
     @staticmethod
     def _handleQualnameSetup(node):
         if node.qualname_setup is not None:
+            provider = node.getParentVariableProvider()
+
             if node.isExpressionClassBody():
                 class_variable_name, qualname_assign = node.qualname_setup
 
-                class_variable = node.getParentVariableProvider().getVariableForAssignment(
-                    class_variable_name
-                )
+                if provider.hasProvidedVariable(class_variable_name):
+                    class_variable = provider.getVariableForReference(
+                        class_variable_name
+                    )
 
-                if class_variable.isModuleVariable():
-                    qualname_node = qualname_assign.getAssignSource()
+                    if class_variable.isModuleVariable():
+                        qualname_node = qualname_assign.subnode_value
 
-                    qualname_node.replaceWith(
-                        makeConstantReplacementNode(
+                        new_node = makeConstantReplacementNode(
                             constant = class_variable.getName(),
                             node     = qualname_node
                         )
+
+                        parent = qualname_node.parent
+                        qualname_node.finalize()
+                        parent.replaceChild(qualname_node, new_node)
+
+                        node.qualname_provider = node.getParentModule()
+            else:
+                if provider.hasProvidedVariable(node.qualname_setup):
+                    function_variable = provider.getVariableForReference(
+                        node.qualname_setup
                     )
 
-                    node.qualname_provider = node.getParentModule()
-            else:
-                function_variable = node.getParentVariableProvider().getVariableForAssignment(
-                    node.qualname_setup
-                )
-
-                if function_variable.isModuleVariable():
-                    node.qualname_provider = node.getParentModule()
+                    if function_variable.isModuleVariable():
+                        node.qualname_provider = node.getParentModule()
 
             # TODO: Actually for nested global classes, this approach
             # may not work, as their "qualname" will be wrong. In that
@@ -123,39 +132,75 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
             node.qualname_setup = None
 
+    @staticmethod
+    def _shouldUseLocalsDict(provider, variable_name):
+        return provider.isExpressionClassBody() and \
+               (not provider.hasProvidedVariable(variable_name) or \
+               provider.getProvidedVariable(variable_name).getOwner() is provider)
+
     def onLeaveNode(self, node):
         if node.isStatementAssignmentVariableName():
-            provider = node.getParentVariableProvider()
+            variable_name = node.getVariableName()
+            provider = node.provider
 
-            variable = provider.getVariableForAssignment(
-                variable_name = node.getVariableName()
-            )
+            # Classes always assign to locals dictionary except for closure
+            # variables taken.
+            if self._shouldUseLocalsDict(provider, variable_name):
+                new_node = StatementLocalsDictOperationSet(
+                    locals_scope  = provider.getFunctionLocalsScope(),
+                    variable_name = variable_name,
+                    value         = node.subnode_source,
+                    source_ref    = node.source_ref
+                )
+            else:
+                variable = provider.getVariableForAssignment(
+                    variable_name = variable_name
+                )
 
-            node.replaceWith(
-                StatementAssignmentVariable(
+                new_node = StatementAssignmentVariable(
                     variable   = variable,
                     source     = node.subnode_source,
                     source_ref = node.source_ref
                 )
-            )
 
-            variable.addVariableUser(provider)
+                variable.addVariableUser(provider)
+
+            node.parent.replaceChild(node, new_node)
+
+            del node.parent
+            del node.provider
         elif node.isStatementDelVariableName():
-            provider = node.getParentVariableProvider()
+            variable_name = node.getVariableName()
 
-            variable = provider.getVariableForAssignment(
-                variable_name = node.getVariableName()
-            )
+            provider = node.provider
 
-            node.replaceWith(
-                StatementDelVariable(
+            if self._shouldUseLocalsDict(provider, variable_name):
+                # Classes always assign to locals dictionary except for closure
+                # variables taken.
+
+                # TODO: This is losing the "tolerant" annotation.
+                new_node = StatementLocalsDictOperationDel(
+                    locals_scope  = provider.getFunctionLocalsScope(),
+                    variable_name = variable_name,
+                    source_ref    = node.source_ref
+                )
+            else:
+                variable = provider.getVariableForAssignment(
+                    variable_name = variable_name
+                )
+
+                new_node = StatementDelVariable(
                     variable   = variable,
                     tolerant   = node.tolerant,
                     source_ref = node.source_ref
                 )
-            )
 
-            variable.addVariableUser(provider)
+                variable.addVariableUser(provider)
+
+            parent = node.parent
+            node.finalize()
+
+            parent.replaceChild(node, new_node)
 
     def onEnterNode(self, node):
         # Mighty complex code with lots of branches and statements, but it
@@ -163,57 +208,36 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
         # pylint: disable=too-many-branches,too-many-statements
 
         if node.isExpressionVariableNameRef():
-            provider = node.getParentVariableProvider()
+            provider = node.provider
 
-            if provider.isEarlyClosure():
-                variable = provider.getVariableForReference(
-                    variable_name = node.getVariableName()
-                )
-
-                # Python3.4 version respects closure variables taken can be
-                # overridden by writes to locals. It should be done for
-                # globals too, on all versions, but for Python2 the locals
-                # dictionary is avoided unless "exec" appears, so it's not
-                # done.
-                owner = variable.getOwner()
-                user = provider
-
-                while user is not owner:
-                    if user.isExpressionFunctionBody() or \
-                       user.isExpressionClassBody():
-                        break
-
-                    user = user.getParentVariableProvider()
-
-                if owner is not user:
-                    if python_version >= 340 or \
-                       (python_version >= 300 and \
-                        variable.isModuleVariable()):
-
-                        node.replaceWith(
-                            ExpressionLocalsVariableRef(
-                                variable_name     = node.getVariableName(),
-                                fallback_variable = variable,
-                                source_ref        = node.getSourceReference()
-                            )
-                        )
-
-                    else:
-                        node.replaceWith(
-                            ExpressionVariableRef(
-                                variable   = variable,
-                                source_ref = node.source_ref
-                            )
-                        )
-                else:
-                    node.replaceWith(
-                        ExpressionVariableRef(
-                            variable   = variable,
-                            source_ref = node.source_ref
-                        )
+            if provider.isExpressionClassBody():
+                if node.needsFallback():
+                    variable = provider.getVariableForReference(
+                        variable_name = node.getVariableName()
                     )
 
-                variable.addVariableUser(provider)
+                    new_node = ExpressionLocalsVariableRefORFallback(
+                        locals_scope  = provider.getFunctionLocalsScope(),
+                        variable_name = node.getVariableName(),
+                        fallback      = ExpressionVariableRef(
+                            variable   = variable,
+                            source_ref = node.source_ref
+                        ),
+                        source_ref    = node.source_ref
+                    )
+
+                    variable.addVariableUser(provider)
+                else:
+                    new_node = ExpressionLocalsVariableRef(
+                        locals_scope  = provider.getFunctionLocalsScope(),
+                        variable_name = node.getVariableName(),
+                        source_ref    = node.source_ref
+                    )
+
+                parent = node.parent
+                node.finalize()
+
+                parent.replaceChild(node, new_node)
         elif node.isExpressionTempVariableRef():
             if node.getVariable().getOwner() != node.getParentVariableProvider():
                 node.getParentVariableProvider().addClosureVariable(
@@ -318,84 +342,61 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
 
         variable_name = node.getVariableName()
 
-        was_taken = provider.hasTakenVariable(variable_name)
-
         variable = provider.getVariableForReference(
             variable_name = variable_name
         )
 
         # Need to catch functions with "exec" and closure variables not allowed.
-        if python_version < 300 and \
-           not was_taken and \
-           provider.isExpressionFunctionBodyBase() and \
-           variable.getOwner() is not provider:
-            parent_provider = provider.getParentVariableProvider()
+        if python_version < 300 and provider.isExpressionFunctionBodyBase():
+            was_taken = provider.hasTakenVariable(variable_name)
 
-            while parent_provider.isExpressionClassBody():
-                parent_provider = parent_provider.getParentVariableProvider()
+            if not was_taken and \
+               variable.getOwner() is not provider:
+                parent_provider = provider.getParentVariableProvider()
 
-            if parent_provider.isExpressionFunctionBody() and \
-               parent_provider.isUnqualifiedExec():
-                raiseSyntaxError(
-                    getErrorMessageExecWithNestedFunction() % parent_provider.getName(),
-                    node.getSourceReference(),
-                    display_line = False # Wrong line anyway
-                )
+                while parent_provider.isExpressionClassBody():
+                    parent_provider = parent_provider.getParentVariableProvider()
+
+                if parent_provider.isExpressionFunctionBody() and \
+                   parent_provider.isUnqualifiedExec():
+                    raiseSyntaxError(
+                        getErrorMessageExecWithNestedFunction() % parent_provider.getName(),
+                        node.getSourceReference(),
+                        display_line = False # Wrong line anyway
+                    )
 
         return variable
 
     def onEnterNode(self, node):
         if node.isExpressionVariableNameRef():
-            provider = node.getParentVariableProvider()
+            provider = node.provider
 
             try:
                 variable = self._attachVariable(node, provider)
             except MaybeLocalVariableUsage:
                 variable_name = node.getVariableName()
 
-                node.replaceWith(
-                    ExpressionLocalsVariableRef(
-                        variable_name     = variable_name,
-                        fallback_variable = node.getParentModule().getVariableForReference(variable_name),
-                        source_ref        = node.getSourceReference()
-                    )
-                )
-            else:
-                node.replaceWith(
-                    ExpressionVariableRef(
-                        variable   = variable,
+                new_node = ExpressionLocalsVariableRefORFallback(
+                    locals_scope  = provider.getFunctionLocalsScope(),
+                    variable_name = variable_name,
+                    fallback      = ExpressionVariableRef(
+                        variable   = node.getParentModule().getVariableForReference(variable_name),
                         source_ref = node.source_ref
-                    )
+                    ),
+                    source_ref    = node.source_ref
+                )
+            else:
+                new_node = ExpressionVariableRef(
+                    variable   = variable,
+                    source_ref = node.source_ref
                 )
 
                 variable.addVariableUser(provider)
 
-        elif node.isExpressionVariableRef():
-            provider = node.getParentVariableProvider()
+            parent = node.parent
+            node.finalize()
 
-            variable = node.getVariable()
-
-            if variable is None:
-                try:
-                    variable = self._attachVariable(node, provider)
-                except MaybeLocalVariableUsage:
-                    variable_name = node.getVariableName()
-
-                    node.replaceWith(
-                        ExpressionLocalsVariableRef(
-                            variable_name     = variable_name,
-                            fallback_variable = node.getParentModule().getVariableForReference(variable_name),
-                            source_ref        = node.getSourceReference()
-                        )
-                    )
-                else:
-                    node.setVariable(
-                        variable
-                    )
-
-                    variable.addVariableUser(provider)
-            else:
-                variable.addVariableUser(provider)
+            parent.replaceChild(node, new_node)
 
 
 class VariableClosureLookupVisitorPhase3(VisitorNoopMixin):
