@@ -32,7 +32,10 @@ if __name__ == "__main__":
     # The sys.argv[0] might contain ".exe", ".py" or no suffix at all.
     # Remove it, so the "diff" output is more acceptable.
     args = sys.argv[:]
-    args[0] = os.path.basename(args[0]).replace(".exe", ".py").replace(".py", "")
+    args[0] = os.path.basename(args[0]).\
+      replace(".exe", ".py").\
+      replace(".bin", ".py").\
+      replace(".py", "")
 
     print("Arguments were (stripped argv[0] suffix):", repr(args))
 
